@@ -8,8 +8,10 @@ of the Functional Connectivity Megatrawl project.
 See :func:`nilearn.datasets.fetch_megatrawls_netmats` documentation for more details.
 """
 ################################################################################
+# Fetching the Megatrawls Network matrices
+# ----------------------------------------
 # Fetching the partial correlation matrices of dimensionality d=300 with
-# timeseries method 'eigen regression' by importing datasets module
+# timeseries method 'eigen regression'
 from nilearn import datasets
 
 netmats = datasets.fetch_megatrawls_netmats(dimensionality=300,
@@ -21,14 +23,12 @@ partial_correlation = netmats.correlation_matrices
 
 ################################################################################
 # Visualization
-# Importing matplotlib and nilearn plotting modules to use its utilities for
-# plotting correlation matrices
-import matplotlib.pyplot as plt
+# -------------
+# Import nilearn plotting modules to use its utilities for plotting
+# correlation matrices
 from nilearn import plotting
 
-title = "Partial correlation matrices of d=300 with timeseries='eigen_regression'"
-plt.figure()
-plt.imshow(partial_correlation, interpolation="nearest", cmap=plotting.cm.bwr)
-plt.colorbar()
-plt.title(title)
-plt.show()
+title = "Partial correlation matrices\n for d=300"
+display = plotting.plot_matrix(partial_correlation, colorbar=True,
+                               title=title)
+plotting.show()
