@@ -33,44 +33,46 @@ signal                  --- Set of preprocessing functions for time series
 """
 
 import gzip
-import sys
+import os
+import pkg_resources
 import warnings
 
 from distutils.version import LooseVersion
 
 from .version import _check_module_dependencies, __version__
 
-
-def _py2_deprecation_warning():
-    py2_warning = ('Python2 support is deprecated and will be removed in '
-                   'the next release. Consider switching to Python 3.6 or 3.7.'
-                   )
-    warnings.filterwarnings('once', message=py2_warning)
-    warnings.warn(message=py2_warning,
-                  category=DeprecationWarning,
-                  stacklevel=3,
-                  )
-
-def _py34_deprecation_warning():
-    py34_warning = ('Python 3.4 support is deprecated and will be removed in '
-                   'the next release. Consider switching to Python 3.6 or 3.7.'
-                   )
-    warnings.filterwarnings('once', message=py34_warning)
-    warnings.warn(message=py34_warning,
-                  category=DeprecationWarning,
-                  stacklevel=3,
-                  )
+# Workaround issue discovered in intel-openmp 2019.5:
+# https://github.com/ContinuumIO/anaconda-issues/issues/11294
+#
+# see also https://github.com/scikit-learn/scikit-learn/pull/15020
+os.environ.setdefault("KMP_INIT_AT_FORK", "FALSE")
 
 
-def _python_deprecation_warnings():
-    if sys.version_info.major == 2:
-        _py2_deprecation_warning()
-    elif sys.version_info.major == 3 and sys.version_info.minor == 4:
-        _py34_deprecation_warning()
+def _nibabel2_deprecation_warning():
+    msg = ('Support for Nibabel 2.x is deprecated and will stop '
+           'in release 0.9.0. Please consider upgrading to '
+           'Nibabel 3.x.')
+    warnings.filterwarnings('once', message=msg)
+    warnings.warn(message=msg,
+                  category=FutureWarning,
+                  stacklevel=3)
+
+
+def _nibabel_deprecation_warnings():
+    """Give a deprecation warning is the version of
+    Nibabel is < 3.0.0.
+    """
+    # Nibabel should be installed or we would
+    # have had an error when calling
+    # _check_module_dependencies
+    dist = pkg_resources.get_distribution('nibabel')
+    nib_version = LooseVersion(dist.version)
+    if nib_version < '3.0':
+        _nibabel2_deprecation_warning()
 
 
 _check_module_dependencies()
-_python_deprecation_warnings()
+_nibabel_deprecation_warnings()
 
 # Temporary work around to address formatting issues in doc tests
 # with NumPy 1.14. NumPy had made more consistent str/repr formatting
@@ -79,8 +81,8 @@ import numpy as np
 if LooseVersion(np.__version__) >= LooseVersion("1.14"):
     # See issue #1600 in nilearn for reason to add try and except
     try:
-        from ._utils.testing import is_nose_running
-        if is_nose_running():
+        from ._utils.testing import are_tests_running
+        if are_tests_running():
             np.set_printoptions(legacy='1.13')
     except ImportError:
         pass
@@ -105,5 +107,4 @@ CHECK_CACHE_VERSION = True
 # list all submodules available in nilearn and version
 __all__ = ['datasets', 'decoding', 'decomposition', 'connectome',
            'image', 'input_data', 'masking', 'mass_univariate', 'plotting',
-           'region', 'signal', 'surface', 'parcellations', '__version__']
-
+           'regions', 'signal', 'surface', '__version__']
