@@ -13,19 +13,12 @@ from distutils.version import LooseVersion
 import nibabel
 import sklearn
 
-from sklearn.externals.joblib import Memory
+from joblib import Memory
 
 MEMORY_CLASSES = (Memory, )
 
-try:
-    from joblib import Memory as JoblibMemory
-    MEMORY_CLASSES = (Memory, JoblibMemory)
-except ImportError:
-    pass
-
 import nilearn
 
-from .compat import _basestring
 
 __CACHE_CHECKED = dict()
 
@@ -35,20 +28,21 @@ def _check_memory(memory, verbose=0):
 
     Parameters
     ----------
-    memory: None or instance of joblib.Memory or str
+    memory : None or instance of joblib.Memory or str
         Used to cache the masking process.
         If a str is given, it is the path to the caching directory.
 
-    verbose : int, optional (default 0)
-        Verbosity level.
+    verbose : int, optional
+        Verbosity level. Default=0.
 
     Returns
     -------
-    instance of joblib.Memory.
+    memory : instance of joblib.Memory.
+
     """
     if memory is None:
-        memory = Memory(cachedir=None, verbose=verbose)
-    if isinstance(memory, _basestring):
+        memory = Memory(location=None, verbose=verbose)
+    if isinstance(memory, str):
         cache_dir = memory
         if nilearn.EXPAND_PATH_WILDCARDS:
             cache_dir = os.path.expanduser(cache_dir)
@@ -81,13 +75,14 @@ def _check_memory(memory, verbose=0):
                              .format(split_cache_dir[0]))
             raise ValueError(error_msg)
 
-        memory = Memory(cachedir=cache_dir, verbose=verbose)
+        memory = Memory(location=cache_dir, verbose=verbose)
     return memory
 
 
 def _safe_cache(memory, func, **kwargs):
-    """ A wrapper for mem.cache that flushes the cache if the version
-        number of nibabel has changed.
+    """A wrapper for mem.cache that flushes the cache if the version
+    number of nibabel has changed.
+
     """
     ''' Workaround for
      https://github.com/scikit-learn-contrib/imbalanced-learn/issues/482
@@ -96,16 +91,16 @@ def _safe_cache(memory, func, **kwargs):
     if that fails, ensuring th warning is not generated in any case.
     '''
     try:
-        cachedir = os.path.join(memory.location, 'joblib')
+        location = os.path.join(memory.location, 'joblib')
     except AttributeError:
-        cachedir = memory.cachedir
+        location = memory.location
     except TypeError:
-        cachedir = None
+        location = None
 
-    if cachedir is None or cachedir in __CACHE_CHECKED:
+    if location is None or location in __CACHE_CHECKED:
         return memory.cache(func, **kwargs)
 
-    version_file = os.path.join(cachedir, 'module_versions.json')
+    version_file = os.path.join(location, 'module_versions.json')
 
     versions = dict()
     if os.path.exists(version_file):
@@ -126,34 +121,34 @@ def _safe_cache(memory, func, **kwargs):
                           "different version of nibabel. Deleting "
                           "the cache. Put nilearn.CHECK_CACHE_VERSION "
                           "to false to avoid this behavior."
-                          % cachedir)
+                          % location)
             try:
-                tmp_dir = (os.path.split(cachedir)[:-1]
+                tmp_dir = (os.path.split(location)[:-1]
                            + ('old_%i' % os.getpid(), ))
                 tmp_dir = os.path.join(*tmp_dir)
                 # We use rename + unlink to be more robust to race
                 # conditions
-                os.rename(cachedir, tmp_dir)
+                os.rename(location, tmp_dir)
                 shutil.rmtree(tmp_dir)
             except OSError:
                 # Another process could have removed this dir
                 pass
 
             try:
-                os.makedirs(cachedir)
+                os.makedirs(location)
             except OSError:
                 # File exists?
                 pass
         else:
             warnings.warn("Incompatible cache in %s: "
-                          "old version of nibabel." % cachedir)
+                          "old version of nibabel." % location)
 
     # Write json files if configuration is different
     if versions != my_versions:
         with open(version_file, 'w') as _version_file:
             json.dump(my_versions, _version_file)
 
-    __CACHE_CHECKED[cachedir] = True
+    __CACHE_CHECKED[location] = True
 
     return memory.cache(func, **kwargs)
 
@@ -170,7 +165,7 @@ class _ShelvedFunc(object):
 
 def cache(func, memory, func_memory_level=None, memory_level=None,
           shelve=False, **kwargs):
-    """ Return a joblib.Memory object.
+    """Return a joblib.Memory object.
 
     The memory_level determines the level above which the wrapped
     function output is cached. By specifying a numeric value for
@@ -180,35 +175,37 @@ def cache(func, memory, func_memory_level=None, memory_level=None,
 
     Parameters
     ----------
-    func: function
+    func : function
         The function which output is to be cached.
 
-    memory: instance of joblib.Memory or string
+    memory : instance of joblib.Memory or string
         Used to cache the function call.
 
-    func_memory_level: int, optional
+    func_memory_level : int, optional
         The memory_level from which caching must be enabled for the wrapped
         function.
 
-    memory_level: int, optional
+    memory_level : int, optional
         The memory_level used to determine if function call must
         be cached or not (if user_memory_level is equal of greater than
-        func_memory_level the function is cached)
+        func_memory_level the function is cached).
 
-    shelve: bool
+    shelve : bool, optional
         Whether to return a joblib MemorizedResult, callable by a .get()
-        method, instead of the return value of func
+        method, instead of the return value of func.
+        Default=False.
 
-    kwargs: keyword arguments
-        The keyword arguments passed to memory.cache
+    kwargs : keyword arguments, optional
+        The keyword arguments passed to memory.cache.
 
     Returns
     -------
-    mem: joblib.MemorizedFunc, wrapped in _ShelvedFunc if shelving
+    mem : joblib.MemorizedFunc, wrapped in _ShelvedFunc if shelving
         Object that wraps the function func to cache its further call.
         This object may be a no-op, if the requested level is lower
         than the value given to _cache()).
         For consistency, a callable object is always returned.
+
     """
     verbose = kwargs.get('verbose', 0)
 
@@ -223,13 +220,13 @@ def cache(func, memory, func_memory_level=None, memory_level=None,
 
     if memory is not None and (func_memory_level is None or
                                memory_level >= func_memory_level):
-        if isinstance(memory, _basestring):
-            memory = Memory(cachedir=memory, verbose=verbose)
+        if isinstance(memory, str):
+            memory = Memory(location=memory, verbose=verbose)
         if not isinstance(memory, MEMORY_CLASSES):
             raise TypeError("'memory' argument must be a string or a "
                             "joblib.Memory object. "
                             "%s %s was given." % (memory, type(memory)))
-        if (memory.cachedir is None and memory_level is not None
+        if (memory.location is None and memory_level is not None
                 and memory_level > 1):
             warnings.warn("Caching has been enabled (memory_level = %d) "
                           "but no Memory object or path has been provided"
@@ -238,7 +235,7 @@ def cache(func, memory, func_memory_level=None, memory_level=None,
                           (memory_level, func.__name__),
                           stacklevel=2)
     else:
-        memory = Memory(cachedir=None, verbose=verbose)
+        memory = Memory(location=None, verbose=verbose)
     cached_func = _safe_cache(memory, func, **kwargs)
     if shelve:
         cached_func = _ShelvedFunc(cached_func)
@@ -251,10 +248,13 @@ class CacheMixin(object):
     This class is a thin layer on top of joblib.Memory, that mainly adds a
     "caching level", similar to a "log level".
 
+    Notes
+    -----
     Usage: to cache the results of a method, wrap it in self._cache()
     defined by this class. Caching is performed only if the user-specified
     cache level (self._memory_level) is greater than the value given as a
     parameter to self._cache(). See _cache() documentation for details.
+
     """
     def _cache(self, func, func_memory_level=1, shelve=False, **kwargs):
         """Return a joblib.Memory object.
@@ -267,24 +267,25 @@ class CacheMixin(object):
 
         Parameters
         ----------
-        func: function
+        func : function
             The function the output of which is to be cached.
 
-        memory_level: int
+        func_memory_level : int, optional
             The memory_level from which caching must be enabled for the wrapped
-            function.
+            function. Default=1.
 
-        shelve: bool
+        shelve : bool, optional
             Whether to return a joblib MemorizedResult, callable by a .get()
-            method, instead of the return value of func
+            method, instead of the return value of func. Default=False.
 
         Returns
         -------
-        mem: joblib.MemorizedFunc, wrapped in _ShelvedFunc if shelving
+        mem : joblib.MemorizedFunc, wrapped in _ShelvedFunc if shelving
             Object that wraps the function func to cache its further call.
             This object may be a no-op, if the requested level is lower
             than the value given to _cache()).
             For consistency, a callable object is always returned.
+
         """
         verbose = getattr(self, 'verbose', 0)
 
@@ -293,12 +294,12 @@ class CacheMixin(object):
         if not hasattr(self, "memory_level"):
             self.memory_level = 0
         if not hasattr(self, "memory"):
-            self.memory = Memory(cachedir=None, verbose=verbose)
+            self.memory = Memory(location=None, verbose=verbose)
         self.memory = _check_memory(self.memory, verbose=verbose)
 
         # If cache level is 0 but a memory object has been provided, set
         # memory_level to 1 with a warning.
-        if self.memory_level == 0 and self.memory.cachedir is not None:
+        if self.memory_level == 0 and self.memory.location is not None:
             warnings.warn("memory_level is currently set to 0 but "
                           "a Memory object has been provided. "
                           "Setting memory_level to 1.")
