@@ -6,23 +6,22 @@ graphical models.
 # License: simplified BSD
 
 import warnings
-import collections
+import collections.abc
 import operator
 import itertools
 
 import numpy as np
 import scipy.linalg
 
+from joblib import Memory, delayed, Parallel
 from sklearn.base import BaseEstimator
 from sklearn.covariance import empirical_covariance
-from sklearn.externals.joblib import Memory, delayed, Parallel
 from sklearn.model_selection import check_cv
 from sklearn.utils.extmath import fast_logdet
 
 from .._utils import CacheMixin
 from .._utils import logger
 from .._utils.extmath import is_spd
-from .._utils.compat import izip
 
 
 def compute_alpha_max(emp_covs, n_samples):
@@ -34,6 +33,9 @@ def compute_alpha_max(emp_covs, n_samples):
     This function also returns the value below which the precision
     matrices are fully dense (i.e. minimal number of zero coefficients).
 
+    The formula used in this function was derived using the same method
+    as in :footcite:`Duchi2012`.
+
     Parameters
     ----------
     emp_covs : array-like, shape (n_features, n_features, n_subjects)
@@ -53,13 +55,10 @@ def compute_alpha_max(emp_covs, n_samples):
         minimal value for the regularization parameter that gives a fully
         dense matrix.
 
-    See also
-    --------
-    The formula used in this function was derived using the same method as in:
+    References
+    ---------
+    .. footbibliography::
 
-    Duchi, John, Stephen Gould, and Daphne Koller. 'Projected Subgradient
-    Methods for Learning Sparse Gaussians'. ArXiv E-prints 1206 (1 June
-    2012): 3249.
     """
     A = np.copy(emp_covs)
     n_samples = np.asarray(n_samples).copy()
@@ -86,8 +85,8 @@ def _update_submatrix(full, sub, sub_inv, p, h, v):
     and column.
 
     This computation is based on the Sherman-Woodbury-Morrison identity.
-    """
 
+    """
     n = p - 1
     v[:n + 1] = full[:n + 1, n]
     v[n + 1:] = full[n + 2:, n]
@@ -120,6 +119,7 @@ def _update_submatrix(full, sub, sub_inv, p, h, v):
 def _assert_submatrix(full, sub, n):
     """Check that "sub" is the matrix obtained by removing the p-th col and row
     in "full". Used only for debugging.
+
     """
     true_sub = np.empty_like(sub)
     true_sub[:n, :n] = full[:n, :n]
@@ -143,6 +143,8 @@ def group_sparse_covariance(subjects, alpha, max_iter=50, tol=1e-3, verbose=0,
     Running time is linear on max_iter, and number of subjects (len(subjects)),
     but cubic on number of features (subjects[0].shape[1]).
 
+    The present algorithm is based on :footcite:`Honorio2015`.
+
     Parameters
     ----------
     subjects : list of numpy.ndarray
@@ -157,16 +159,17 @@ def group_sparse_covariance(subjects, alpha, max_iter=50, tol=1e-3, verbose=0,
         no regularization: output is not sparse)
 
     max_iter : int, optional
-        maximum number of iterations.
+        maximum number of iterations. Default=50.
 
     tol : positive float or None, optional
         The tolerance to declare convergence: if the duality gap goes below
         this value, optimization is stopped. If None, no check is performed.
+        Default=1e-3.
 
     verbose : int, optional
-        verbosity level. Zero means "no message".
+        verbosity level. Zero means "no message". Default=0.
 
-    probe_function : callable or None
+    probe_function : callable or None, optional
         This value is called before the first iteration and after each
         iteration. If it returns True, then optimization is stopped
         prematurely.
@@ -181,13 +184,14 @@ def group_sparse_covariance(subjects, alpha, max_iter=50, tol=1e-3, verbose=0,
         - current value of precisions (ndarray).
         - previous value of precisions (ndarray). None before first iteration.
 
-    precisions_init: numpy.ndarray
+    precisions_init : numpy.ndarray, optional
         initial value of the precision matrices. If not provided, a diagonal
         matrix with the variances of each input signal is used.
 
     debug : bool, optional
         if True, perform checks during computation. It can help find
         numerical problems, but increases computation time a lot.
+        Default=False.
 
     Returns
     -------
@@ -197,13 +201,10 @@ def group_sparse_covariance(subjects, alpha, max_iter=50, tol=1e-3, verbose=0,
     precisions : numpy.ndarray, shape (n_features, n_features, n_subjects)
         estimated precision matrices
 
-    Notes
-    -----
-    The present algorithm is based on:
+    References
+    ----------
+    .. footbibliography::
 
-    Jean Honorio and Dimitris Samaras.
-    "Simultaneous and Group-Sparse Multi-Task Learning of Gaussian Graphical
-    Models". arXiv:1207.4255 (17 July 2012). http://arxiv.org/abs/1207.4255.
     """
 
     emp_covs, n_samples = empirical_covariances(
@@ -222,6 +223,7 @@ def _group_sparse_covariance(emp_covs, n_samples, alpha, max_iter=10, tol=1e-3,
                              verbose=0, debug=False):
     """Internal version of group_sparse_covariance.
     See its docstring for details.
+
     """
     if tol == -1:
         tol = None
@@ -246,7 +248,7 @@ def _group_sparse_covariance(emp_covs, n_samples, alpha, max_iter=10, tol=1e-3,
 
     if precisions_init is None:
         # Fortran order make omega[..., k] contiguous, which is often useful.
-        omega = np.ndarray(shape=emp_covs.shape, dtype=np.float,
+        omega = np.ndarray(shape=emp_covs.shape, dtype=np.float64,
                            order="F")
         for k in range(n_subjects):
             # Values on main diagonals are far from zero, because they
@@ -256,21 +258,21 @@ def _group_sparse_covariance(emp_covs, n_samples, alpha, max_iter=10, tol=1e-3,
         omega = precisions_init.copy()
 
     # Preallocate arrays
-    y = np.ndarray(shape=(n_subjects, n_features - 1), dtype=np.float)
-    u = np.ndarray(shape=(n_subjects, n_features - 1), dtype=np.float)
-    y_1 = np.ndarray(shape=(n_subjects, n_features - 2), dtype=np.float)
-    h_12 = np.ndarray(shape=(n_subjects, n_features - 2), dtype=np.float)
-    q = np.ndarray(shape=(n_subjects,), dtype=np.float)
-    aq = np.ndarray(shape=(n_subjects,), dtype=np.float)  # temp. array
-    c = np.ndarray(shape=(n_subjects,), dtype=np.float)
+    y = np.ndarray(shape=(n_subjects, n_features - 1), dtype=np.float64)
+    u = np.ndarray(shape=(n_subjects, n_features - 1), dtype=np.float64)
+    y_1 = np.ndarray(shape=(n_subjects, n_features - 2), dtype=np.float64)
+    h_12 = np.ndarray(shape=(n_subjects, n_features - 2), dtype=np.float64)
+    q = np.ndarray(shape=(n_subjects,), dtype=np.float64)
+    aq = np.ndarray(shape=(n_subjects,), dtype=np.float64)  # temp. array
+    c = np.ndarray(shape=(n_subjects,), dtype=np.float64)
     W = np.ndarray(shape=(omega.shape[0] - 1, omega.shape[1] - 1,
                           omega.shape[2]),
-                   dtype=np.float, order="F")
-    W_inv = np.ndarray(shape=W.shape, dtype=np.float, order="F")
+                   dtype=np.float64, order="F")
+    W_inv = np.ndarray(shape=W.shape, dtype=np.float64, order="F")
 
-    # Auxilliary arrays.
-    v = np.ndarray((omega.shape[0] - 1,), dtype=np.float)
-    h = np.ndarray((omega.shape[1] - 1,), dtype=np.float)
+    # Auxiliary arrays.
+    v = np.ndarray((omega.shape[0] - 1,), dtype=np.float64)
+    h = np.ndarray((omega.shape[1] - 1,), dtype=np.float64)
 
     # Optional.
     tolerance_reached = False
@@ -306,7 +308,7 @@ def _group_sparse_covariance(emp_covs, n_samples, alpha, max_iter=10, tol=1e-3,
             if p == 0:
                 # Initial state: remove first col/row
                 W = omega[1:, 1:, :].copy()   # stack of W(k)
-                W_inv = np.ndarray(shape=W.shape, dtype=np.float)
+                W_inv = np.ndarray(shape=W.shape, dtype=np.float64)
                 for k in range(W.shape[2]):
                     # stack of W^-1(k)
                     W_inv[..., k] = scipy.linalg.inv(W[..., k])
@@ -450,23 +452,27 @@ def _group_sparse_covariance(emp_covs, n_samples, alpha, max_iter=10, tol=1e-3,
 class GroupSparseCovariance(BaseEstimator, CacheMixin):
     """Covariance and precision matrix estimator.
 
+    The model used has been introduced in :footcite:`Varoquaux2010a`, and the
+    algorithm used is based on what is described in :footcite:`Honorio2015`.
+
     Parameters
     ----------
-    alpha : float
+    alpha : float, optional
         regularization parameter. With normalized covariances matrices and
         number of samples, sensible values lie in the [0, 1] range(zero is
-        no regularization: output is not sparse)
+        no regularization: output is not sparse).
+        Default=0.1.
 
     tol : positive float, optional
         The tolerance to declare convergence: if the dual gap goes below
-        this value, iterations are stopped
+        this value, iterations are stopped. Default=1e-3.
 
     max_iter : int, optional
-        maximum number of iterations. The default value (10) is rather
-        conservative.
+        maximum number of iterations. The default value is rather
+        conservative. Default=10.
 
     verbose : int, optional
-        verbosity level. Zero means "no message".
+        verbosity level. Zero means "no message". Default=0.
 
     memory : instance of joblib.Memory or string, optional
         Used to cache the masking process.
@@ -475,6 +481,7 @@ class GroupSparseCovariance(BaseEstimator, CacheMixin):
 
     memory_level : int, optional
         Caching aggressiveness. Higher values mean more caching.
+        Default=0.
 
     Attributes
     ----------
@@ -484,24 +491,14 @@ class GroupSparseCovariance(BaseEstimator, CacheMixin):
     `precisions_` : numpy.ndarraye, shape (n_features, n_features, n_subjects)
         precisions matrices estimated using the group-sparse algorithm.
 
-    Notes
-    ------
+    References
+    ----------
+    .. footbibliography::
 
-    The model used has been introduced in:
-
-    Gael Varoquaux, et al. `Brain Covariance Selection: Better Individual
-    Functional Connectivity Models Using Population Prior
-    <http://arxiv.org/abs/1008.5071>`_'.
-
-    The algorithm used is based on what is described in:
-
-    Jean Honorio and Dimitris Samaras.
-    "Simultaneous and Group-Sparse Multi-Task Learning of Gaussian Graphical
-    Models". http://arxiv.org/abs/1207.4255.
     """
 
     def __init__(self, alpha=0.1, tol=1e-3, max_iter=10, verbose=0,
-                 memory=Memory(cachedir=None), memory_level=0):
+                 memory=Memory(location=None), memory_level=0):
         self.alpha = alpha
         self.tol = tol
         self.max_iter = max_iter
@@ -525,8 +522,8 @@ class GroupSparseCovariance(BaseEstimator, CacheMixin):
         -------
         self : GroupSparseCovariance instance
             the object itself. Useful for chaining operations.
-        """
 
+        """
         logger.log("Computing covariance matrices", verbose=self.verbose)
         self.covariances_, n_samples = empirical_covariances(
             subjects, assume_centered=False)
@@ -554,10 +551,12 @@ def empirical_covariances(subjects, assume_centered=False, standardize=False):
     assume_centered : bool, optional
         if True, assume that all input signals are centered. This slightly
         decreases computation time by avoiding useless computation.
+        Default=False.
 
     standardize : bool, optional
         if True, set every signal variance to one before computing their
         covariance matrix (i.e. compute a correlation matrix).
+        Default=False.
 
     Returns
     -------
@@ -565,7 +564,8 @@ def empirical_covariances(subjects, assume_centered=False, standardize=False):
         empirical covariances.
 
     n_samples : numpy.ndarray, shape: (subject number,)
-        number of samples for each subject. dtype is np.float.
+        number of samples for each subject. dtype is np.float64.
+
     """
     if not hasattr(subjects, "__iter__"):
         raise ValueError("'subjects' input argument must be an iterable. "
@@ -592,7 +592,7 @@ def empirical_covariances(subjects, assume_centered=False, standardize=False):
         emp_covs[..., k] = M + M.T
     emp_covs /= 2
 
-    n_samples = np.asarray([s.shape[0] for s in subjects], dtype=np.float)
+    n_samples = np.asarray([s.shape[0] for s in subjects], dtype=np.float64)
 
     return emp_covs, n_samples
 
@@ -620,11 +620,11 @@ def group_sparse_scores(precisions, n_samples, emp_covs, alpha,
         regularization parameter
 
     duality_gap : bool, optional
-        if True, also returns a duality gap upper bound.
+        if True, also returns a duality gap upper bound. Default=False.
 
     debug : bool, optional
         if True, some consistency checks are performed to help solving
-        numerical problems
+        numerical problems. Default=False.
 
     Returns
     -------
@@ -639,8 +639,8 @@ def group_sparse_scores(precisions, n_samples, emp_covs, alpha,
     duality_gap : float
         duality gap upper bound. The returned bound is tight: it vanishes for
         the optimal precision matrices
-    """
 
+    """
     n_features, _, n_subjects = emp_covs.shape
 
     log_lik = 0
@@ -656,7 +656,7 @@ def group_sparse_scores(precisions, n_samples, emp_covs, alpha,
 
     # Compute duality gap if requested
     if duality_gap is True:
-        A = np.empty(precisions.shape, dtype=np.float, order="F")
+        A = np.empty(precisions.shape, dtype=np.float64, order="F")
         for k in range(n_subjects):
             # TODO: can be computed more efficiently using W_inv. See
             # Friedman, Jerome, Trevor Hastie, and Robert Tibshirani.
@@ -680,7 +680,6 @@ def group_sparse_scores(precisions, n_samples, emp_covs, alpha,
             # duality gap.
             A[..., k].flat[::A.shape[0] + 1] = 0
 
-        alpha_max = np.sqrt((A ** 2).sum(axis=-1)).max()
         dual_obj = 0  # dual objective
         for k in range(n_subjects):
             B = emp_covs[..., k] + A[..., k] / n_samples[k]
@@ -727,18 +726,18 @@ def group_sparse_covariance_path(train_subjs, alphas, test_subjs=None,
     alphas : list of float
          values of alpha to use. Best results for sorted values (decreasing)
 
-    test_subjs : list of numpy.ndarray
+    test_subjs : list of numpy.ndarray, optional
         list of signals, independent from those in train_subjs, on which to
         compute a score. If None, no score is computed.
 
-    verbose : int
-        verbosity level
+    verbose : int, optional
+        verbosity level. Default=0.
 
     tol, max_iter, debug, precisions_init :
         Passed to group_sparse_covariance(). See the corresponding docstring
         for details.
 
-    probe_function : callable
+    probe_function : callable, optional
         This value is called before the first iteration and after each
         iteration. If it returns True, then optimization is stopped
         prematurely.
@@ -762,6 +761,7 @@ def group_sparse_covariance_path(train_subjs, alphas, test_subjs=None,
     scores : list of float
         for each estimated precision, score obtained on the test set. Output
         only if test_subjs is not None.
+
     """
     train_covs, train_n_samples = empirical_covariances(
         train_subjs, assume_centered=False, standardize=True)
@@ -795,6 +795,7 @@ class EarlyStopProbe(object):
     Stop optimizing as soon as the score on the test set starts decreasing.
     An instance of this class is supposed to be passed in the probe_function
     argument of group_sparse_covariance().
+
     """
     def __init__(self, test_subjs, verbose=0):
         self.test_emp_covs, _ = empirical_covariances(test_subjs)
@@ -824,49 +825,51 @@ class GroupSparseCovarianceCV(BaseEstimator, CacheMixin):
 
     Parameters
     ----------
-    alphas : integer
+    alphas : integer, optional
         initial number of points in the grid of regularization parameter
         values. Each step of grid refinement adds that many points as well.
+        Default=4.
 
-    n_refinements : integer
-        number of times the initial grid should be refined.
+    n_refinements : integer, optional
+        number of times the initial grid should be refined. Default=4.
 
-    cv : integer
+    cv : integer, optional
         number of folds in a K-fold cross-validation scheme. If None is passed,
         defaults to 3.
 
-    tol_cv : float
+    tol_cv : float, optional
         tolerance used to get the optimal alpha value. It has the same meaning
         as the `tol` parameter in :func:`group_sparse_covariance`.
+        Default=1e-2.
 
-    max_iter_cv : integer
+    max_iter_cv : integer, optional
         maximum number of iterations for each optimization, during the alpha-
-        selection phase.
+        selection phase. Default=50.
 
-    tol : float
+    tol : float, optional
         tolerance used during the final optimization for determining precision
-        matrices value.
+        matrices value. Default=1e-3.
 
-    max_iter : integer
-        maximum number of iterations in the final optimization.
+    max_iter : integer, optional
+        maximum number of iterations in the final optimization. Default=100.
 
-    verbose : integer
-        verbosity level. 0 means nothing is printed to the user.
+    verbose : integer, optional
+        verbosity level. 0 means nothing is printed to the user. Default=0.
 
-    n_jobs : integer
+    n_jobs : integer, optional
         maximum number of cpu cores to use. The number of cores actually used
         at the same time cannot exceed the number of folds in folding strategy
-        (that is, the value of cv).
+        (that is, the value of cv). Default=1.
 
-    debug : bool
+    debug : bool, optional
         if True, activates some internal checks for consistency. Only useful
-        for nilearn developers, not users.
+        for nilearn developers, not users. Default=False.
 
-    early_stopping : bool
+    early_stopping : bool, optional
         if True, reduce computation time by using a heuristic to reduce the
         number of iterations required to get the optimal value for alpha. Be
         aware that this can lead to slightly different values for the optimal
-        alpha compared to early_stopping=False.
+        alpha compared to early_stopping=False. Default=True.
 
     Attributes
     ----------
@@ -891,7 +894,7 @@ class GroupSparseCovarianceCV(BaseEstimator, CacheMixin):
     See also
     --------
     GroupSparseCovariance,
-    sklearn.covariance.GraphLassoCV
+    sklearn.covariance.GraphicalLassoCV
 
     Notes
     -----
@@ -899,6 +902,7 @@ class GroupSparseCovarianceCV(BaseEstimator, CacheMixin):
     iteratively refined grid: first the cross-validated scores on a grid are
     computed, then a new refined grid is centered around the maximum, and so
     on.
+
     """
     def __init__(self, alphas=4, n_refinements=4, cv=None,
                  tol_cv=1e-2, max_iter_cv=50,
@@ -929,8 +933,9 @@ class GroupSparseCovarianceCV(BaseEstimator, CacheMixin):
 
         Returns
         -------
-        self: GroupSparseCovarianceCV
+        self : GroupSparseCovarianceCV
             the object instance itself.
+
         """
         # Empirical covariances
         emp_covs, n_samples = \
@@ -949,9 +954,8 @@ class GroupSparseCovarianceCV(BaseEstimator, CacheMixin):
         path = list()  # List of (alpha, scores, covs)
         n_alphas = self.alphas
 
-        if isinstance(n_alphas, collections.Sequence):
+        if isinstance(n_alphas, collections.abc.Sequence):
             alphas = list(self.alphas)
-            n_alphas = len(alphas)
             n_refinements = 1
         else:
             n_refinements = self.n_refinements
@@ -963,7 +967,7 @@ class GroupSparseCovarianceCV(BaseEstimator, CacheMixin):
         covs_init = itertools.repeat(None)
 
         # Copying the cv generators to use them n_refinements times.
-        cv_ = izip(*cv)
+        cv_ = zip(*cv)
 
         for i, (this_cv) in enumerate(itertools.tee(cv_, n_refinements)):
             # Compute the cross-validated loss on the current grid
@@ -1000,7 +1004,7 @@ class GroupSparseCovarianceCV(BaseEstimator, CacheMixin):
             #   value of alpha.
             precisions_list, scores = list(zip(*this_path))
             # now scores[i][j] is the score for the i-th folding, j-th value of
-            # alpha (analoguous for precisions_list)
+            # alpha (analogous for precisions_list)
             precisions_list = list(zip(*precisions_list))
             scores = [np.mean(sc) for sc in zip(*scores)]
             # scores[i] is the mean score obtained for the i-th value of alpha.
@@ -1014,7 +1018,7 @@ class GroupSparseCovarianceCV(BaseEstimator, CacheMixin):
             best_score = -np.inf
             last_finite_idx = 0
             for index, (alpha, this_score, _) in enumerate(path):
-                if this_score >= .1 / np.finfo(np.float).eps:
+                if this_score >= .1 / np.finfo(np.float64).eps:
                     this_score = np.nan
                 if np.isfinite(this_score):
                     last_finite_idx = index
