@@ -5,74 +5,40 @@ Test the datasets module
 # License: simplified BSD
 
 import contextlib
+import gzip
 import os
 import shutil
-import numpy as np
-import zipfile
 import tarfile
-import gzip
+import zipfile
+import urllib
+import json
+from unittest.mock import MagicMock
 from tempfile import mkdtemp, mkstemp
 
-from nose import with_setup
-from nose.tools import assert_true, assert_false, assert_equal
+try:
+    import boto3  # noqa: F401
+
+except ImportError:
+    BOTO_INSTALLED = False
+else:
+    BOTO_INSTALLED = True
+
+import numpy as np
+import pytest
+import requests
 
 from nilearn import datasets
-from nilearn._utils.testing import (mock_request, wrap_chunk_read_,
-                                    FetchFilesMock, assert_raises_regex)
+from nilearn.datasets.utils import (_get_dataset_dir,
+                                    make_fresh_openneuro_dataset_urls_index)
+from nilearn.datasets import utils
+
 
 currdir = os.path.dirname(os.path.abspath(__file__))
 datadir = os.path.join(currdir, 'data')
-tmpdir = None
-url_request = None
-file_mock = None
 
 
-def setup_tmpdata():
-    # create temporary dir
-    global tmpdir
-    tmpdir = mkdtemp()
-
-
-def setup_mock(utils_mod=datasets.utils, dataset_mod=datasets.utils):
-    global original_url_request
-    global mock_url_request
-    mock_url_request = mock_request()
-    original_url_request = utils_mod._urllib.request
-    utils_mod._urllib.request = mock_url_request
-
-    global original_chunk_read
-    global mock_chunk_read
-    mock_chunk_read = wrap_chunk_read_(utils_mod._chunk_read_)
-    original_chunk_read = utils_mod._chunk_read_
-    utils_mod._chunk_read_ = mock_chunk_read
-
-    global original_fetch_files
-    global mock_fetch_files
-    mock_fetch_files = FetchFilesMock()
-    original_fetch_files = dataset_mod._fetch_files
-    dataset_mod._fetch_files = mock_fetch_files
-
-
-def teardown_mock(utils_mod=datasets.utils, dataset_mod=datasets.utils):
-    global original_url_request
-    utils_mod._urllib.request = original_url_request
-
-    global original_chunk_read
-    utils_mod.chunk_read_ = original_chunk_read
-
-    global original_fetch_files
-    dataset_mod._fetch_files = original_fetch_files
-
-
-def teardown_tmpdata():
-    # remove temporary dir
-    global tmpdir
-    if tmpdir is not None:
-        shutil.rmtree(tmpdir)
-
-
-@with_setup(setup_tmpdata, teardown_tmpdata)
-def test_get_dataset_dir():
+@pytest.mark.parametrize("should_cast_path_to_string", [False, True])
+def test_get_dataset_dir(should_cast_path_to_string, tmp_path):
     # testing folder creation under different environments, enforcing
     # a custom clean install
     os.environ.pop('NILEARN_DATA', None)
@@ -80,57 +46,59 @@ def test_get_dataset_dir():
 
     expected_base_dir = os.path.expanduser('~/nilearn_data')
     data_dir = datasets.utils._get_dataset_dir('test', verbose=0)
-    assert_equal(data_dir, os.path.join(expected_base_dir, 'test'))
+    assert data_dir == os.path.join(expected_base_dir, 'test')
     assert os.path.exists(data_dir)
     shutil.rmtree(data_dir)
 
-    expected_base_dir = os.path.join(tmpdir, 'test_nilearn_data')
+    expected_base_dir = str(tmp_path / 'test_nilearn_data')
     os.environ['NILEARN_DATA'] = expected_base_dir
     data_dir = datasets.utils._get_dataset_dir('test', verbose=0)
-    assert_equal(data_dir, os.path.join(expected_base_dir, 'test'))
+    assert data_dir == os.path.join(expected_base_dir, 'test')
     assert os.path.exists(data_dir)
     shutil.rmtree(data_dir)
 
-    expected_base_dir = os.path.join(tmpdir, 'nilearn_shared_data')
+    expected_base_dir = str(tmp_path / 'nilearn_shared_data')
     os.environ['NILEARN_SHARED_DATA'] = expected_base_dir
     data_dir = datasets.utils._get_dataset_dir('test', verbose=0)
-    assert_equal(data_dir, os.path.join(expected_base_dir, 'test'))
+    assert data_dir == os.path.join(expected_base_dir, 'test')
     assert os.path.exists(data_dir)
     shutil.rmtree(data_dir)
 
-    expected_base_dir = os.path.join(tmpdir, 'env_data')
-    expected_dataset_dir = os.path.join(expected_base_dir, 'test')
+    expected_base_dir = tmp_path / 'env_data'
+    expected_dataset_dir = expected_base_dir / 'test'
+    if should_cast_path_to_string:
+        expected_dataset_dir = str(expected_dataset_dir)
     data_dir = datasets.utils._get_dataset_dir(
         'test', default_paths=[expected_dataset_dir], verbose=0)
-    assert_equal(data_dir, os.path.join(expected_base_dir, 'test'))
+    assert data_dir == str(expected_dataset_dir)
     assert os.path.exists(data_dir)
     shutil.rmtree(data_dir)
 
-    no_write = os.path.join(tmpdir, 'no_write')
+    no_write = str(tmp_path / 'no_write')
     os.makedirs(no_write)
     os.chmod(no_write, 0o400)
 
-    expected_base_dir = os.path.join(tmpdir, 'nilearn_shared_data')
+    expected_base_dir = str(tmp_path / 'nilearn_shared_data')
     os.environ['NILEARN_SHARED_DATA'] = expected_base_dir
     data_dir = datasets.utils._get_dataset_dir('test',
                                                default_paths=[no_write],
                                                verbose=0)
     # Non writeable dir is returned because dataset may be in there.
-    assert_equal(data_dir, no_write)
+    assert data_dir == no_write
     assert os.path.exists(data_dir)
     os.chmod(no_write, 0o600)
     shutil.rmtree(data_dir)
 
     # Verify exception for a path which exists and is a file
-    test_file = os.path.join(tmpdir, 'some_file')
+    test_file = str(tmp_path / 'some_file')
     with open(test_file, 'w') as out:
         out.write('abcfeg')
 
-    assert_raises_regex(OSError,
-                        'Nilearn tried to store the dataset in the following '
-                        'directories, but',
-                        datasets.utils._get_dataset_dir,
-                        'test', test_file, verbose=0)
+    with pytest.raises(
+            OSError,
+            match='Nilearn tried to store the dataset in the following '
+                  'directories, but'):
+        datasets.utils._get_dataset_dir('test', test_file, verbose=0)
 
 
 def test_md5_sum_file():
@@ -138,7 +106,7 @@ def test_md5_sum_file():
     out, f = mkstemp()
     os.write(out, b'abcfeg')
     os.close(out)
-    assert_equal(datasets.utils._md5_sum_file(f),
+    assert (datasets.utils._md5_sum_file(f) ==
                  '18f32295c556b2a1a3a8e68fe1ad40f7')
     os.remove(f)
 
@@ -150,10 +118,10 @@ def test_read_md5_sum_file():
              b'70886dcabe7bf5c5a1c24ca24e4cbd94  test/some_image.nii')
     os.close(out)
     h = datasets.utils._read_md5_sum_file(f)
-    assert_true('/tmp/test' in h)
-    assert_false('/etc/test' in h)
-    assert_equal(h['test/some_image.nii'], '70886dcabe7bf5c5a1c24ca24e4cbd94')
-    assert_equal(h['/tmp/test'], '20861c8c3fe177da19a7e9539a5dbac')
+    assert '/tmp/test' in h
+    assert not '/etc/test' in h
+    assert h['test/some_image.nii'] == '70886dcabe7bf5c5a1c24ca24e4cbd94'
+    assert h['/tmp/test'] == '20861c8c3fe177da19a7e9539a5dbac'
     os.remove(f)
 
 
@@ -186,13 +154,13 @@ def test_tree():
     # assert_equal(tree_[1]['dir2'][0], 'file21')
     # assert_equal(tree_[2], 'file1')
     # assert_equal(tree_[3], 'file2')
-    assert_equal(tree_[0][1][0][1][0], os.path.join(dir11, 'file111'))
-    assert_equal(len(tree_[0][1][1][1]), 0)
-    assert_equal(tree_[0][1][2], os.path.join(dir1, 'file11'))
-    assert_equal(tree_[0][1][3], os.path.join(dir1, 'file12'))
-    assert_equal(tree_[1][1][0], os.path.join(dir2, 'file21'))
-    assert_equal(tree_[2], os.path.join(parent, 'file1'))
-    assert_equal(tree_[3], os.path.join(parent, 'file2'))
+    assert tree_[0][1][0][1][0] == os.path.join(dir11, 'file111')
+    assert len(tree_[0][1][1][1]) == 0
+    assert tree_[0][1][2] == os.path.join(dir1, 'file11')
+    assert tree_[0][1][3] == os.path.join(dir1, 'file12')
+    assert tree_[1][1][0] == os.path.join(dir2, 'file21')
+    assert tree_[2] == os.path.join(parent, 'file1')
+    assert tree_[3] == os.path.join(parent, 'file2')
 
     # Clean
     shutil.rmtree(parent)
@@ -219,21 +187,21 @@ def test_movetree():
 
     datasets.utils.movetree(dir1, dir2)
 
-    assert_false(os.path.exists(dir11))
-    assert_false(os.path.exists(dir12))
-    assert_false(os.path.exists(os.path.join(dir1, 'file11')))
-    assert_false(os.path.exists(os.path.join(dir1, 'file12')))
-    assert_false(os.path.exists(os.path.join(dir11, 'file111')))
-    assert_false(os.path.exists(os.path.join(dir12, 'file121')))
+    assert not os.path.exists(dir11)
+    assert not os.path.exists(dir12)
+    assert not os.path.exists(os.path.join(dir1, 'file11'))
+    assert not os.path.exists(os.path.join(dir1, 'file12'))
+    assert not os.path.exists(os.path.join(dir11, 'file111'))
+    assert not os.path.exists(os.path.join(dir12, 'file121'))
     dir11 = os.path.join(dir2, 'dir11')
     dir12 = os.path.join(dir2, 'dir12')
 
-    assert_true(os.path.exists(dir11))
-    assert_true(os.path.exists(dir12))
-    assert_true(os.path.exists(os.path.join(dir2, 'file11')))
-    assert_true(os.path.exists(os.path.join(dir2, 'file12')))
-    assert_true(os.path.exists(os.path.join(dir11, 'file111')))
-    assert_true(os.path.exists(os.path.join(dir12, 'file121')))
+    assert os.path.exists(dir11)
+    assert os.path.exists(dir12)
+    assert os.path.exists(os.path.join(dir2, 'file11'))
+    assert os.path.exists(os.path.join(dir2, 'file12'))
+    assert os.path.exists(os.path.join(dir11, 'file111'))
+    assert os.path.exists(os.path.join(dir12, 'file121'))
 
 
 def test_filter_columns():
@@ -246,10 +214,10 @@ def test_filter_columns():
                         dtype=[('INT', int), ('STR', 'S1')])
 
     f = datasets.utils._filter_columns(values, {'INT': (23, 46)})
-    assert_equal(np.sum(f), 24)
+    assert np.sum(f) == 24
 
     f = datasets.utils._filter_columns(values, {'INT': [0, 9, (12, 24)]})
-    assert_equal(np.sum(f), 15)
+    assert np.sum(f) == 15
 
     value1 = value1 % 2
     values = np.asarray(list(zip(value1, value2)),
@@ -257,20 +225,20 @@ def test_filter_columns():
 
     # No filter
     f = datasets.utils._filter_columns(values, [])
-    assert_equal(np.sum(f), 500)
+    assert np.sum(f) == 500
 
     f = datasets.utils._filter_columns(values, {'STR': b'b'})
-    assert_equal(np.sum(f), 167)
+    assert np.sum(f) == 167
 
     f = datasets.utils._filter_columns(values, {'STR': u'b'})
-    assert_equal(np.sum(f), 167)
+    assert np.sum(f) == 167
 
     f = datasets.utils._filter_columns(values, {'INT': 1, 'STR': b'b'})
-    assert_equal(np.sum(f), 84)
+    assert np.sum(f) == 84
 
     f = datasets.utils._filter_columns(values, {'INT': 1, 'STR': b'b'},
                                        combination='or')
-    assert_equal(np.sum(f), 333)
+    assert np.sum(f) == 333
 
 
 def test_uncompress():
@@ -316,67 +284,138 @@ def test_uncompress():
             shutil.rmtree(dtemp)
 
 
-@with_setup(setup_mock, teardown_mock)
-@with_setup(setup_tmpdata, teardown_tmpdata)
-def test_fetch_file_overwrite():
+@pytest.mark.parametrize("should_cast_path_to_string", [False, True])
+def test_fetch_file_overwrite(should_cast_path_to_string,
+                              tmp_path, request_mocker):
+    if should_cast_path_to_string:
+        tmp_path = str(tmp_path)
+
     # overwrite non-exiting file.
-    fil = datasets.utils._fetch_file(url='http://foo/', data_dir=tmpdir,
+    fil = datasets.utils._fetch_file(url='http://foo/', data_dir=str(tmp_path),
                                      verbose=0, overwrite=True)
-    assert_equal(len(mock_url_request.urls), 1)
-    assert_true(os.path.exists(fil))
+    assert request_mocker.url_count == 1
+    assert os.path.exists(fil)
     with open(fil, 'r') as fp:
-        assert_equal(fp.read(), '')
+        assert fp.read() == ''
 
     # Modify content
     with open(fil, 'w') as fp:
         fp.write('some content')
 
     # Don't overwrite existing file.
-    fil = datasets.utils._fetch_file(url='http://foo/', data_dir=tmpdir,
+    fil = datasets.utils._fetch_file(url='http://foo/', data_dir=str(tmp_path),
                                      verbose=0, overwrite=False)
-    assert_equal(len(mock_url_request.urls), 1)
-    assert_true(os.path.exists(fil))
+
+    assert request_mocker.url_count == 1
+    assert os.path.exists(fil)
     with open(fil, 'r') as fp:
-        assert_equal(fp.read(), 'some content')
+        assert fp.read() == 'some content'
 
     # Overwrite existing file.
     # Overwrite existing file.
-    fil = datasets.utils._fetch_file(url='http://foo/', data_dir=tmpdir,
+    fil = datasets.utils._fetch_file(url='http://foo/', data_dir=str(tmp_path),
                                      verbose=0, overwrite=True)
-    assert_equal(len(mock_url_request.urls), 1)
-    assert_true(os.path.exists(fil))
+    assert request_mocker.url_count == 2
+    assert os.path.exists(fil)
     with open(fil, 'r') as fp:
-        assert_equal(fp.read(), '')
+        assert fp.read() == ''
 
 
-@with_setup(setup_mock, teardown_mock)
-@with_setup(setup_tmpdata, teardown_tmpdata)
-def test_fetch_files_overwrite():
+@pytest.mark.parametrize("should_cast_path_to_string", [False, True])
+def test_fetch_files_use_session(should_cast_path_to_string,
+                                 tmp_path, request_mocker):
+    if should_cast_path_to_string:
+        tmp_path = str(tmp_path)
+
+    # regression test for https://github.com/nilearn/nilearn/issues/2863
+    session = MagicMock()
+    datasets.utils._fetch_files(
+        files=[
+            ("example1", "https://example.org/example1", {"overwrite": True}),
+            ("example2", "https://example.org/example2", {"overwrite": True}),
+        ],
+        data_dir=str(tmp_path),
+        session=session,
+    )
+    assert session.send.call_count == 2
+
+
+@pytest.mark.parametrize("should_cast_path_to_string", [False, True])
+def test_fetch_files_overwrite(should_cast_path_to_string,
+                               tmp_path, request_mocker):
+    if should_cast_path_to_string:
+        tmp_path = str(tmp_path)
+
     # overwrite non-exiting file.
     files = ('1.txt', 'http://foo/1.txt')
-    fil = datasets.utils._fetch_files(data_dir=tmpdir, verbose=0,
+    fil = datasets.utils._fetch_files(data_dir=str(tmp_path), verbose=0,
                                       files=[files + (dict(overwrite=True),)])
-    assert_equal(len(mock_url_request.urls), 1)
-    assert_true(os.path.exists(fil[0]))
+    assert request_mocker.url_count == 1
+    assert os.path.exists(fil[0])
     with open(fil[0], 'r') as fp:
-        assert_equal(fp.read(), '')
+        assert fp.read() == ''
 
     # Modify content
     with open(fil[0], 'w') as fp:
         fp.write('some content')
 
     # Don't overwrite existing file.
-    fil = datasets.utils._fetch_files(data_dir=tmpdir, verbose=0,
+    fil = datasets.utils._fetch_files(data_dir=str(tmp_path), verbose=0,
                                       files=[files + (dict(overwrite=False),)])
-    assert_equal(len(mock_url_request.urls), 1)
-    assert_true(os.path.exists(fil[0]))
+    assert request_mocker.url_count == 1
+    assert os.path.exists(fil[0])
     with open(fil[0], 'r') as fp:
-        assert_equal(fp.read(), 'some content')
+        assert fp.read() == 'some content'
 
     # Overwrite existing file.
-    fil = datasets.utils._fetch_files(data_dir=tmpdir, verbose=0,
+    fil = datasets.utils._fetch_files(data_dir=str(tmp_path), verbose=0,
                                       files=[files + (dict(overwrite=True),)])
-    assert_equal(len(mock_url_request.urls), 1)
-    assert_true(os.path.exists(fil[0]))
+    assert request_mocker.url_count == 2
+    assert os.path.exists(fil[0])
     with open(fil[0], 'r') as fp:
-        assert_equal(fp.read(), '')
+        assert fp.read() == ''
+
+
+@pytest.mark.skipif(not BOTO_INSTALLED,
+                    reason='Boto3  missing; necessary for this test')
+def test_make_fresh_openneuro_dataset_urls_index(tmp_path, request_mocker):
+    dataset_version = 'ds000030_R1.0.4'
+    data_prefix = '{}/{}/uncompressed'.format(
+        dataset_version.split('_')[0], dataset_version)
+    data_dir = _get_dataset_dir(data_prefix, data_dir=str(tmp_path),
+                                verbose=1)
+    url_file = os.path.join(data_dir,
+                            'nistats_fetcher_openneuro_dataset_urls.json',
+                            )
+    # Prepare url files for subject and filter tests
+    file_list = [data_prefix + '/stuff.html',
+                 data_prefix + '/sub-xxx.html',
+                 data_prefix + '/sub-yyy.html',
+                 data_prefix + '/sub-xxx/ses-01_task-rest.txt',
+                 data_prefix + '/sub-xxx/ses-01_task-other.txt',
+                 data_prefix + '/sub-xxx/ses-02_task-rest.txt',
+                 data_prefix + '/sub-xxx/ses-02_task-other.txt',
+                 data_prefix + '/sub-yyy/ses-01.txt',
+                 data_prefix + '/sub-yyy/ses-02.txt']
+    with open(url_file, 'w') as f:
+        json.dump(file_list, f)
+
+    # Only 1 subject and not subject specific files get downloaded
+    datadir, dl_files = make_fresh_openneuro_dataset_urls_index(
+        str(tmp_path), dataset_version)
+    assert isinstance(datadir, str)
+    assert isinstance(dl_files, list)
+    assert len(dl_files) == len(file_list)
+
+
+def test_naive_ftp_adapter():
+    sender = utils._NaiveFTPAdapter()
+    resp = sender.send(
+        requests.Request("GET", "ftp://example.com").prepare())
+    resp.close()
+    resp.raw.close.assert_called_with()
+    urllib.request.OpenerDirector.open.side_effect = urllib.error.URLError(
+        "timeout")
+    with pytest.raises(requests.RequestException, match="timeout"):
+        resp = sender.send(
+            requests.Request("GET", "ftp://example.com").prepare())
