@@ -11,8 +11,6 @@ TV-L1, Graph-Net, etc.)
 #         THIRION Bertrand
 # License: simplified BSD
 
-from distutils.version import LooseVersion
-import sklearn
 import warnings
 import numbers
 import time
@@ -20,25 +18,29 @@ import sys
 from functools import partial
 import numpy as np
 from scipy import stats, ndimage
-from sklearn.base import RegressorMixin
 from sklearn.utils.extmath import safe_sparse_dot
 from sklearn.utils import check_array
-from sklearn.linear_model.base import LinearModel
+from sklearn.linear_model import LinearRegression
 from sklearn.feature_selection import (SelectPercentile, f_regression,
                                        f_classif)
-from sklearn.externals.joblib import Memory, Parallel, delayed
+from joblib import Memory, Parallel, delayed
 from sklearn.preprocessing import LabelBinarizer
 from sklearn.metrics import accuracy_score
 from ..input_data.masker_validation import check_embedded_nifti_masker
 from .._utils.param_validation import _adjust_screening_percentile
+from .._utils import fill_doc
 from sklearn.utils import check_X_y
 from sklearn.model_selection import check_cv
-from sklearn.linear_model.base import _preprocess_data as center_data
-from .._utils.compat import _basestring
+try:
+    from sklearn.linear_model._base import _preprocess_data as center_data
+except ImportError:
+    # Sklearn < 0.23
+    from sklearn.linear_model.base import _preprocess_data as center_data
 from .._utils.cache_mixin import CacheMixin
-from .objective_functions import _unmask
+from nilearn.masking import _unmask_from_to_3d_array
 from .space_net_solvers import (tvl1_solver, _graph_net_logistic,
                                 _graph_net_squared_loss)
+from nilearn.image import get_data
 
 
 def _crop_mask(mask):
@@ -59,6 +61,7 @@ def _crop_mask(mask):
     return mask[i_min:i_max + 1, j_min:j_max + 1, k_min:k_max + 1]
 
 
+@fill_doc
 def _univariate_feature_screening(
         X, y, mask, is_classif, screening_percentile, smoothing_fwhm=2.):
     """
@@ -81,10 +84,8 @@ def _univariate_feature_screening(
     screening_percentile : float in the closed interval [0., 100.]
         Only the `screening_percentile * 100" percent most import voxels will
         be retained.
-
-    smoothing_fwhm : float, optional (default 2.)
-        FWHM for isotropically smoothing the data X before F-testing. A value
-        of zero means "don't smooth".
+    %(smoothing_fwhm)s
+        Default=2.
 
     Returns
     -------
@@ -105,9 +106,9 @@ def _univariate_feature_screening(
         sX = np.empty(X.shape)
         for sample in range(sX.shape[0]):
             sX[sample] = ndimage.gaussian_filter(
-                _unmask(X[sample].copy(),  # avoid modifying X
-                        mask), (smoothing_fwhm, smoothing_fwhm,
-                                smoothing_fwhm))[mask]
+                _unmask_from_to_3d_array(X[sample].copy(),  # avoid modifying X
+                                         mask), (smoothing_fwhm, smoothing_fwhm,
+                                                 smoothing_fwhm))[mask]
     else:
         sX = X
 
@@ -121,7 +122,7 @@ def _univariate_feature_screening(
     mask_ = mask.copy()
     mask_[mask] = (support > 0)
     mask_ = ndimage.binary_dilation(ndimage.binary_erosion(
-        mask_)).astype(np.bool)
+        mask_)).astype(bool)
     mask_[np.logical_not(mask)] = 0
     support = mask_[mask]
     X = X[:, support]
@@ -141,25 +142,28 @@ def _space_net_alpha_grid(X, y, eps=1e-3, n_alphas=10, l1_ratio=1.,
     y : ndarray, shape (n_samples,)
         Target / response vector.
 
-    l1_ratio : float
+    l1_ratio : float, optional
         The ElasticNet mixing parameter, with ``0 <= l1_ratio <= 1``.
         For ``l1_ratio = 0`` the penalty is purely a spatial prior
         (Graph-Net, TV, etc.). ``For l1_ratio = 1`` it is an L1 penalty.
         For ``0 < l1_ratio < 1``, the penalty is a combination of L1
         and a spatial prior.
+        Default=1.
 
     eps : float, optional
         Length of the path. ``eps=1e-3`` means that
         ``alpha_min / alpha_max = 1e-3``.
+        Default=1e-3.
 
     n_alphas : int, optional
         Number of alphas along the regularization path.
+        Default=10.
 
-    logistic : bool, optional (default False)
+    logistic : bool, optional
         Indicates where the underlying loss function is logistic.
+        Default=False.
 
     """
-
     if logistic:
         # Computes the theoretical upper bound for the overall
         # regularization, as derived in "An Interior-Point Method for
@@ -281,6 +285,7 @@ class _EarlyStoppingCallback(object):
             return pearson_score, spearman_score
 
 
+@fill_doc
 def path_scores(solver, X, y, mask, alphas, l1_ratios, train, test,
                 solver_params, is_classif=False, n_alphas=10, eps=1E-3,
                 key=None, debias=False, Xmean=None,
@@ -323,8 +328,31 @@ def path_scores(solver, X, y, mask, alphas, l1_ratios, train, test,
     solver : function handle
        See for example tv.TVl1Classifier documentation.
 
-    solver_params: dict
+    solver_params : dict
        Dictionary of param-value pairs to be passed to solver.
+
+    is_classif : bool, optional
+        Indicates whether the loss is a classification loss or a
+        regression loss. Default=False.
+
+    Xmean: ??? TODO: Add description.
+
+    key: ??? TODO: Add description.
+
+    debias : bool, optional
+        If set, then the estimated weights maps will be debiased.
+        Default=False.
+
+    screening_percentile : float in the interval [0, 100], optional
+        Percentile value for ANOVA univariate feature selection. A value of
+        100 means 'keep all features'. This percentile is expressed
+        w.r.t the volume of a standard (MNI152) brain, and so is corrected
+        at runtime to correspond to the volume of the user-supplied mask
+        (which is typically smaller). If '100' is given, all the features
+        are used, regardless of the number of voxels.
+        Default=20.
+    %(verbose)s
+
     """
     if l1_ratios is None:
         raise ValueError("l1_ratios must be specified!")
@@ -381,6 +409,9 @@ def path_scores(solver, X, y, mask, alphas, l1_ratios, train, test,
             if best_alpha is None:
                 best_alpha = alphas_[0]
             init = None
+            path_solver_params = solver_params.copy()
+            # Use a lighter tol during the path
+            path_solver_params['tol'] = 2 * path_solver_params.get('tol', 1e-4)
             for alpha in alphas_:
                 # setup callback mechanism for early stopping
                 early_stopper = _EarlyStoppingCallback(
@@ -389,7 +420,7 @@ def path_scores(solver, X, y, mask, alphas, l1_ratios, train, test,
                 w, _, init = solver(
                     X_train, y_train, alpha, l1_ratio, mask=mask, init=init,
                     callback=early_stopper, verbose=max(verbose - 1, 0.),
-                    **solver_params)
+                    **path_solver_params)
 
                 # We use 2 scores for model selection: the second one is to
                 # disambiguate between regions of equivalent Spearman
@@ -447,12 +478,13 @@ def path_scores(solver, X, y, mask, alphas, l1_ratios, train, test,
             y_train_mean, key)
 
 
-class BaseSpaceNet(LinearModel, RegressorMixin, CacheMixin):
+@fill_doc
+class BaseSpaceNet(LinearRegression, CacheMixin):
     """
     Regression and classification learners with sparsity and spatial priors
 
     `SpaceNet` implements Graph-Net and TV-L1 priors /
-    penalties. Thus, the penalty is a sum an L1 term and a spatial term. The
+    penalties. Thus, the penalty is a sum of an L1 term and a spatial term. The
     aim of such a hybrid prior is to obtain weights maps which are structured
     (due to the spatial prior) and sparse (enforced by L1 norm).
 
@@ -492,29 +524,13 @@ class BaseSpaceNet(LinearModel, RegressorMixin, CacheMixin):
         Mask to be used on data. If an instance of masker is passed,
         then its mask will be used. If no mask is it will be computed
         automatically by a NiftiMasker.
-
-    target_affine : 3x3 or 4x4 matrix, optional (default None)
-        This parameter is passed to image.resample_img. An important use-case
-        of this parameter is for downsampling the input data to a coarser
-        resolution (to speed of the model fit). Please see the related
-        documentation for details.
-
-    target_shape : 3-tuple of integers, optional (default None)
-        This parameter is passed to image.resample_img. Please see the
-        related documentation for details.
-
-    low_pass: None or float, optional
-        This parameter is passed to signal.clean. Please see the related
-        documentation for details
-
-    high_pass: None or float, optional
-        This parameter is passed to signal.clean. Please see the related
-        documentation for details
-
-    t_r : float, optional (default None)
-        This parameter is passed to signal.clean. Please see the related
-        documentation for details.
-
+    %(target_affine)s
+        An important use-case of this parameter is for downsampling the
+        input data to a coarser resolution (to speed of the model fit).
+    %(target_shape)s
+    %(low_pass)s
+    %(high_pass)s
+    %(t_r)s
     screening_percentile : float in the interval [0, 100]; Optional (
     default 20)
         Percentile value for ANOVA univariate feature selection. A value of
@@ -532,27 +548,15 @@ class BaseSpaceNet(LinearModel, RegressorMixin, CacheMixin):
     fit_intercept : bool, optional (default True)
         Fit or not an intercept.
 
-    max_iter : int (default 1000)
+    max_iter : int (default 200)
         Defines the iterations for the solver.
 
     tol : float, optional (default 5e-4)
         Defines the tolerance for convergence for the backend FISTA solver.
-
-    verbose : int, optional (default 1)
-        Verbosity level.
-
-    n_jobs : int, optional (default 1)
-        Number of jobs in solving the sub-problems.
-
-    memory: instance of joblib.Memory or string
-        Used to cache the masking process.
-        By default, no caching is done. If a string is given, it is the
-        path to the caching directory.
-
-    memory_level: integer, optional (default 1)
-        Rough estimator of the amount of memory used by caching. Higher value
-        means more memory for caching.
-
+    %(verbose)s
+    %(n_jobs)s
+    %(memory)s
+    %(memory_level1)s
     cv : int, a cv generator instance, or None (default 8)
         The input specifying which cross-validation generator to use.
         It can be an integer, in which case it is the number of folds in a
@@ -640,7 +644,7 @@ class BaseSpaceNet(LinearModel, RegressorMixin, CacheMixin):
     def __init__(self, penalty="graph-net", is_classif=False, loss=None,
                  l1_ratios=.5, alphas=None, n_alphas=10, mask=None,
                  target_affine=None, target_shape=None, low_pass=None,
-                 high_pass=None, t_r=None, max_iter=1000, tol=5e-4,
+                 high_pass=None, t_r=None, max_iter=200, tol=5e-4,
                  memory=None, memory_level=1, standardize=True, verbose=1,
                  mask_args=None,
                  n_jobs=1, eps=1e-3, cv=8, fit_intercept=True,
@@ -744,7 +748,7 @@ class BaseSpaceNet(LinearModel, RegressorMixin, CacheMixin):
         """
         # misc
         self.check_params()
-        if self.memory is None or isinstance(self.memory, _basestring):
+        if self.memory is None or isinstance(self.memory, str):
             self.memory_ = Memory(self.memory,
                                   verbose=max(0, self.verbose - 1))
         else:
@@ -756,11 +760,11 @@ class BaseSpaceNet(LinearModel, RegressorMixin, CacheMixin):
         self.masker_ = check_embedded_nifti_masker(self, multi_subject=False)
         X = self.masker_.fit_transform(X)
 
-        X, y = check_X_y(X, y, ['csr', 'csc', 'coo'], dtype=np.float,
+        X, y = check_X_y(X, y, ['csr', 'csc', 'coo'], dtype=float,
                          multi_output=True, y_numeric=not self.is_classif)
 
         if not self.is_classif and np.all(np.diff(y) == 0.):
-            raise ValueError("The given input y must have atleast 2 targets"
+            raise ValueError("The given input y must have at least 2 targets"
                              " to do regression analysis. You provided only"
                              " one target {0}".format(np.unique(y)))
 
@@ -769,7 +773,7 @@ class BaseSpaceNet(LinearModel, RegressorMixin, CacheMixin):
         self.Xstd_ = X.std(axis=0)
         self.Xstd_[self.Xstd_ < 1e-8] = 1
         self.mask_img_ = self.masker_.mask_img_
-        self.mask_ = self.mask_img_.get_data().astype(np.bool)
+        self.mask_ = get_data(self.mask_img_).astype(bool)
         n_samples, _ = X.shape
         y = np.array(y).copy()
         l1_ratios = self.l1_ratios
@@ -839,12 +843,13 @@ class BaseSpaceNet(LinearModel, RegressorMixin, CacheMixin):
              y_train_mean, (cls, fold)) in Parallel(
             n_jobs=self.n_jobs, verbose=2 * self.verbose)(
                 delayed(self._cache(path_scores, func_memory_level=2))(
-                solver, X, y[:, cls] if n_problems > 1 else y, self.mask_,
-                alphas, l1_ratios, self.cv_[fold][0], self.cv_[fold][1],
-                solver_params, n_alphas=self.n_alphas, eps=self.eps,
-                is_classif=self.loss == "logistic", key=(cls, fold),
-                debias=self.debias, verbose=self.verbose,
-                screening_percentile=self.screening_percentile_,
+                    solver, X, y[:, cls] if n_problems > 1 else y,
+                    self.mask_, alphas, l1_ratios, self.cv_[fold][0],
+                    self.cv_[fold][1], solver_params, n_alphas=self.n_alphas,
+                    eps=self.eps, is_classif=self.loss == "logistic",
+                    key=(cls, fold), debias=self.debias,
+                    verbose=self.verbose,
+                    screening_percentile=self.screening_percentile_,
                 ) for cls in range(n_problems) for fold in range(n_folds)):
             self.best_model_params_.append((best_alpha, best_l1_ratio))
             self.alpha_grids_.append(alphas)
@@ -897,12 +902,13 @@ class BaseSpaceNet(LinearModel, RegressorMixin, CacheMixin):
         -------
         array, shape=(n_samples,) if n_classes == 2 else (n_samples, n_classes)
             Confidence scores per (sample, class) combination. In the binary
-            case, confidence score for self.classes_[1] where >0 means this
+            case, confidence score for `self.classes_[1]` where >0 means this
             class would be predicted.
         """
         # handle regression (least-squared loss)
         if not self.is_classif:
-            return LinearModel.decision_function(self, X)
+            raise ValueError(
+                'There is no decision_function in classification')
 
         X = check_array(X)
         n_features = self.coef_.shape[1]
@@ -937,17 +943,18 @@ class BaseSpaceNet(LinearModel, RegressorMixin, CacheMixin):
 
         # handle regression (least-squared loss)
         if not self.is_classif:
-            return LinearModel.predict(self, X)
+            return LinearRegression.predict(self, X)
 
         # prediction proper
         scores = self.decision_function(X)
         if len(scores.shape) == 1:
-            indices = (scores > 0).astype(np.int)
+            indices = (scores > 0).astype(int)
         else:
             indices = scores.argmax(axis=1)
         return self.classes_[indices]
 
 
+@fill_doc
 class SpaceNetClassifier(BaseSpaceNet):
     """Classification learners with sparsity and spatial priors.
 
@@ -989,27 +996,11 @@ class SpaceNetClassifier(BaseSpaceNet):
         Mask to be used on data. If an instance of masker is passed,
         then its mask will be used. If no mask is it will be computed
         automatically by a MultiNiftiMasker with default parameters.
-
-    target_affine : 3x3 or 4x4 matrix, optional (default None)
-        This parameter is passed to image.resample_img. Please see the
-        related documentation for details.
-
-    target_shape : 3-tuple of integers, optional (default None)
-        This parameter is passed to image.resample_img. Please see the
-        related documentation for details.
-
-    low_pass: None or float, optional
-        This parameter is passed to signal.clean. Please see the related
-        documentation for details
-
-    high_pass: None or float, optional
-        This parameter is passed to signal.clean. Please see the related
-        documentation for details
-
-    t_r : float, optional (default None)
-        This parameter is passed to signal.clean. Please see the related
-        documentation for details.
-
+    %(target_affine)s
+    %(target_shape)s
+    %(low_pass)s
+    %(high_pass)s
+    %(t_r)s
     screening_percentile : float in the interval [0, 100]; Optional (default 20)
         Percentile value for ANOVA univariate feature selection. A value of
         100 means 'keep all features'. This percentile is is expressed
@@ -1026,27 +1017,15 @@ class SpaceNetClassifier(BaseSpaceNet):
     fit_intercept : bool, optional (default True)
         Fit or not an intercept.
 
-    max_iter : int (default 1000)
+    max_iter : int (default 200)
         Defines the iterations for the solver.
 
     tol : float
         Defines the tolerance for convergence. Defaults to 1e-4.
-
-    verbose : int, optional (default 1)
-        Verbosity level.
-
-    n_jobs : int, optional (default 1)
-        Number of jobs in solving the sub-problems.
-
-    memory: instance of joblib.Memory or string
-        Used to cache the masking process.
-        By default, no caching is done. If a string is given, it is the
-        path to the caching directory.
-
-    memory_level: integer, optional (default 1)
-        Rough estimator of the amount of memory used by caching. Higher value
-        means more memory for caching.
-
+    %(verbose)s
+    %(n_jobs)s
+    %(memory)s
+    %(memory_level1)s
     cv : int, a cv generator instance, or None (default 8)
         The input specifying which cross-validation generator to use.
         It can be an integer, in which case it is the number of folds in a
@@ -1126,12 +1105,17 @@ class SpaceNetClassifier(BaseSpaceNet):
 
     `Xstd_` : array, shape (n_features,)
         Standard deviation of X across samples
+
+    See Also
+    --------
+    nilearn.decoding.SpaceNetRegressor: Graph-Net and TV-L1 priors/penalties
+
     """
 
     def __init__(self, penalty="graph-net", loss="logistic",
                  l1_ratios=.5, alphas=None, n_alphas=10, mask=None,
                  target_affine=None, target_shape=None, low_pass=None,
-                 high_pass=None, t_r=None, max_iter=1000, tol=1e-4,
+                 high_pass=None, t_r=None, max_iter=200, tol=1e-4,
                  memory=Memory(None), memory_level=1, standardize=True,
                  verbose=1, n_jobs=1, eps=1e-3,
                  cv=8, fit_intercept=True, screening_percentile=20.,
@@ -1179,6 +1163,7 @@ class SpaceNetClassifier(BaseSpaceNet):
         return accuracy_score(y, self.predict(X))
 
 
+@fill_doc
 class SpaceNetRegressor(BaseSpaceNet):
     """Regression learners with sparsity and spatial priors.
 
@@ -1217,27 +1202,11 @@ class SpaceNetRegressor(BaseSpaceNet):
         Mask to be used on data. If an instance of masker is passed,
         then its mask will be used. If no mask is it will be computed
         automatically by a MultiNiftiMasker with default parameters.
-
-    target_affine : 3x3 or 4x4 matrix, optional (default None)
-        This parameter is passed to image.resample_img. Please see the
-        related documentation for details.
-
-    target_shape : 3-tuple of integers, optional (default None)
-        This parameter is passed to image.resample_img. Please see the
-        related documentation for details.
-
-    low_pass: None or float, optional
-        This parameter is passed to signal.clean. Please see the related
-        documentation for details
-
-    high_pass: None or float, optional
-        This parameter is passed to signal.clean. Please see the related
-        documentation for details
-
-    t_r : float, optional (default None)
-        This parameter is passed to signal.clean. Please see the related
-        documentation for details
-
+    %(target_affine)s
+    %(target_shape)s
+    %(low_pass)s
+    %(high_pass)s
+    %(t_r)s
     screening_percentile : float in the interval [0, 100]; Optional (default 20)
         Percentile value for ANOVA univariate feature selection. A value of
         100 means 'keep all features'. This percentile is is expressed
@@ -1253,27 +1222,15 @@ class SpaceNetRegressor(BaseSpaceNet):
     fit_intercept : bool, optional (default True)
         Fit or not an intercept.
 
-    max_iter : int (default 1000)
+    max_iter : int (default 200)
         Defines the iterations for the solver.
 
     tol : float
         Defines the tolerance for convergence. Defaults to 1e-4.
-
-    verbose : int, optional (default 1)
-        Verbosity level.
-
-    n_jobs : int, optional (default 1)
-        Number of jobs in solving the sub-problems.
-
-    memory: instance of joblib.Memory or string
-        Used to cache the masking process.
-        By default, no caching is done. If a string is given, it is the
-        path to the caching directory.
-
-    memory_level: integer, optional (default 1)
-        Rough estimator of the amount of memory used by caching. Higher value
-        means more memory for caching.
-
+    %(verbose)s
+    %(n_jobs)s
+    %(memory)s
+    %(memory_level1)s
     cv : int, a cv generator instance, or None (default 8)
         The input specifying which cross-validation generator to use.
         It can be an integer, in which case it is the number of folds in a
@@ -1340,12 +1297,17 @@ class SpaceNetRegressor(BaseSpaceNet):
 
     `Xstd_` : array, shape (n_features,)
         Standard deviation of X across samples
+
+    See Also
+    --------
+    nilearn.decoding.SpaceNetClassifier: Graph-Net and TV-L1 priors/penalties
+
     """
 
     def __init__(self, penalty="graph-net", l1_ratios=.5, alphas=None,
                  n_alphas=10, mask=None, target_affine=None,
                  target_shape=None, low_pass=None, high_pass=None, t_r=None,
-                 max_iter=1000, tol=1e-4, memory=Memory(None), memory_level=1,
+                 max_iter=200, tol=1e-4, memory=Memory(None), memory_level=1,
                  standardize=True, verbose=1, n_jobs=1, eps=1e-3, cv=8,
                  fit_intercept=True, screening_percentile=20., debias=False):
         super(SpaceNetRegressor, self).__init__(
