@@ -13,8 +13,9 @@ import numpy as np
 import scipy
 from scipy import ndimage, linalg
 
+from .image import crop_img
 from .. import _utils
-from .._utils.compat import _basestring
+from .._utils.niimg import _get_data
 
 ###############################################################################
 # Affine utils
@@ -32,13 +33,13 @@ def to_matrix_vector(transform):
 
     Parameters
     ----------
-    transform: numpy.ndarray
+    transform : numpy.ndarray
         Homogeneous transform matrix. Example: a (4, 4) transform representing
         linear transformation and translation in 3 dimensions.
 
     Returns
     -------
-    matrix, vector: numpy.ndarray
+    matrix, vector : numpy.ndarray
         The matrix and vector components of the transform matrix.  For
         an (N, N) transform, matrix will be (N-1, N-1) and vector will be
         a 1D array of shape (N-1,).
@@ -46,8 +47,8 @@ def to_matrix_vector(transform):
     See Also
     --------
     from_matrix_vector
-    """
 
+    """
     ndimin = transform.shape[0] - 1
     ndimout = transform.shape[1] - 1
     matrix = transform[0:ndimin, 0:ndimout]
@@ -63,10 +64,10 @@ def from_matrix_vector(matrix, vector):
 
     Parameters
     ----------
-    matrix: numpy.ndarray
+    matrix : numpy.ndarray
         An (N, N) array representing the rotation matrix.
 
-    vector: numpy.ndarray
+    vector : numpy.ndarray
         A (1, N) array representing the translation.
 
     Returns
@@ -77,8 +78,8 @@ def from_matrix_vector(matrix, vector):
     See Also
     --------
     nilearn.resampling.to_matrix_vector
-    """
 
+    """
     nin, nout = matrix.shape
     t = np.zeros((nin + 1, nout + 1), matrix.dtype)
     t[0:nin, 0:nout] = matrix
@@ -94,22 +95,27 @@ def coord_transform(x, y, z, affine):
     Parameters
     ----------
     x : number or ndarray (any shape)
-        The x coordinates in the input space
+        The x coordinates in the input space.
+
     y : number or ndarray (same shape as x)
-        The y coordinates in the input space
+        The y coordinates in the input space.
+
     z : number or ndarray
-        The z coordinates in the input space
+        The z coordinates in the input space.
+
     affine : 2D 4x4 ndarray
-        affine that maps from input to output space.
+        Affine that maps from input to output space.
 
     Returns
     -------
     x : number or ndarray (same shape as input)
-        The x coordinates in the output space
+        The x coordinates in the output space.
+
     y : number or ndarray (same shape as input)
-        The y coordinates in the output space
+        The y coordinates in the output space.
+
     z : number or ndarray (same shape as input)
-        The z coordinates in the output space
+        The z coordinates in the output space.
 
     Warning: The x, y and z have their output space (e.g. MNI) coordinate
     ordering, not 3D numpy image ordering.
@@ -124,7 +130,7 @@ def coord_transform(x, y, z, affine):
         >>> niimg = datasets.load_mni152_template()
         >>> # Find the MNI coordinates of the voxel (50, 50, 50)
         >>> image.coord_transform(50, 50, 50, niimg.affine)
-        (-10.0, -26.0, 28.0)
+        (2.0, -34.0, 28.0)
 
     """
     squeeze = (not hasattr(x, '__iter__'))
@@ -150,18 +156,19 @@ def get_bounds(shape, affine):
 
     Parameters
     ----------
-    shape: tuple
+    shape : tuple
         shape of the array. Must have 3 integer values.
 
-    affine: numpy.ndarray
+    affine : numpy.ndarray
         affine giving the linear transformation between voxel coordinates
         and world-space coordinates.
 
     Returns
     -------
-    coord: list of tuples
+    coord : list of tuples
         coord[i] is a 2-tuple giving minimal and maximal coordinates along
         i-th axis.
+
     """
     adim, bdim, cdim = shape
     adim -= 1
@@ -185,20 +192,19 @@ def get_mask_bounds(img):
 
         Parameters
         ----------
-        img: Niimg-like object
+        img : Niimg-like object
             See http://nilearn.github.io/manipulating_images/input_output.html
             The image to inspect. Zero values are considered as
             background.
 
         Returns
         --------
-        xmin, xmax, ymin, ymax, zmin, zmax: floats
+        xmin, xmax, ymin, ymax, zmin, zmax : floats
             The world-space bounds (field of view) occupied by the
             non-zero values in the image
 
         Notes
         -----
-
         The image should have only one connect component.
 
         The affine should be diagonal or diagonal-permuted, use
@@ -206,7 +212,7 @@ def get_mask_bounds(img):
 
     """
     img = _utils.check_niimg_3d(img)
-    mask = _utils.numpy_conversions._asarray(img.get_data(), dtype=np.bool)
+    mask = _utils.numpy_conversions._asarray(_get_data(img), dtype=bool)
     affine = img.affine
     (xmin, xmax), (ymin, ymax), (zmin, zmax) = get_bounds(mask.shape, affine)
     slices = ndimage.find_objects(mask)
@@ -261,6 +267,14 @@ def _resample_one_img(data, A, b, target_shape,
         data = _extrapolate_out_mask(data, np.logical_not(not_finite),
                                      iterations=2)[0]
 
+    # If data is binary and interpolation is continuous or linear,
+    # warn the user as this might be unintentional
+    if sorted(list(np.unique(data))) == [0,1] and interpolation_order != 0:
+        warnings.warn("Resampling binary images with continuous or "
+                      "linear interpolation. This might lead to "
+                      "unexpected results. You might consider using "
+                      "nearest interpolation instead.")
+
     # Suppresses warnings in https://github.com/nilearn/nilearn/issues/1363
     with warnings.catch_warnings():
         if LooseVersion(scipy.__version__) >= LooseVersion('0.18'):
@@ -289,51 +303,57 @@ def _resample_one_img(data, A, b, target_shape,
 
 def resample_img(img, target_affine=None, target_shape=None,
                  interpolation='continuous', copy=True, order="F",
-                 clip=True, fill_value=0):
+                 clip=True, fill_value=0, force_resample=False):
     """Resample a Niimg-like object
 
     Parameters
     ----------
-    img: Niimg-like object
+    img : Niimg-like object
         See http://nilearn.github.io/manipulating_images/input_output.html
         Image(s) to resample.
 
-    target_affine: numpy.ndarray, optional
+    target_affine : numpy.ndarray, optional
         If specified, the image is resampled corresponding to this new affine.
         target_affine can be a 3x3 or a 4x4 matrix. (See notes)
 
-    target_shape: tuple or list, optional
+    target_shape : tuple or list, optional
         If specified, the image will be resized to match this new shape.
         len(target_shape) must be equal to 3.
         If target_shape is specified, a target_affine of shape (4, 4)
         must also be given. (See notes)
 
-    interpolation: str, optional
-        Can be 'continuous' (default), 'linear', or 'nearest'. Indicates the resample
-        method.
+    interpolation : str, optional
+        Can be 'continuous', 'linear', or 'nearest'. Indicates the resample
+        method. Default='continuous'.
 
-    copy: bool, optional
+    copy : bool, optional
         If True, guarantees that output array has no memory in common with
         input array.
         In all cases, input images are never modified by this function.
+        Default=True.
 
-    order: "F" or "C"
+    order : "F" or "C", optional
         Data ordering in output array. This function is slightly faster with
-        Fortran ordering.
+        Fortran ordering. Default='F'.
 
-    clip: bool, optional
+    clip : bool, optional
         If True (default) all resampled image values above max(img) and
         under min(img) are clipped to min(img) and max(img). Note that
         0 is added as an image value for clipping, and it is the padding
         value when extrapolating out of field of view.
-        If False no clip is preformed.
+        If False no clip is performed.
+        Default=True.
 
-    fill_value: float, optional
-        Use a fill value for points outside of input volume (default 0).
+    fill_value : float, optional
+        Use a fill value for points outside of input volume. Default=0.
+
+    force_resample : bool, optional
+        Intended for testing, this prevents the use of a padding optimzation.
+        Default=False.
 
     Returns
     -------
-    resampled: nibabel.Nifti1Image
+    resampled : nibabel.Nifti1Image
         input image, resampled to have respectively target_shape and
         target_affine as shape and affine.
 
@@ -343,7 +363,6 @@ def resample_img(img, target_affine=None, target_shape=None,
 
     Notes
     -----
-
     **BoundingBoxError**
     If a 4x4 transformation matrix (target_affine) is given and all of the
     transformed data points have a negative voxel index along one of the
@@ -377,10 +396,10 @@ def resample_img(img, target_affine=None, target_shape=None,
     **Handling non-native endian in given Nifti images**
     This function automatically changes the byte-ordering information
     in the image dtype to new byte order. From non-native to native, which
-    implies that if the given image has non-native endianess then the output
+    implies that if the given image has non-native endianness then the output
     data in Nifti image will have native dtype. This is only the case when
     if the given target_affine (transformation matrix) is diagonal and
-    homogenous.
+    homogeneous.
 
     """
     from .image import new_img_like  # avoid circular imports
@@ -411,25 +430,30 @@ def resample_img(img, target_affine=None, target_shape=None,
                    "or 'nearest' but it was set to '{0}'").format(interpolation)
         raise ValueError(message)
 
-    if isinstance(img, _basestring):
+    if isinstance(img, str):
         # Avoid a useless copy
         input_img_is_string = True
     else:
         input_img_is_string = False
 
     img = _utils.check_niimg(img)
+    shape = img.shape
+    affine = img.affine
 
     # noop cases
     if target_affine is None and target_shape is None:
         if copy and not input_img_is_string:
             img = _utils.copy_img(img)
         return img
+    if (
+        np.shape(target_affine) == np.shape(affine)
+        and np.allclose(target_affine, affine)
+        and np.array_equal(target_shape, shape)
+    ):
+        return img
     if target_affine is not None:
         target_affine = np.asarray(target_affine)
 
-    shape = img.shape
-    affine = img.affine
-
     if (np.all(np.array(target_shape) == shape[:3]) and
             np.allclose(target_affine, affine)):
         if copy and not input_img_is_string:
@@ -439,7 +463,7 @@ def resample_img(img, target_affine=None, target_shape=None,
     # We now know that some resampling must be done.
     # The value of "copy" is of no importance: output is always a separate
     # array.
-    data = img.get_data()
+    data = _get_data(img)
 
     # Get a bounding box for the transformed data
     # Embed target_affine in 4x4 shape if necessary
@@ -486,14 +510,6 @@ def resample_img(img, target_affine=None, target_shape=None,
     else:
         transform_affine = np.dot(linalg.inv(affine), target_affine)
     A, b = to_matrix_vector(transform_affine)
-    # If A is diagonal, ndimage.affine_transform is clever enough to use a
-    # better algorithm.
-    if np.all(np.diag(np.diag(A)) == A):
-        if LooseVersion(scipy.__version__) < LooseVersion('0.18'):
-            # Before scipy 0.18, ndimage.affine_transform was applying a
-            # different logic to the offset for diagonal affine
-            b = np.dot(linalg.inv(A), b)
-        A = np.diag(A)
 
     data_shape = list(data.shape)
     # Make sure that we have a list here
@@ -501,6 +517,11 @@ def resample_img(img, target_affine=None, target_shape=None,
         target_shape = target_shape.tolist()
     target_shape = tuple(target_shape)
 
+    if LooseVersion(scipy.__version__) < LooseVersion('0.20'):
+        # Before scipy 0.20, force native data types due to endian issues
+        # that caused instability.
+        data = data.astype(data.dtype.newbyteorder('N'))
+
     if interpolation == 'continuous' and data.dtype.kind == 'i':
         # cast unsupported data types to closest support dtype
         aux = data.dtype.name.replace('int', 'float')
@@ -514,11 +535,11 @@ def resample_img(img, target_affine=None, target_shape=None,
 
     # Since the release of 0.17, resampling nifti images have some issues
     # when affine is passed as 1D array and if data is of non-native
-    # endianess.
+    # endianness.
     # See issue https://github.com/nilearn/nilearn/issues/1445.
     # If affine is passed as 1D, scipy uses _nd_image.zoom_shift rather
     # than _geometric_transform (2D) where _geometric_transform is able
-    # to swap byte order in scipy later than 0.15 for nonnative endianess.
+    # to swap byte order in scipy later than 0.15 for nonnative endianness.
 
     # We convert to 'native' order to not have any issues either with
     # 'little' or 'big' endian data dtypes (non-native endians).
@@ -527,20 +548,58 @@ def resample_img(img, target_affine=None, target_shape=None,
 
     # Code is generic enough to work for both 3D and 4D images
     other_shape = data_shape[3:]
-    resampled_data = np.empty(list(target_shape) + other_shape,
+    resampled_data = np.zeros(list(target_shape) + other_shape,
                               order=order, dtype=resampled_data_dtype)
 
     all_img = (slice(None), ) * 3
 
-    # Iterate over a set of 3D volumes, as the interpolation problem is
-    # separable in the extra dimensions. This reduces the
-    # computational cost
-    for ind in np.ndindex(*other_shape):
-        _resample_one_img(data[all_img + ind], A, b, target_shape,
-                          interpolation_order,
-                          out=resampled_data[all_img + ind],
-                          copy=not input_img_is_string,
-                          fill_value=fill_value)
+    # if (A == I OR some combination of permutation(I) and sign-flipped(I)) AND
+    # all(b == integers):
+    if (np.all(np.eye(3) == A) and all(bt == np.round(bt) for bt in b) and
+        not force_resample):
+        # TODO: also check for sign flips
+        # TODO: also check for permutations of I
+
+        # ... special case: can be solved with padding alone
+        # crop source image and keep N voxels offset before/after volume
+        cropped_img, offsets = crop_img(img, pad=False, return_offset=True)
+
+        # TODO: flip axes that are flipped
+        # TODO: un-shuffle permuted dimensions
+
+        # offset the original un-cropped image indices by the relative
+        # translation, b.
+        indices = [(int(off.start - dim_b), int(off.stop - dim_b))
+                   for off, dim_b in zip(offsets[:3], b[:3])]
+
+        # If image are not fully overlapping, place only portion of image.
+        slices = []
+        for dimsize, index in zip(resampled_data.shape, indices):
+            slices.append(slice(np.max((0, index[0])),
+                                np.min((dimsize, index[1]))))
+        slices = tuple(slices)
+
+        # ensure the source image being placed isn't larger than the dest
+        subset_indices = tuple(slice(0, s.stop-s.start) for s in slices)
+        resampled_data[slices] = _get_data(cropped_img)[subset_indices]
+    else:
+        # If A is diagonal, ndimage.affine_transform is clever enough to use a
+        # better algorithm.
+        if np.all(np.diag(np.diag(A)) == A):
+            if LooseVersion(scipy.__version__) < LooseVersion('0.18'):
+                # Before scipy 0.18, ndimage.affine_transform was applying a
+                # different logic to the offset for diagonal affine
+                b = np.dot(linalg.inv(A), b)
+            A = np.diag(A)
+        # Iterate over a set of 3D volumes, as the interpolation problem is
+        # separable in the extra dimensions. This reduces the
+        # computational cost
+        for ind in np.ndindex(*other_shape):
+            _resample_one_img(data[all_img + ind], A, b, target_shape,
+                              interpolation_order,
+                              out=resampled_data[all_img + ind],
+                              copy=not input_img_is_string,
+                              fill_value=fill_value)
 
     if clip:
         # force resampled data to have a range contained in the original data
@@ -556,7 +615,7 @@ def resample_img(img, target_affine=None, target_shape=None,
 
 def resample_to_img(source_img, target_img,
                     interpolation='continuous', copy=True, order='F',
-                    clip=False, fill_value=0):
+                    clip=False, fill_value=0, force_resample=False):
     """Resample a Niimg-like source image on a target Niimg-like image
     (no registration is performed: the image should already be aligned).
 
@@ -564,34 +623,39 @@ def resample_to_img(source_img, target_img,
 
     Parameters
     ----------
-    source_img: Niimg-like object
+    source_img : Niimg-like object
         See http://nilearn.github.io/manipulating_images/input_output.html
         Image(s) to resample.
 
-    target_img: Niimg-like object
+    target_img : Niimg-like object
         See http://nilearn.github.io/manipulating_images/input_output.html
         Reference image taken for resampling.
 
-    interpolation: str, optional
-        Can be 'continuous' (default), 'linear', or 'nearest'. Indicates the resample
-        method.
+    interpolation : str, optional
+        Can be 'continuous', 'linear', or 'nearest'. Indicates the resample
+        method. Default='continuous'.
 
-    copy: bool, optional
+    copy : bool, optional
         If True, guarantees that output array has no memory in common with
         input array.
         In all cases, input images are never modified by this function.
+        Default=True.
 
-    order: "F" or "C"
+    order : "F" or "C", optional
         Data ordering in output array. This function is slightly faster with
-        Fortran ordering.
+        Fortran ordering. Default="F".
 
-    clip: bool, optional
-        If False (default) no clip is preformed.
+    clip : bool, optional
+        If False (default) no clip is performed.
         If True all resampled image values above max(img) and under min(img) are
-        clipped to min(img) and max(img)
+        clipped to min(img) and max(img). Default=False.
 
-    fill_value: float, optional
-        Use a fill value for points outside of input volume (default 0).
+    fill_value : float, optional
+        Use a fill value for points outside of input volume. Default=0.
+
+    force_resample : bool, optional
+        Intended for testing, this prevents the use of a padding optimzation.
+        Default=False.
 
     Returns
     -------
@@ -602,8 +666,8 @@ def resample_to_img(source_img, target_img,
     See Also
     --------
     nilearn.image.resample_img
-    """
 
+    """
     target = _utils.check_niimg(target_img)
     target_shape = target.shape
 
@@ -616,7 +680,8 @@ def resample_to_img(source_img, target_img,
                         target_affine=target.affine,
                         target_shape=target_shape,
                         interpolation=interpolation, copy=copy, order=order,
-                        clip=clip, fill_value=fill_value)
+                        clip=clip, fill_value=fill_value,
+                        force_resample=force_resample)
 
 
 def reorder_img(img, resample=None):
@@ -625,18 +690,18 @@ def reorder_img(img, resample=None):
     If it is impossible to get xyz ordering by permuting the axes, a
     'ValueError' is raised.
 
-        Parameters
-        -----------
-        img: Niimg-like object
-            See http://nilearn.github.io/manipulating_images/input_output.html
-            Image to reorder.
+    Parameters
+    -----------
+    img : Niimg-like object
+        See http://nilearn.github.io/manipulating_images/input_output.html
+        Image to reorder.
 
-        resample: None or string in {'continuous', 'linear', 'nearest'}, optional
-            If resample is None (default), no resampling is performed, the
-            axes are only permuted.
-            Otherwise resampling is performed and 'resample' will
-            be passed as the 'interpolation' argument into
-            resample_img.
+    resample : None or string in {'continuous', 'linear', 'nearest'}, optional
+        If resample is None (default), no resampling is performed, the
+        axes are only permuted.
+        Otherwise resampling is performed and 'resample' will
+        be passed as the 'interpolation' argument into
+        resample_img.
 
     """
     from .image import new_img_like
@@ -662,7 +727,7 @@ def reorder_img(img, resample=None):
                                 interpolation=resample)
 
     axis_numbers = np.argmax(np.abs(A), axis=0)
-    data = img.get_data()
+    data = _get_data(img)
     while not np.all(np.sort(axis_numbers) == axis_numbers):
         first_inversion = np.argmax(np.diff(axis_numbers)<0)
         axis1 = first_inversion + 1
