@@ -1,17 +1,17 @@
 """
 Test image pre-processing functions
 """
-from nose.tools import assert_true, assert_false, assert_equal, assert_raises
-from nose import SkipTest
-
 import platform
 import os
 import sys
+import tempfile
+
 import nibabel
 from nibabel import Nifti1Image
 import numpy as np
+import pytest
+
 from numpy.testing import assert_array_equal, assert_allclose
-from nilearn._utils.testing import assert_raises_regex, assert_warns
 from nilearn._utils.exceptions import DimensionError
 
 from nilearn import signal
@@ -21,9 +21,11 @@ from nilearn.image import concat_imgs
 from nilearn._utils import testing, niimg_conversions, data_gen
 from nilearn.image import new_img_like
 from nilearn.image import threshold_img
-from nilearn.image import iter_img
+from nilearn.image import iter_img, index_img
 from nilearn.image import math_img
+from nilearn.image import binarize_img
 from nilearn.image import largest_connected_component_img
+from nilearn.image import get_data
 
 try:
     import pandas as pd
@@ -36,6 +38,25 @@ currdir = os.path.dirname(os.path.abspath(__file__))
 datadir = os.path.join(currdir, 'data')
 
 
+def test_get_data():
+    img, *_ = data_gen.generate_fake_fmri(shape=(10, 11, 12))
+    data = get_data(img)
+    assert data.shape == img.shape
+    assert data is img._data_cache
+    mask_img = new_img_like(img, data > 0)
+    data = get_data(mask_img)
+    assert data.dtype == np.dtype('int8')
+    img_3d = index_img(img, 0)
+    with tempfile.TemporaryDirectory() as tempdir:
+        filename = os.path.join(tempdir, 'img_{}.nii.gz')
+        img_3d.to_filename(filename.format('a'))
+        img_3d.to_filename(filename.format('b'))
+        data = get_data(filename.format('a'))
+        assert len(data.shape) == 3
+        data = get_data(filename.format('*'))
+        assert len(data.shape) == 4
+
+
 def test_high_variance_confounds():
     # See also test_signals.test_high_variance_confounds()
     # There is only tests on what is added by image.high_variance_confounds()
@@ -50,12 +71,12 @@ def test_high_variance_confounds():
     confounds1 = image.high_variance_confounds(img, mask_img=mask_img,
                                                percentile=10.,
                                                n_confounds=n_confounds)
-    assert_true(confounds1.shape == (length, n_confounds))
+    assert confounds1.shape == (length, n_confounds)
 
     # No mask.
     confounds2 = image.high_variance_confounds(img, percentile=10.,
                                                n_confounds=n_confounds)
-    assert_true(confounds2.shape == (length, n_confounds))
+    assert confounds2.shape == (length, n_confounds)
 
 
 def test__fast_smooth_array():
@@ -94,7 +115,7 @@ def test__smooth_array():
     for affine in test_affines:
         filtered = image._smooth_array(data, affine,
                                          fwhm=fwhm, copy=True)
-        assert_false(np.may_share_memory(filtered, data))
+        assert not np.may_share_memory(filtered, data)
 
         # We are expecting a full-width at half maximum of
         # fwhm / voxel_size:
@@ -110,7 +131,7 @@ def test__smooth_array():
     data[10, 10, 10] = np.NaN
     filtered = image._smooth_array(data, affine, fwhm=fwhm,
                                    ensure_finite=True, copy=True)
-    assert_true(np.all(np.isfinite(filtered)))
+    assert np.all(np.isfinite(filtered))
 
     # Check copy=False.
     for affine in test_affines:
@@ -135,7 +156,8 @@ def test__smooth_array():
 
     # Check corner case when fwhm=0. See #1537
     # Test whether function _smooth_array raises a warning when fwhm=0.
-    assert_warns(UserWarning, image._smooth_array, data, affine, fwhm=0.)
+    with pytest.warns(UserWarning):
+        image._smooth_array(data, affine, fwhm=0.)
 
     # Test output equal when fwhm=None and fwhm=0
     out_fwhm_none = image._smooth_array(data, affine, fwhm=None)
@@ -160,24 +182,25 @@ def test_smooth_img():
                                     create_files=create_files) as imgs:
             # List of images as input
             out = image.smooth_img(imgs, fwhm)
-            assert_true(isinstance(out, list))
-            assert_true(len(out) == 2)
+            assert isinstance(out, list)
+            assert len(out) == 2
             for o, s, l in zip(out, shapes, lengths):
-                assert_true(o.shape == (s + (l,)))
+                assert o.shape == (s + (l,))
 
             # Single image as input
             out = image.smooth_img(imgs[0], fwhm)
-            assert_true(isinstance(out, nibabel.Nifti1Image))
-            assert_true(out.shape == (shapes[0] + (lengths[0],)))
+            assert isinstance(out, nibabel.Nifti1Image)
+            assert out.shape == (shapes[0] + (lengths[0],))
 
     # Check corner case situations when fwhm=0, See issue #1537
     # Test whether function smooth_img raises a warning when fwhm=0.
-    assert_warns(UserWarning, image.smooth_img, img1, fwhm=0.)
+    with pytest.warns(UserWarning):
+        image.smooth_img(img1, fwhm=0.)
 
     # Test output equal when fwhm=None and fwhm=0
     out_fwhm_none = image.smooth_img(img1, fwhm=None)
     out_fwhm_zero = image.smooth_img(img1, fwhm=0.)
-    assert_array_equal(out_fwhm_none.get_data(), out_fwhm_zero.get_data())
+    assert_array_equal(get_data(out_fwhm_none), get_data(out_fwhm_zero))
 
     data1 = np.zeros((10, 11, 12))
     data1[2:4, 1:5, 3:6] = 1
@@ -200,20 +223,20 @@ def test__crop_img_to():
     new_origin = np.array((4, 3, 2)) * np.array((2, 1, 3))
 
     # check that correct part was extracted:
-    assert_true((cropped_img.get_data() == 1).all())
-    assert_true(cropped_img.shape == (2, 4, 3))
+    assert (get_data(cropped_img) == 1).all()
+    assert cropped_img.shape == (2, 4, 3)
 
     # check that affine was adjusted correctly
-    assert_true((cropped_img.affine[:3, 3] == new_origin).all())
+    assert (cropped_img.affine[:3, 3] == new_origin).all()
 
     # check that data was really not copied
     data[2:4, 1:5, 3:6] = 2
-    assert_true((cropped_img.get_data() == 2).all())
+    assert (get_data(cropped_img) == 2).all()
 
     # check that copying works
     copied_cropped_img = image._crop_img_to(img, slices)
     data[2:4, 1:5, 3:6] = 1
-    assert_true((copied_cropped_img.get_data() == 2).all())
+    assert (get_data(copied_cropped_img) == 2).all()
 
 
 def test_crop_img():
@@ -229,8 +252,8 @@ def test_crop_img():
 
     # check that correct part was extracted:
     # This also corrects for padding
-    assert_true((cropped_img.get_data()[1:-1, 1:-1, 1:-1] == 1).all())
-    assert_true(cropped_img.shape == (2 + 2, 4 + 2, 3 + 2))
+    assert (get_data(cropped_img)[1:-1, 1:-1, 1:-1] == 1).all()
+    assert cropped_img.shape == (2 + 2, 4 + 2, 3 + 2)
 
 
 def test_crop_threshold_tolerance():
@@ -247,14 +270,14 @@ def test_crop_threshold_tolerance():
     img = nibabel.Nifti1Image(data, affine=affine)
 
     cropped_img = image.crop_img(img)
-    assert_true(cropped_img.shape == active_shape)
+    assert cropped_img.shape == active_shape
 
 
 def test_mean_img():
     rng = np.random.RandomState(42)
     data1 = np.zeros((5, 6, 7))
-    data2 = rng.rand(5, 6, 7)
-    data3 = rng.rand(5, 6, 7, 3)
+    data2 = rng.uniform(size=(5, 6, 7))
+    data3 = rng.uniform(size=(5, 6, 7, 3))
     affine = np.diag((4, 3, 2, 1))
     img1 = nibabel.Nifti1Image(data1, affine=affine)
     img2 = nibabel.Nifti1Image(data2, affine=affine)
@@ -268,7 +291,7 @@ def test_mean_img():
         arrays = list()
         # Ground-truth:
         for img in imgs:
-            img = img.get_data()
+            img = get_data(img)
             if img.ndim == 4:
                 img = np.mean(img, axis=-1)
             arrays.append(img)
@@ -276,20 +299,20 @@ def test_mean_img():
 
         mean_img = image.mean_img(imgs)
         assert_array_equal(mean_img.affine, affine)
-        assert_array_equal(mean_img.get_data(), truth)
+        assert_array_equal(get_data(mean_img), truth)
 
         # Test with files
         with testing.write_tmp_imgs(*imgs) as imgs:
             mean_img = image.mean_img(imgs)
             assert_array_equal(mean_img.affine, affine)
             if X64:
-                assert_array_equal(mean_img.get_data(), truth)
+                assert_array_equal(get_data(mean_img), truth)
             else:
                 # We don't really understand but arrays are not
                 # exactly equal on 32bit. Given that you can not do
                 # much real world data analysis with nilearn on a
                 # 32bit machine it is not worth investigating more
-                assert_allclose(mean_img.get_data(), truth,
+                assert_allclose(get_data(mean_img), truth,
                                 rtol=np.finfo(truth.dtype).resolution,
                                 atol=0)
 
@@ -297,7 +320,7 @@ def test_mean_img():
 def test_mean_img_resample():
     # Test resampling in mean_img with a permutation of the axes
     rng = np.random.RandomState(42)
-    data = rng.rand(5, 6, 7, 40)
+    data = rng.uniform(size=(5, 6, 7, 40))
     affine = np.diag((4, 3, 2, 1))
     img = nibabel.Nifti1Image(data, affine=affine)
     mean_img = nibabel.Nifti1Image(data.mean(axis=-1), affine=affine)
@@ -307,41 +330,43 @@ def test_mean_img_resample():
                                               target_affine=target_affine)
     resampled_mean_image = resampling.resample_img(mean_img,
                                               target_affine=target_affine)
-    assert_array_equal(resampled_mean_image.get_data(),
-                       mean_img_with_resampling.get_data())
+    assert_array_equal(get_data(resampled_mean_image),
+                       get_data(mean_img_with_resampling))
     assert_array_equal(resampled_mean_image.affine,
                        mean_img_with_resampling.affine)
     assert_array_equal(mean_img_with_resampling.affine, target_affine)
 
 
 def test_swap_img_hemispheres():
+    rng = np.random.RandomState(42)
+
     # make sure input image data is not overwritten inside function
-    data = np.random.randn(4, 5, 7)
+    data = rng.standard_normal(size=(4, 5, 7))
     data_img = nibabel.Nifti1Image(data, np.eye(4))
     image.swap_img_hemispheres(data_img)
-    np.testing.assert_array_equal(data_img.get_data(), data)
+    np.testing.assert_array_equal(get_data(data_img), data)
 
     # swapping operations work
     np.testing.assert_array_equal(  # one turn
-        image.swap_img_hemispheres(data_img).get_data(),
+        get_data(image.swap_img_hemispheres(data_img)),
         data[::-1])
     np.testing.assert_array_equal(  # two turns -> back to original data
-        image.swap_img_hemispheres(
-            image.swap_img_hemispheres(data_img)).get_data(),
+        get_data(image.swap_img_hemispheres(
+            image.swap_img_hemispheres(data_img))),
         data)
 
 
 def test_concat_imgs():
-    assert_true(concat_imgs is niimg_conversions.concat_niimgs)
+    assert concat_imgs is niimg_conversions.concat_niimgs
 
 
 def test_index_img():
     img_3d = nibabel.Nifti1Image(np.ones((3, 4, 5)), np.eye(4))
-    testing.assert_raises_regex(TypeError,
-                                "Input data has incompatible dimensionality: "
-                                "Expected dimension is 4D and you provided "
-                                "a 3D image.",
-                                image.index_img, img_3d, 0)
+    expected_error_msg = ("Input data has incompatible dimensionality: "
+                          "Expected dimension is 4D and you provided "
+                          "a 3D image.")
+    with pytest.raises(TypeError, match=expected_error_msg):
+        image.index_img(img_3d, 0)
 
     affine = np.array([[1., 2., 3., 4.],
                        [5., 6., 7., 8.],
@@ -355,24 +380,25 @@ def test_index_img():
                        (np.arange(fourth_dim_size) % 3) == 1])
     for i in tested_indices:
         this_img = image.index_img(img_4d, i)
-        expected_data_3d = img_4d.get_data()[..., i]
-        assert_array_equal(this_img.get_data(),
+        expected_data_3d = get_data(img_4d)[..., i]
+        assert_array_equal(get_data(this_img),
                            expected_data_3d)
         assert_array_equal(this_img.affine, img_4d.affine)
 
     for i in [fourth_dim_size, - fourth_dim_size - 1,
               [0, fourth_dim_size],
               np.repeat(True, fourth_dim_size + 1)]:
-        testing.assert_raises_regex(
-            IndexError,
-            'out of bounds|invalid index|out of range|boolean index',
-            image.index_img, img_4d, i)
+        with pytest.raises(
+                IndexError,
+                match='out of bounds|invalid index|out of range|'
+                      'boolean index'):
+            image.index_img(img_4d, i)
 
 
 def test_pd_index_img():
     # confirm indices from pandas dataframes are handled correctly
     if 'pandas' not in sys.modules:
-        raise SkipTest
+        raise pytest.skip(msg='Pandas not available')
 
     affine = np.array([[1., 2., 3., 4.],
                        [5., 6., 7., 8.],
@@ -382,24 +408,23 @@ def test_pd_index_img():
 
     fourth_dim_size = img_4d.shape[3]
 
-    rng = np.random.RandomState(0)
-
-    arr = rng.rand(fourth_dim_size) > 0.5
+    rng = np.random.RandomState(42)
+    arr = rng.uniform(size=fourth_dim_size) > 0.5
     df = pd.DataFrame({"arr": arr})
 
     np_index_img = image.index_img(img_4d, arr)
     pd_index_img = image.index_img(img_4d, df)
-    assert_array_equal(np_index_img.get_data(),
-                       pd_index_img.get_data())
+    assert_array_equal(get_data(np_index_img),
+                       get_data(pd_index_img))
 
 
 def test_iter_img():
     img_3d = nibabel.Nifti1Image(np.ones((3, 4, 5)), np.eye(4))
-    testing.assert_raises_regex(TypeError,
-                                "Input data has incompatible dimensionality: "
-                                "Expected dimension is 4D and you provided "
-                                "a 3D image.",
-                                image.iter_img, img_3d)
+    expected_error_msg = ("Input data has incompatible dimensionality: "
+                          "Expected dimension is 4D and you provided "
+                          "a 3D image.")
+    with pytest.raises(TypeError, match=expected_error_msg):
+        image.iter_img(img_3d)
 
     affine = np.array([[1., 2., 3., 4.],
                        [5., 6., 7., 8.],
@@ -408,15 +433,15 @@ def test_iter_img():
     img_4d, _ = data_gen.generate_fake_fmri(affine=affine)
 
     for i, img in enumerate(image.iter_img(img_4d)):
-        expected_data_3d = img_4d.get_data()[..., i]
-        assert_array_equal(img.get_data(),
+        expected_data_3d = get_data(img_4d)[..., i]
+        assert_array_equal(get_data(img),
                            expected_data_3d)
         assert_array_equal(img.affine, img_4d.affine)
 
     with testing.write_tmp_imgs(img_4d) as img_4d_filename:
         for i, img in enumerate(image.iter_img(img_4d_filename)):
-            expected_data_3d = img_4d.get_data()[..., i]
-            assert_array_equal(img.get_data(),
+            expected_data_3d = get_data(img_4d)[..., i]
+            assert_array_equal(get_data(img),
                                expected_data_3d)
             assert_array_equal(img.affine, img_4d.affine)
         # enables to delete "img_4d_filename" on windows
@@ -424,15 +449,15 @@ def test_iter_img():
 
     img_3d_list = list(image.iter_img(img_4d))
     for i, img in enumerate(image.iter_img(img_3d_list)):
-        expected_data_3d = img_4d.get_data()[..., i]
-        assert_array_equal(img.get_data(),
+        expected_data_3d = get_data(img_4d)[..., i]
+        assert_array_equal(get_data(img),
                            expected_data_3d)
         assert_array_equal(img.affine, img_4d.affine)
 
     with testing.write_tmp_imgs(*img_3d_list) as img_3d_filenames:
         for i, img in enumerate(image.iter_img(img_3d_filenames)):
-            expected_data_3d = img_4d.get_data()[..., i]
-            assert_array_equal(img.get_data(),
+            expected_data_3d = get_data(img_4d)[..., i]
+            assert_array_equal(get_data(img),
                                expected_data_3d)
             assert_array_equal(img.affine, img_4d.affine)
         # enables to delete "img_3d_filename" on windows
@@ -446,7 +471,7 @@ def test_new_img_like_mgz():
     """
 
     ref_img = nibabel.load(os.path.join(datadir, 'test.mgz'))
-    data = np.ones(ref_img.get_data().shape, dtype=np.bool)
+    data = np.ones(get_data(ref_img).shape, dtype=bool)
     affine = ref_img.affine
     new_img_like(ref_img, data, affine, copy_header=False)
 
@@ -458,12 +483,27 @@ def test_new_img_like():
     affine = np.diag((4, 3, 2, 1))
     img = nibabel.Nifti1Image(data, affine=affine)
     img2 = new_img_like([img, ], data)
-    np.testing.assert_array_equal(img.get_data(), img2.get_data())
+    np.testing.assert_array_equal(get_data(img), get_data(img2))
 
     # test_new_img_like_with_nifti2image_copy_header
     img_nifti2 = nibabel.Nifti2Image(data, affine=affine)
     img2_nifti2 = new_img_like([img_nifti2, ], data, copy_header=True)
-    np.testing.assert_array_equal(img_nifti2.get_data(), img2_nifti2.get_data())
+    np.testing.assert_array_equal(get_data(img_nifti2), get_data(img2_nifti2))
+
+
+def test_new_img_like_non_iterable_header():
+    """
+    Tests that when an niimg's header is not iterable
+    & it is set to be copied, an error is not raised.
+    """
+    rng = np.random.RandomState(42)
+    fake_fmri_data = rng.uniform(size=(10, 10, 10, 10))
+    fake_affine = rng.uniform(size=(4, 4))
+    fake_spatial_image = nibabel.spatialimages.SpatialImage(fake_fmri_data,
+                                                            fake_affine)
+    assert new_img_like(fake_spatial_image,
+                        data=fake_fmri_data,
+                        copy_header=True)
 
 
 def test_validity_threshold_value_in_threshold_img():
@@ -471,17 +511,17 @@ def test_validity_threshold_value_in_threshold_img():
     maps, _ = data_gen.generate_maps(shape, n_regions=2)
 
     # testing to raise same error when threshold=None case
-    testing.assert_raises_regex(ValueError,
-                                "The input parameter 'threshold' is empty. ",
-                                threshold_img, maps, threshold=None)
+    with pytest.raises(ValueError,
+                       match="The input parameter 'threshold' is empty. "):
+        threshold_img(maps, threshold=None)
 
     invalid_threshold_values = ['90t%', 's%', 't', '0.1']
     name = 'threshold'
     for thr in invalid_threshold_values:
-        testing.assert_raises_regex(ValueError,
-                                    '{0}.+should be a number followed by '
-                                    'the percent sign'.format(name),
-                                    threshold_img, maps, threshold=thr)
+        with pytest.raises(ValueError,
+                           match='{0}.+should be a number followed by '
+                                 'the percent sign'.format(name)):
+            threshold_img(maps, threshold=thr)
 
 
 def test_threshold_img():
@@ -499,11 +539,32 @@ def test_threshold_img():
         # when threshold is a percentile
         thr_maps_percent2 = threshold_img(img, threshold='2%')
 
+def test_threshold_img_copy():
+
+    img_zeros = Nifti1Image(np.zeros((10, 10, 10, 10)), np.eye(4))
+    img_ones = Nifti1Image(np.ones((10, 10, 10, 10)), np.eye(4))
+
+    # Check that copy does not mutate. It returns modified copy.
+    thresholded = threshold_img(img_ones, 2)  # threshold 2 > 1
+
+    # Original img_ones should have all ones.
+    assert_array_equal(get_data(img_ones), np.ones((10, 10, 10, 10)))
+    # Thresholded should have all zeros.
+    assert_array_equal(get_data(thresholded), np.zeros((10, 10, 10, 10)))
+
+    # Check that not copying does mutate.
+    img_to_mutate = Nifti1Image(np.ones((10, 10, 10, 10)), np.eye(4))
+    thresholded = threshold_img(img_to_mutate, 2, copy=False)
+    # Check that original mutates
+    assert_array_equal(get_data(img_to_mutate), np.zeros((10, 10, 10, 10)))
+    # And that returned value is also thresholded.
+    assert_array_equal(get_data(img_to_mutate), get_data(thresholded))
+
 
 def test_isnan_threshold_img_data():
     shape = (10, 10, 10)
     maps, _ = data_gen.generate_maps(shape, n_regions=2)
-    data = maps.get_data()
+    data = get_data(maps)
     data[:, :, 0] = np.nan
 
     maps_img = nibabel.Nifti1Image(data, np.eye(4))
@@ -519,19 +580,17 @@ def test_math_img_exceptions():
 
     formula = "np.mean(img1, axis=-1) - np.mean(img2, axis=-1)"
     # Images with different shapes should raise a ValueError exception.
-    assert_raises_regex(ValueError,
-                        "Input images cannot be compared",
-                        math_img, formula, img1=img1, img2=img2)
+    with pytest.raises(ValueError, match="Input images cannot be compared"):
+        math_img(formula, img1=img1, img2=img2)
 
     # Images with different affines should raise a ValueError exception.
-    assert_raises_regex(ValueError,
-                        "Input images cannot be compared",
-                        math_img, formula, img1=img1, img2=img4)
+    with pytest.raises(ValueError, match="Input images cannot be compared"):
+        math_img(formula, img1=img1, img2=img4)
 
     bad_formula = "np.toto(img1, axis=-1) - np.mean(img3, axis=-1)"
-    assert_raises_regex(AttributeError,
-                        "Input formula couldn't be processed",
-                        math_img, bad_formula, img1=img1, img3=img3)
+    with pytest.raises(AttributeError,
+                       match="Input formula couldn't be processed"):
+        math_img(bad_formula, img1=img1, img3=img3)
 
 
 def test_math_img():
@@ -544,21 +603,36 @@ def test_math_img():
         with testing.write_tmp_imgs(img1, img2,
                                     create_files=create_files) as imgs:
             result = math_img(formula, img1=imgs[0], img2=imgs[1])
-            assert_array_equal(result.get_data(),
-                               expected_result.get_data())
+            assert_array_equal(get_data(result),
+                               get_data(expected_result))
             assert_array_equal(result.affine, expected_result.affine)
-            assert_equal(result.shape, expected_result.shape)
+            assert result.shape == expected_result.shape
+
+
+def test_binarize_img():
+    img = Nifti1Image(np.random.rand(10, 10, 10, 10), np.eye(4))
+    # Test that all output values are 1.
+    img1 = binarize_img(img)
+    np.testing.assert_array_equal(np.unique(img1.dataobj),
+                                  np.array([1]))
+    # Test that it works with threshold
+    img2 = binarize_img(img, threshold=0.5)
+    np.testing.assert_array_equal(np.unique(img2.dataobj),
+                                  np.array([0, 1]))
+    # Test that manual binarization equals binarize_img results.
+    img3 = Nifti1Image(np.random.rand(10, 10, 10, 10), np.eye(4))
+    img3.dataobj[img.dataobj < 0.5] = 0
+    img3.dataobj[img.dataobj >= 0.5] = 1
+    np.testing.assert_array_equal(img2.dataobj, img3.dataobj)
 
 
 def test_clean_img():
-
-    rng = np.random.RandomState(0)
-
-    data = rng.randn(10, 10, 10, 100) + .5
+    rng = np.random.RandomState(42)
+    data = rng.standard_normal(size=(10, 10, 10, 100)) + .5
     data_flat = data.T.reshape(100, -1)
     data_img = nibabel.Nifti1Image(data, np.eye(4))
 
-    assert_raises(
+    pytest.raises(
         ValueError, image.clean_img, data_img, t_r=None, low_pass=0.1)
 
     data_img_ = image.clean_img(
@@ -566,7 +640,7 @@ def test_clean_img():
     data_flat_ = signal.clean(
         data_flat, detrend=True, standardize=False, low_pass=0.1, t_r=1.0)
 
-    np.testing.assert_almost_equal(data_img_.get_data().T.reshape(100, -1),
+    np.testing.assert_almost_equal(get_data(data_img_).T.reshape(100, -1),
                                    data_flat_)
     # if NANs
     data[:, 9, 9] = np.nan
@@ -574,7 +648,7 @@ def test_clean_img():
     data[:, 5, 5] = np.inf
     nan_img = nibabel.Nifti1Image(data, np.eye(4))
     clean_im = image.clean_img(nan_img, ensure_finite=True)
-    assert_true(np.any(np.isfinite(clean_im.get_data())), True)
+    assert np.any(np.isfinite(get_data(clean_im))), True
 
     # test_clean_img_passing_nifti2image
     data_img_nifti2 = nibabel.Nifti2Image(data, np.eye(4))
@@ -588,14 +662,14 @@ def test_clean_img():
 
     # Checks that output with full mask and without is equal
     data_img_ = image.clean_img(img)
-    np.testing.assert_almost_equal(data_img_.get_data(),
-                                   data_img_mask_.get_data())
+    np.testing.assert_almost_equal(get_data(data_img_),
+                                   get_data(data_img_mask_))
 
 
 def test_largest_cc_img():
     """ Check the extraction of the largest connected component, for niftis
 
-    Similiar to smooth_img tests for largest connected_component_img, here also
+    Similar to smooth_img tests for largest connected_component_img, here also
     only the added features for largest_connected_component are tested.
     """
 
@@ -613,24 +687,24 @@ def test_largest_cc_img():
                                     create_files=create_files) as imgs:
             # List of images as input
             out = largest_connected_component_img(imgs)
-            assert_true(isinstance(out, list))
-            assert_true(len(out) == 2)
+            assert isinstance(out, list)
+            assert len(out) == 2
             for o, s in zip(out, shapes):
-                assert_true(o.shape == (s))
+                assert o.shape == (s)
 
             # Single image as input
             out = largest_connected_component_img(imgs[0])
-            assert_true(isinstance(out, Nifti1Image))
-            assert_true(out.shape == (shapes[0]))
+            assert isinstance(out, Nifti1Image)
+            assert out.shape == (shapes[0])
 
         # Test whether 4D Nifti throws the right error.
         img_4D = data_gen.generate_fake_fmri(shapes[0], length=17)
-        assert_raises(DimensionError, largest_connected_component_img, img_4D)
+        pytest.raises(DimensionError, largest_connected_component_img, img_4D)
 
     # tests adapted to non-native endian data dtype
-    img1_change_dtype = nibabel.Nifti1Image(img1.get_data().astype('>f8'),
+    img1_change_dtype = nibabel.Nifti1Image(get_data(img1).astype('>f8'),
                                             affine=img1.affine)
-    img2_change_dtype = nibabel.Nifti1Image(img2.get_data().astype('>f8'),
+    img2_change_dtype = nibabel.Nifti1Image(get_data(img2).astype('>f8'),
                                             affine=img2.affine)
 
     for create_files in (False, True):
@@ -638,21 +712,21 @@ def test_largest_cc_img():
                                     create_files=create_files) as imgs:
             # List of images as input
             out = largest_connected_component_img(imgs)
-            assert_true(isinstance(out, list))
-            assert_true(len(out) == 2)
+            assert isinstance(out, list)
+            assert len(out) == 2
             for o, s in zip(out, shapes):
-                assert_true(o.shape == (s))
+                assert o.shape == (s)
 
             # Single image as input
             out = largest_connected_component_img(imgs[0])
-            assert_true(isinstance(out, Nifti1Image))
-            assert_true(out.shape == (shapes[0]))
+            assert isinstance(out, Nifti1Image)
+            assert out.shape == (shapes[0])
 
     # Test the output with native and without native
     out_native = largest_connected_component_img(img1)
 
     out_non_native = largest_connected_component_img(img1_change_dtype)
-    np.testing.assert_equal(out_native.get_data(), out_non_native.get_data())
+    np.testing.assert_equal(get_data(out_native), get_data(out_non_native))
 
 
 def test_new_img_like_mgh_image():
