@@ -5,20 +5,22 @@ import os
 import copy
 import math
 
-from nose import SkipTest
-from nose.tools import assert_equal, assert_raises, \
-    assert_false, assert_true, assert_almost_equal
-from numpy.testing import assert_array_equal, assert_array_almost_equal
-
+from numpy.testing import (assert_almost_equal,
+                           assert_array_equal,
+                           assert_array_almost_equal)
 import numpy as np
+import pytest
 
 from nibabel import Nifti1Image
 
+from nilearn import _utils
 from nilearn.image.resampling import resample_img, resample_to_img, reorder_img
 from nilearn.image.resampling import from_matrix_vector, coord_transform
 from nilearn.image.resampling import get_bounds
 from nilearn.image.resampling import BoundingBoxError
+from nilearn.image.image import _pad_array, crop_img
 from nilearn._utils import testing
+from nilearn.image import get_data
 
 
 ###############################################################################
@@ -37,49 +39,19 @@ def rotation(theta, phi):
     return np.dot(a1, a2)
 
 
-def pad(array, *args):
-    """Pad an ndarray with zeros of quantity specified
-    in args as follows args = (x1minpad, x1maxpad, x2minpad,
-    x2maxpad, x3minpad, ...)
-    """
-
-    if len(args) % 2 != 0:
-        raise ValueError("Please specify as many max paddings as min"
-                         " paddings. You have specified %d arguments" %
-                         len(args))
-
-    all_paddings = np.zeros([array.ndim, 2], dtype=np.int64)
-    all_paddings[:len(args) // 2] = np.array(args).reshape(-1, 2)
-
-    lower_paddings, upper_paddings = all_paddings.T
-    new_shape = np.array(array.shape) + upper_paddings + lower_paddings
-
-    padded = np.zeros(new_shape, dtype=array.dtype)
-    source_slices = [slice(max(-lp, 0), min(s + up, s))
-                     for lp, up, s in zip(lower_paddings,
-                                          upper_paddings,
-                                          array.shape)]
-    target_slices = [slice(max(lp, 0), min(s - up, s))
-                     for lp, up, s in zip(lower_paddings,
-                                          upper_paddings,
-                                          new_shape)]
-
-    padded[target_slices] = array[source_slices].copy()
-    return padded
-
-
 ###############################################################################
 # Tests
 def test_identity_resample():
     """ Test resampling with an identity affine.
     """
+    rng = np.random.RandomState(42)
     shape = (3, 2, 5, 2)
-    data = np.random.randint(0, 10, shape)
+    data = rng.randint(0, 10, shape)
     affine = np.eye(4)
     affine[:3, -1] = 0.5 * np.array(shape[:3])
     rot_img = resample_img(Nifti1Image(data, affine),
                            target_affine=affine, interpolation='nearest')
-    np.testing.assert_almost_equal(data, rot_img.get_data())
+    np.testing.assert_almost_equal(data, get_data(rot_img))
     # Smoke-test with a list affine
     rot_img = resample_img(Nifti1Image(data, affine),
                            target_affine=affine.tolist(),
@@ -88,7 +60,7 @@ def test_identity_resample():
     rot_img = resample_img(Nifti1Image(data, affine),
                            target_affine=affine[:3, :3],
                            interpolation='nearest')
-    np.testing.assert_almost_equal(data, rot_img.get_data())
+    np.testing.assert_almost_equal(data, get_data(rot_img))
 
     # Test with non native endian data
 
@@ -97,30 +69,35 @@ def test_identity_resample():
         rot_img = resample_img(Nifti1Image(data.astype('>f8'), affine),
                                target_affine=affine.tolist(),
                                interpolation=interpolation)
-        np.testing.assert_almost_equal(data, rot_img.get_data())
+        np.testing.assert_almost_equal(data, get_data(rot_img))
 
     # Test with little endian data ('<f8')
     for interpolation in ['nearest', 'linear', 'continuous']:
         rot_img = resample_img(Nifti1Image(data.astype('<f8'), affine),
                                target_affine=affine.tolist(),
                                interpolation=interpolation)
-        np.testing.assert_almost_equal(data, rot_img.get_data())
+        np.testing.assert_almost_equal(data, get_data(rot_img))
 
 
 def test_downsample():
     """ Test resampling with a 1/2 down-sampling affine.
     """
-    rand_gen = np.random.RandomState(0)
+    rng = np.random.RandomState(42)
     shape = (6, 3, 6, 2)
-    data = rand_gen.random_sample(shape)
+    data = rng.random_sample(shape)
     affine = np.eye(4)
     rot_img = resample_img(Nifti1Image(data, affine),
                            target_affine=2 * affine, interpolation='nearest')
     downsampled = data[::2, ::2, ::2, ...]
     x, y, z = downsampled.shape[:3]
     np.testing.assert_almost_equal(downsampled,
-                                   rot_img.get_data()[:x, :y, :z, ...])
+                                   get_data(rot_img)[:x, :y, :z, ...])
 
+    rot_img_2 = resample_img(Nifti1Image(data, affine),
+                             target_affine=2 * affine, interpolation='nearest',
+                             force_resample=True)
+    np.testing.assert_almost_equal(get_data(rot_img_2),
+                                   get_data(rot_img))
     # Test with non native endian data
 
     # Test to check that if giving non native endian data as input should
@@ -133,7 +110,7 @@ def test_downsample():
                                interpolation='nearest',
                                copy=copy)
         np.testing.assert_almost_equal(downsampled,
-                                       rot_img.get_data()[:x, :y, :z, ...])
+                                       get_data(rot_img)[:x, :y, :z, ...])
 
     # Little endian data
     for copy in [True, False]:
@@ -142,16 +119,16 @@ def test_downsample():
                                interpolation='nearest',
                                copy=copy)
         np.testing.assert_almost_equal(downsampled,
-                                       rot_img.get_data()[:x, :y, :z, ...])
+                                       get_data(rot_img)[:x, :y, :z, ...])
 
 
 def test_resampling_fill_value():
     """ Test resampling with a non-zero fill value
     """
-    prng = np.random.RandomState(10)
+    rng = np.random.RandomState(42)
 
-    data_3d = prng.rand(1, 4, 4)
-    data_4d = prng.rand(1, 4, 4, 3)
+    data_3d = rng.uniform(size=(1, 4, 4))
+    data_4d = rng.uniform(size=(1, 4, 4, 3))
 
     angle = np.pi/4
     rot = rotation(0, angle)
@@ -170,24 +147,24 @@ def test_resampling_fill_value():
                                        target_affine=rot,
                                        interpolation='nearest',
                                        clip=False)
-            assert_equal(rot_img.get_data().flatten()[0],
+            assert (get_data(rot_img).flatten()[0] ==
                          val)
 
             rot_img2 = resample_to_img(Nifti1Image(data, np.eye(4)),
                                        rot_img,
                                        interpolation='nearest',
                                        fill_value=val)
-            assert_equal(rot_img2.get_data().flatten()[0],
+            assert (get_data(rot_img2).flatten()[0] ==
                          val)
 
 
 def test_resampling_with_affine():
     """ Test resampling with a given rotation part of the affine.
     """
-    prng = np.random.RandomState(10)
+    rng = np.random.RandomState(42)
 
-    data_3d = prng.randint(4, size=(1, 4, 4))
-    data_4d = prng.randint(4, size=(1, 4, 4, 3))
+    data_4d = rng.randint(4, size=(1, 4, 4, 3))
+    data_3d = rng.randint(4, size=(1, 4, 4))
 
     for data in [data_3d, data_4d]:
         for angle in (0, np.pi, np.pi / 2., np.pi / 4., np.pi / 3.):
@@ -195,9 +172,9 @@ def test_resampling_with_affine():
             rot_img = resample_img(Nifti1Image(data, np.eye(4)),
                                    target_affine=rot,
                                    interpolation='nearest')
-            assert_equal(np.max(data),
-                         np.max(rot_img.get_data()))
-            assert_equal(rot_img.get_data().dtype, data.dtype)
+            assert (np.max(data) ==
+                         np.max(get_data(rot_img)))
+            assert get_data(rot_img).dtype == data.dtype
 
     # We take the same rotation logic as above and test with nonnative endian
     # data as input
@@ -207,15 +184,15 @@ def test_resampling_with_affine():
             rot = rotation(0, angle)
             rot_img = resample_img(img, target_affine=rot,
                                    interpolation='nearest')
-            assert_equal(np.max(data),
-                         np.max(rot_img.get_data()))
+            assert (np.max(data) ==
+                         np.max(get_data(rot_img)))
 
 
 def test_resampling_continuous_with_affine():
-    prng = np.random.RandomState(10)
+    rng = np.random.RandomState(42)
 
-    data_3d = prng.randint(1, 4, size=(1, 10, 10))
-    data_4d = prng.randint(1, 4, size=(1, 10, 10, 3))
+    data_3d = rng.randint(1, 4, size=(1, 10, 10))
+    data_4d = rng.randint(1, 4, size=(1, 10, 10, 3))
 
     for data in [data_3d, data_4d]:
         for angle in (0, np.pi / 2., np.pi, 3 * np.pi / 2.):
@@ -235,17 +212,18 @@ def test_resampling_continuous_with_affine():
             # values on the edges are wrong for some reason
             mask = (0, center, center)
             np.testing.assert_allclose(
-                img.get_data()[mask],
-                rot_img_back.get_data()[mask])
-            assert_equal(rot_img.get_data().dtype,
+                get_data(img)[mask],
+                get_data(rot_img_back)[mask])
+            assert (get_data(rot_img).dtype ==
                          np.dtype(data.dtype.name.replace('int', 'float')))
 
 
 def test_resampling_error_checks():
+    rng = np.random.RandomState(42)
     shape = (3, 2, 5, 2)
     target_shape = (5, 3, 2)
     affine = np.eye(4)
-    data = np.random.randint(0, 10, shape)
+    data = rng.randint(0, 10, shape)
     img = Nifti1Image(data, affine)
 
     # Correct parameters: no exception
@@ -256,40 +234,63 @@ def test_resampling_error_checks():
         resample_img(filename, target_shape=target_shape, target_affine=affine)
 
     # Missing parameter
-    assert_raises(ValueError, resample_img, img, target_shape=target_shape)
+    pytest.raises(ValueError, resample_img, img, target_shape=target_shape)
 
     # Invalid shape
-    assert_raises(ValueError, resample_img, img, target_shape=(2, 3),
+    pytest.raises(ValueError, resample_img, img, target_shape=(2, 3),
                   target_affine=affine)
 
     # Invalid interpolation
     interpolation = 'an_invalid_interpolation'
     pattern = "interpolation must be either.+{0}".format(interpolation)
-    testing.assert_raises_regex(ValueError, pattern,
-                                resample_img, img, target_shape=target_shape,
-                                target_affine=affine,
-                                interpolation="an_invalid_interpolation")
+    with pytest.raises(ValueError, match=pattern):
+        resample_img(img,
+                     target_shape=target_shape,
+                     target_affine=affine,
+                     interpolation="an_invalid_interpolation"
+                     )
+
+    # Resampling a binary image with continuous or
+    # linear interpolation should raise a warning.
+    data_binary = rng.randint(4, size=(1, 4, 4))
+    data_binary[data_binary>0] = 1
+    assert sorted(list(np.unique(data_binary))) == [0,1]
+
+    rot = rotation(0, np.pi / 4)
+    img_binary = Nifti1Image(data_binary, np.eye(4))
+    assert _utils.niimg._is_binary_niimg(img_binary)
+
+    with pytest.warns(Warning, match="Resampling binary images with"):
+        rot_img = resample_img(img_binary,
+                               target_affine=rot,
+                               interpolation='continuous')
+
+    with pytest.warns(Warning, match="Resampling binary images with"):
+        rot_img = resample_img(img_binary,
+                               target_affine=rot,
+                               interpolation='linear')
+
 
     # Noop
     target_shape = shape[:3]
 
     img_r = resample_img(img, copy=False)
-    assert_equal(img_r, img)
+    assert img_r == img
 
     img_r = resample_img(img, copy=True)
-    assert_false(np.may_share_memory(img_r.get_data(), img.get_data()))
+    assert not np.may_share_memory(get_data(img_r), get_data(img))
 
-    np.testing.assert_almost_equal(img_r.get_data(), img.get_data())
+    np.testing.assert_almost_equal(get_data(img_r), get_data(img))
     np.testing.assert_almost_equal(img_r.affine, img.affine)
 
     img_r = resample_img(img, target_affine=affine, target_shape=target_shape,
                          copy=False)
-    assert_equal(img_r, img)
+    assert img_r == img
 
     img_r = resample_img(img, target_affine=affine, target_shape=target_shape,
                          copy=True)
-    assert_false(np.may_share_memory(img_r.get_data(), img.get_data()))
-    np.testing.assert_almost_equal(img_r.get_data(), img.get_data())
+    assert not np.may_share_memory(get_data(img_r), get_data(img))
+    np.testing.assert_almost_equal(get_data(img_r), get_data(img))
     np.testing.assert_almost_equal(img_r.affine, img.affine)
 
 
@@ -331,18 +332,20 @@ def test_4d_affine_bounding_box_error():
 
     # The first 2 should pass
     assert_almost_equal(l2_norm(small_data),
-                 l2_norm(small_to_big_with_shape.get_data()))
-    assert_almost_equal(l2_norm(small_data),
-                 l2_norm(small_to_big_without_shape_3D_affine.get_data()))
+                        l2_norm(get_data(small_to_big_with_shape)))
+    assert_almost_equal(
+        l2_norm(small_data),
+        l2_norm(get_data(small_to_big_without_shape_3D_affine)))
 
     # After correcting decision tree for 4x4 affine given + no target shape
     # from "use initial shape" to "calculate minimal bounding box respecting
     # the affine anchor and the data"
     assert_almost_equal(l2_norm(small_data),
-                 l2_norm(small_to_big_without_shape.get_data()))
+                        l2_norm(get_data(small_to_big_without_shape)))
 
-    assert_array_equal(small_to_big_without_shape.shape,
-                 small_data_4D_affine[:3, -1] + np.array(small_img.shape))
+    assert_array_equal(
+        small_to_big_without_shape.shape,
+        small_data_4D_affine[:3, -1] + np.array(small_img.shape))
 
 
 def test_raises_upon_3x3_affine_and_no_shape():
@@ -350,12 +353,12 @@ def test_raises_upon_3x3_affine_and_no_shape():
                       affine=np.eye(4))
     exception = ValueError
     message = ("Given target shape without anchor "
-               "vector: Affine shape should be \(4, 4\) and "
-               "not \(3, 3\)")
-    testing.assert_raises_regex(
-        exception, message,
-        resample_img, img, target_affine=np.eye(3) * 2,
-        target_shape=(10, 10, 10))
+               "vector: Affine shape should be \\(4, 4\\) and "
+               "not \\(3, 3\\)")
+    with pytest.raises(exception, match=message):
+        resample_img(img, target_affine=np.eye(3) * 2,
+                     target_shape=(10, 10, 10)
+                     )
 
 
 def test_3x3_affine_bbox():
@@ -380,7 +383,7 @@ def test_3x3_affine_bbox():
 
     # If the bounding box is computed wrong, the image will be only
     # zeros
-    np.testing.assert_allclose(img_3d_affine.get_data().max(), image.max())
+    np.testing.assert_allclose(get_data(img_3d_affine).max(), image.max())
 
 
 def test_raises_bbox_error_if_data_outside_box():
@@ -424,9 +427,8 @@ def test_raises_bbox_error_if_data_outside_box():
                    "by the target affine does "
                    "not contain any of the data")
 
-        testing.assert_raises_regex(
-            exception, message,
-            resample_img, img, target_affine=new_affine)
+        with pytest.raises(exception, match=message):
+            resample_img(img, target_affine=new_affine)
 
 
 def test_resampling_result_axis_permutation():
@@ -453,7 +455,7 @@ def test_resampling_result_axis_permutation():
         resampled_img = resample_img(source_img,
                                      target_affine=target_affine)
 
-        resampled_data = resampled_img.get_data()
+        resampled_data = get_data(resampled_img)
         what_resampled_data_should_be = full_data.transpose(ap)
         assert_array_almost_equal(resampled_data,
                                   what_resampled_data_should_be)
@@ -467,12 +469,12 @@ def test_resampling_result_axis_permutation():
 
         resampled_img = resample_img(source_img,
                                      target_affine=target_affine)
-        resampled_data = resampled_img.get_data()
+        resampled_data = get_data(resampled_img)
         offset_cropping = np.vstack([-offset[ap][np.newaxis, :],
                                      np.zeros([1, 3])]
                                     ).T.ravel().astype(int)
-        what_resampled_data_should_be = pad(full_data.transpose(ap),
-                                            *list(offset_cropping))
+        what_resampled_data_should_be = _pad_array(full_data.transpose(ap),
+                                                   list(offset_cropping))
 
         assert_array_almost_equal(resampled_data,
                                   what_resampled_data_should_be)
@@ -486,7 +488,7 @@ def test_resampling_nan():
         # create deterministic data, padded with one
         # voxel thickness of zeros
         core_data = np.arange(np.prod(core_shape)
-                              ).reshape(core_shape).astype(np.float)
+                              ).reshape(core_shape).astype(np.float64)
         # Introduce a nan
         core_data[2, 2:4, 1] = np.nan
         full_data_shape = np.array(core_shape) + 2
@@ -501,26 +503,26 @@ def test_resampling_nan():
 
         # check 3x3 transformation matrix
         target_affine = np.eye(3)[axis_permutation]
-        resampled_img = testing.assert_warns(
-            RuntimeWarning, resample_img, source_img,
-            target_affine=target_affine)
+        with pytest.warns(Warning, match=r"(\bnan\b|invalid value)"):
+            resampled_img = resample_img(source_img,
+                                         target_affine=target_affine)
 
-        resampled_data = resampled_img.get_data()
+        resampled_data = get_data(resampled_img)
         if full_data.ndim == 4:
             axis_permutation.append(3)
         what_resampled_data_should_be = full_data.transpose(axis_permutation)
         non_nan = np.isfinite(what_resampled_data_should_be)
 
         # Check that the input data hasn't been modified:
-        assert_false(np.all(non_nan))
+        assert not np.all(non_nan)
 
         # Check that for finite value resampling works without problems
         assert_array_almost_equal(resampled_data[non_nan],
                                   what_resampled_data_should_be[non_nan])
 
         # Check that what was not finite is still not finite
-        assert_false(np.any(np.isfinite(
-                        resampled_data[np.logical_not(non_nan)])))
+        assert not np.any(np.isfinite(
+                        resampled_data[np.logical_not(non_nan)]))
 
     # Test with an actual resampling, in the case of a bigish hole
     # This checks the extrapolation mechanism: if we don't do any
@@ -529,19 +531,19 @@ def test_resampling_nan():
     data = 10 * np.ones((10, 10, 10))
     data[4:6, 4:6, 4:6] = np.nan
     source_img = Nifti1Image(data, 2 * np.eye(4))
-    resampled_img = testing.assert_warns(
-        RuntimeWarning, resample_img, source_img,
-        target_affine=np.eye(4))
+    with pytest.warns(RuntimeWarning):
+        resampled_img = resample_img(source_img,
+                                     target_affine=np.eye(4))
 
-    resampled_data = resampled_img.get_data()
+    resampled_data = get_data(resampled_img)
     np.testing.assert_allclose(10, resampled_data[np.isfinite(resampled_data)])
 
 
 def test_resample_to_img():
     # Testing resample to img function
-    rand_gen = np.random.RandomState(0)
+    rng = np.random.RandomState(42)
     shape = (6, 3, 6, 3)
-    data = rand_gen.random_sample(shape)
+    data = rng.random_sample(shape)
 
     source_affine = np.eye(4)
     source_img = Nifti1Image(data, source_affine)
@@ -556,7 +558,57 @@ def test_resample_to_img():
     downsampled = data[::2, ::2, ::2, ...]
     x, y, z = downsampled.shape[:3]
     np.testing.assert_almost_equal(downsampled,
-                                   result_img.get_data()[:x, :y, :z, ...])
+                                   get_data(result_img)[:x, :y, :z, ...])
+
+def test_crop():
+    # Testing that padding of arrays and cropping of images work symmetrically
+    shape = (4, 6, 2)
+    data = np.ones(shape)
+    padded = _pad_array(data, [3, 2, 4, 4, 5, 7])
+    padd_nii = Nifti1Image(padded, np.eye(4))
+
+    cropped = crop_img(padd_nii, pad=False)
+    np.testing.assert_equal(get_data(cropped), data)
+
+
+def test_resample_identify_affine_int_translation():
+    # Testing resample to img function
+    rng = np.random.RandomState(42)
+
+    source_shape = (6, 4, 6)
+    source_affine = np.eye(4)
+    source_affine[:, 3] = np.append(np.random.randint(0, 4, 3), 1)
+    source_data = rng.random_sample(source_shape)
+    source_img = Nifti1Image(source_data, source_affine)
+
+    target_shape = (11, 10, 9)
+    target_data = np.zeros(target_shape)
+    target_affine = source_affine
+    target_affine[:3, 3] -= 3  # add an offset of 3 in x, y, z
+    target_data[3:9, 3:7, 3:9] = source_data  # put the data at the offset location
+    target_img = Nifti1Image(target_data, target_affine)
+
+    result_img = resample_to_img(source_img, target_img,
+                                 interpolation='nearest')
+    np.testing.assert_almost_equal(get_data(target_img),
+                                   get_data(result_img))
+
+    result_img_2 = resample_to_img(result_img, source_img,
+                                   interpolation='nearest')
+    np.testing.assert_almost_equal(get_data(source_img),
+                                   get_data(result_img_2))
+
+    result_img_3 = resample_to_img(result_img, source_img,
+                                   interpolation='nearest',
+                                   force_resample=True)
+    np.testing.assert_almost_equal(get_data(result_img_2),
+                                   get_data(result_img_3))
+
+    result_img_4 = resample_to_img(source_img, target_img,
+                                   interpolation='nearest',
+                                   force_resample=True)
+    np.testing.assert_almost_equal(get_data(target_img),
+                                   get_data(result_img_4))
 
 def test_resample_clip():
     # Resample and image and get larger and smaller
@@ -570,17 +622,17 @@ def test_resample_clip():
     source_img = Nifti1Image(data, source_affine)
 
     target_affine = np.eye(4)
-    no_clip_data = resample_img(source_img, target_affine,
-                                clip=False).get_data()
-    clip_data = resample_img(source_img,
-                             target_affine, clip=True).get_data()
+    no_clip_data = get_data(resample_img(source_img, target_affine,
+                                         clip=False))
+    clip_data = get_data(resample_img(source_img,
+                                      target_affine, clip=True))
 
     not_clip = np.where((no_clip_data > data.min()) & (no_clip_data < data.max()))
 
-    assert_true(np.any(no_clip_data > data.max()))
-    assert_true(np.any(no_clip_data < data.min()))
-    assert_true(np.all(clip_data <= data.max()))
-    assert_true(np.all(clip_data >= data.min()))
+    assert np.any(no_clip_data > data.max())
+    assert np.any(no_clip_data < data.min())
+    assert np.all(clip_data <= data.max())
+    assert np.all(clip_data >= data.min())
     assert_array_equal(no_clip_data[not_clip], clip_data[not_clip])
 
 
@@ -589,12 +641,12 @@ def test_reorder_img():
     # shape, whereas reordering does.
     shape = (5, 5, 5, 2, 2)
     rng = np.random.RandomState(42)
-    data = rng.rand(*shape)
+    data = rng.uniform(size=shape)
     affine = np.eye(4)
     affine[:3, -1] = 0.5 * np.array(shape[:3])
     ref_img = Nifti1Image(data, affine)
     # Test with purely positive matrices and compare to a rotation
-    for theta, phi in np.random.randint(4, size=(5, 2)):
+    for theta, phi in rng.randint(4, size=(5, 2)):
         rot = rotation(theta * np.pi / 2, phi * np.pi / 2)
         rot[np.abs(rot) < 0.001] = 0
         rot[rot > 0.9] = 1
@@ -603,19 +655,19 @@ def test_reorder_img():
         new_affine = from_matrix_vector(rot, b)
         rot_img = resample_img(ref_img, target_affine=new_affine)
         np.testing.assert_array_equal(rot_img.affine, new_affine)
-        np.testing.assert_array_equal(rot_img.get_data().shape, shape)
+        np.testing.assert_array_equal(get_data(rot_img).shape, shape)
         reordered_img = reorder_img(rot_img)
         np.testing.assert_array_equal(reordered_img.affine[:3, :3],
                                       np.eye(3))
-        np.testing.assert_almost_equal(reordered_img.get_data(),
+        np.testing.assert_almost_equal(get_data(reordered_img),
                                        data)
 
     # Create a non-diagonal affine, and check that we raise a sensible
     # exception
     affine[1, 0] = 0.1
     ref_img = Nifti1Image(data, affine)
-    testing.assert_raises_regex(ValueError, 'Cannot reorder the axes',
-                                reorder_img, ref_img)
+    with pytest.raises(ValueError, match='Cannot reorder the axes'):
+        reorder_img(ref_img)
 
     # Test that no exception is raised when resample='continuous'
     reorder_img(ref_img, resample='continuous')
@@ -626,18 +678,17 @@ def test_reorder_img():
     resampled_img = resample_img(ref_img,
                                  target_affine=reordered_img.affine,
                                  interpolation=interpolation)
-    np.testing.assert_array_equal(reordered_img.get_data(),
-                                  resampled_img.get_data())
+    np.testing.assert_array_equal(get_data(reordered_img),
+                                  get_data(resampled_img))
 
     # Make sure invalid resample argument is included in the error message
     interpolation = 'an_invalid_interpolation'
     pattern = "interpolation must be either.+{0}".format(interpolation)
-    testing.assert_raises_regex(ValueError, pattern,
-                                reorder_img, ref_img,
-                                resample=interpolation)
+    with pytest.raises(ValueError, match=pattern):
+        reorder_img(ref_img, resample=interpolation)
 
     # Test flipping an axis
-    data = rng.rand(*shape)
+    data = rng.uniform(size=shape)
     for i in (0, 1, 2):
         # Make a diagonal affine with a negative axis, and check that
         # can be reordered, also vary the shape
@@ -651,12 +702,12 @@ def test_reorder_img():
         img2 = reorder_img(img)
         # Check that img has not been changed
         np.testing.assert_array_equal(img.affine, orig_img.affine)
-        np.testing.assert_array_equal(img.get_data(),
-                                      orig_img.get_data())
+        np.testing.assert_array_equal(get_data(img),
+                                      get_data(orig_img))
         # Test that the affine is indeed diagonal:
         np.testing.assert_array_equal(img2.affine[:3, :3],
                                       np.diag(np.diag(img2.affine[:3, :3])))
-        assert_true(np.all(np.diag(img2.affine) >= 0))
+        assert np.all(np.diag(img2.affine) >= 0)
 
 
 def test_reorder_img_non_native_endianness():
@@ -681,7 +732,7 @@ def test_reorder_img_non_native_endianness():
     img_1 = _get_resampled_img('<f8')
     img_2 = _get_resampled_img('>f8')
 
-    np.testing.assert_equal(img_1.get_data(), img_2.get_data())
+    np.testing.assert_equal(get_data(img_1), get_data(img_2))
 
 
 def test_reorder_img_mirror():
@@ -700,10 +751,11 @@ def test_reorder_img_mirror():
 
 
 def test_coord_transform_trivial():
+    rng = np.random.RandomState(42)
     sform = np.eye(4)
-    x = np.random.random((10,))
-    y = np.random.random((10,))
-    z = np.random.random((10,))
+    x = rng.random_sample((10,))
+    y = rng.random_sample((10,))
+    z = rng.random_sample((10,))
 
     x_, y_, z_ = coord_transform(x, y, z, sform)
     np.testing.assert_array_equal(x, x_)
@@ -735,13 +787,15 @@ def test_coord_transform_trivial():
     y = np.ones((3, 2, 4))
     z = np.ones((3, 2, 4))
     x_, y_, z_ = coord_transform(x, y, z, sform)
-    assert_equal(x.shape, x_.shape)
+    assert x.shape == x_.shape
 
 
+@pytest.mark.skipif(os.environ.get('APPVEYOR') == 'True',
+                    reason='This test too slow (7-8 minutes) on AppVeyor')
+@pytest.mark.skipif((os.environ.get('TRAVIS') == 'true'
+                     and os.environ.get('TRAVIS_CPU_ARCH') == 'arm64'),
+                    reason='This test does not run on ARM arch.')
 def test_resample_img_segmentation_fault():
-    if os.environ.get('APPVEYOR') == 'True':
-        raise SkipTest('This test too slow (7-8 minutes) on AppVeyor')
-
     # see https://github.com/nilearn/nilearn/issues/346
     shape_in = (64, 64, 64)
     aff_in = np.diag([2., 2., 2., 1.])
@@ -754,13 +808,13 @@ def test_resample_img_segmentation_fault():
         data = np.ones(shape_in + (fourth_dim, ), dtype=np.float64)
     except MemoryError:
         # This can happen on AppVeyor and for 32-bit Python on Windows
-        raise SkipTest('Not enough RAM to run this test')
+        pytest.skip('Not enough RAM to run this test')
+    else:
+        img_in = Nifti1Image(data, aff_in)
 
-    img_in = Nifti1Image(data, aff_in)
-
-    resample_img(img_in,
-                 target_affine=aff_out,
-                 interpolation='nearest')
+        resample_img(img_in,
+                     target_affine=aff_out,
+                     interpolation='nearest')
 
 
 def test_resampling_with_int_types_no_crash():
