@@ -1,20 +1,23 @@
 """
 Utilities to compute and operate on brain masks
 """
-# Author: Gael Varoquaux, Alexandre Abraham, Philippe Gervais
+# Authors: Gael Varoquaux, Alexandre Abraham, Philippe Gervais, Ana Luisa Pinho
 # License: simplified BSD
 import warnings
 import numbers
 
 import numpy as np
 from scipy import ndimage
-from sklearn.externals.joblib import Parallel, delayed
+from joblib import Parallel, delayed
 
+from sklearn.utils import deprecated
 from . import _utils
-from .image import new_img_like
+from .image import get_data, new_img_like, resampling
 from ._utils.cache_mixin import cache
 from ._utils.ndimage import largest_connected_component, get_border_data
-from ._utils.niimg import _safe_get_data, img_data_dtype
+from ._utils.niimg import _safe_get_data
+from .datasets import (load_mni152_template, load_mni152_gm_template,
+                       load_mni152_wm_template)
 
 
 class MaskWarning(UserWarning):
@@ -42,7 +45,7 @@ def _load_mask_img(mask_img, allow_empty=False):
         boolean version of the mask
     """
     mask_img = _utils.check_niimg_3d(mask_img)
-    mask = mask_img.get_data()
+    mask = _safe_get_data(mask_img, ensure_finite=True)
     values = np.unique(mask)
 
     if len(values) == 1:
@@ -52,7 +55,7 @@ def _load_mask_img(mask_img, allow_empty=False):
                 'The mask is invalid as it is empty: it masks all data.')
     elif len(values) == 2:
         # If there are 2 different values, one of them must be 0 (background)
-        if not 0 in values:
+        if 0 not in values:
             raise ValueError('Background of the mask must be represented with'
                              '0. Given mask contains: %s.' % values)
     elif len(values) != 2:
@@ -70,9 +73,9 @@ def _extrapolate_out_mask(data, mask, iterations=1):
     """
     if iterations > 1:
         data, mask = _extrapolate_out_mask(data, mask,
-                                          iterations=iterations - 1)
+                                           iterations=iterations - 1)
     new_mask = ndimage.binary_dilation(mask)
-    larger_mask = np.zeros(np.array(mask.shape) + 2, dtype=np.bool)
+    larger_mask = np.zeros(np.array(mask.shape) + 2, dtype=bool)
     larger_mask[1:-1, 1:-1, 1:-1] = mask
     # Use nans as missing value: ugly
     masked_data = np.zeros(larger_mask.shape + data.shape[3:])
@@ -82,7 +85,7 @@ def _extrapolate_out_mask(data, mask, iterations=1):
     outer_shell[1:-1, 1:-1, 1:-1] = np.logical_xor(new_mask, mask)
     outer_shell_x, outer_shell_y, outer_shell_z = np.where(outer_shell)
     extrapolation = list()
-    for i, j, k in [(1, 0, 0), (-1, 0, 0), 
+    for i, j, k in [(1, 0, 0), (-1, 0, 0),
                     (0, 1, 0), (0, -1, 0),
                     (0, 0, 1), (0, 0, -1)]:
         this_x = outer_shell_x + i
@@ -91,8 +94,8 @@ def _extrapolate_out_mask(data, mask, iterations=1):
         extrapolation.append(masked_data[this_x, this_y, this_z])
 
     extrapolation = np.array(extrapolation)
-    extrapolation = (np.nansum(extrapolation, axis=0)
-                     / np.sum(np.isfinite(extrapolation), axis=0))
+    extrapolation = (np.nansum(extrapolation, axis=0) /
+                     np.sum(np.isfinite(extrapolation), axis=0))
     extrapolation[np.logical_not(np.isfinite(extrapolation))] = 0
     new_data = np.zeros_like(masked_data)
     new_data[outer_shell] = extrapolation
@@ -162,7 +165,8 @@ def intersect_masks(mask_imgs, threshold=0.5, connected=True):
     if np.any(grp_mask > 0) and connected:
         grp_mask = largest_connected_component(grp_mask)
     grp_mask = _utils.as_ndarray(grp_mask, dtype=np.int8)
-    return new_img_like(_utils.check_niimg_3d(mask_imgs[0]), grp_mask, ref_affine)
+    return new_img_like(_utils.check_niimg_3d(mask_imgs[0]), grp_mask,
+                        ref_affine)
 
 
 def _post_process_mask(mask, affine, opening=2, connected=True,
@@ -173,7 +177,7 @@ def _post_process_mask(mask, affine, opening=2, connected=True,
     mask_any = mask.any()
     if not mask_any:
         warnings.warn("Computed an empty mask. %s" % warning_msg,
-            MaskWarning, stacklevel=2)
+                      MaskWarning, stacklevel=2)
     if connected and mask_any:
         mask = largest_connected_component(mask)
     if opening:
@@ -258,8 +262,8 @@ def compute_epi_mask(epi_img, lower_cutoff=0.2, upper_cutoff=0.85,
 
     # Delayed import to avoid circular imports
     from .image.image import _compute_mean
-    mean_epi, affine = cache(_compute_mean, memory)(epi_img,
-                                     target_affine=target_affine,
+    mean_epi, affine = \
+        cache(_compute_mean, memory)(epi_img, target_affine=target_affine,
                                      target_shape=target_shape,
                                      smooth=(1 if opening else False))
 
@@ -278,14 +282,15 @@ def compute_epi_mask(epi_img, lower_cutoff=0.2, upper_cutoff=0.85,
     delta = sorted_input[lower_cutoff + 1:upper_cutoff + 1] \
         - sorted_input[lower_cutoff:upper_cutoff]
     ia = delta.argmax()
-    threshold = 0.5 * (sorted_input[ia + lower_cutoff]
-                       + sorted_input[ia + lower_cutoff + 1])
+    threshold = 0.5 * (sorted_input[ia + lower_cutoff] +
+                       sorted_input[ia + lower_cutoff + 1])
 
     mask = mean_epi >= threshold
 
     mask, affine = _post_process_mask(mask, affine, opening=opening,
-        connected=connected, warning_msg="Are you sure that input "
-            "data are EPI images not detrended. ")
+                                      connected=connected,
+                                      warning_msg="Are you sure that input "
+                                      "data are EPI images not detrended. ")
     return new_img_like(epi_img, mask, affine)
 
 
@@ -370,9 +375,9 @@ def compute_multi_epi_mask(epi_imgs, lower_cutoff=0.2, upper_cutoff=0.85,
 
 
 def compute_background_mask(data_imgs, border_size=2,
-                     connected=False, opening=False,
-                     target_affine=None, target_shape=None,
-                     memory=None, verbose=0):
+                            connected=False, opening=False,
+                            target_affine=None, target_shape=None,
+                            memory=None, verbose=0):
     """ Compute a brain mask for the images by guessing the value of the
     background from the border of the image.
 
@@ -426,28 +431,30 @@ def compute_background_mask(data_imgs, border_size=2,
     # Delayed import to avoid circular imports
     from .image.image import _compute_mean
     data, affine = cache(_compute_mean, memory)(data_imgs,
-                target_affine=target_affine, target_shape=target_shape,
-                smooth=False)
+                                                target_affine=target_affine,
+                                                target_shape=target_shape,
+                                                smooth=False)
 
-    background = np.median(get_border_data(data, border_size))
-    if np.isnan(background):
+    if np.isnan(get_border_data(data, border_size)).any():
         # We absolutely need to catter for NaNs as a background:
         # SPM does that by default
         mask = np.logical_not(np.isnan(data))
     else:
+        background = np.median(get_border_data(data, border_size))
         mask = data != background
 
     mask, affine = _post_process_mask(mask, affine, opening=opening,
-        connected=connected, warning_msg="Are you sure that input "
-            "images have a homogeneous background.")
+                                      connected=connected,
+                                      warning_msg="Are you sure that input "
+                                      "images have a homogeneous background.")
     return new_img_like(data_imgs, mask, affine)
 
 
 def compute_multi_background_mask(data_imgs, border_size=2, upper_cutoff=0.85,
-                           connected=True, opening=2, threshold=0.5,
-                           target_affine=None, target_shape=None,
-                           exclude_zeros=False, n_jobs=1,
-                           memory=None, verbose=0):
+                                  connected=True, opening=2, threshold=0.5,
+                                  target_affine=None, target_shape=None,
+                                  exclude_zeros=False, n_jobs=1,
+                                  memory=None, verbose=0):
     """ Compute a common mask for several sessions or subjects of data.
 
     Uses the mask-finding algorithms to extract masks for each session
@@ -502,18 +509,21 @@ def compute_multi_background_mask(data_imgs, border_size=2, upper_cutoff=0.85,
                         'image or a list of images' % data_imgs)
     masks = Parallel(n_jobs=n_jobs, verbose=verbose)(
         delayed(compute_background_mask)(img,
-                                  border_size=border_size,
-                                  connected=connected,
-                                  opening=opening,
-                                  target_affine=target_affine,
-                                  target_shape=target_shape,
-                                  memory=memory)
+                                         border_size=border_size,
+                                         connected=connected,
+                                         opening=opening,
+                                         target_affine=target_affine,
+                                         target_shape=target_shape,
+                                         memory=memory)
         for img in data_imgs)
 
     mask = intersect_masks(masks, connected=connected, threshold=threshold)
     return mask
 
 
+@deprecated("Function 'compute_gray_matter_mask' has been renamed to "
+            "'compute_brain_mask' and 'compute_gray_matter_mask' will be "
+            "removed in release 0.9.0.")
 def compute_gray_matter_mask(target_img, threshold=.5,
                              connected=True, opening=2, memory=None,
                              verbose=0):
@@ -523,20 +533,20 @@ def compute_gray_matter_mask(target_img, threshold=.5,
 
     Parameters
     ----------
-    target_img: Niimg-like object
+    target_img : Niimg-like object
         See http://nilearn.github.io/manipulating_images/input_output.html
         Images used to compute the mask. 3D and 4D images are accepted.
         Only the shape and affine of target_img will be used here.
 
-    threshold: float, optional
+    threshold : float, optional
         The value under which the MNI template is cut off.
         Default value is 0.5
 
-    connected: bool, optional
+    connected : bool, optional
         if connected is True, only the largest connected component is kept.
         Default is True
 
-    opening: bool or int, optional
+    opening : bool or int, optional
         if opening is True, a morphological opening is performed, to keep
         only large structures.
         If opening is an integer `n`, it is performed via `n` erosions.
@@ -545,43 +555,105 @@ def compute_gray_matter_mask(target_img, threshold=.5,
         to 1 opening operation of order `n` followed by a closing operator
         of order `n`.
 
-    memory: instance of joblib.Memory or str
+    memory : instance of joblib.Memory or str
         Used to cache the function call.
 
-    verbose: int, optional
+    verbose : int, optional
         Controls the amount of verbosity: higher numbers give
         more messages
 
     Returns
     -------
-    mask: nibabel.Nifti1Image
+    mask : nibabel.Nifti1Image
         The brain mask (3D image)
     """
+
+    return compute_brain_mask(target_img=target_img, threshold=threshold,
+                              connected=connected, opening=opening,
+                              memory=memory, verbose=verbose,
+                              mask_type='whole-brain')
+
+
+def compute_brain_mask(target_img, threshold=.5, connected=True, opening=2,
+                       memory=None, verbose=0, mask_type='whole-brain'):
+    """Compute the whole-brain, grey-matter or white-matter mask.
+    This mask is calculated using MNI152 1mm-resolution template mask onto the
+    target image.
+
+    Parameters
+    ----------
+    target_img : Niimg-like object
+        See http://nilearn.github.io/manipulating_images/input_output.html
+        Images used to compute the mask. 3D and 4D images are accepted.
+        Only the shape and affine of target_img will be used here.
+
+    threshold : float, optional
+        The value under which the MNI template is cut off.
+        Default value is 0.5
+
+    connected : bool, optional
+        if connected is True, only the largest connected component is kept.
+        Default is True
+
+    opening : bool or int, optional
+        if opening is True, a morphological opening is performed, to keep
+        only large structures.
+        If opening is an integer `n`, it is performed via `n` erosions.
+        After estimation of the largest connected constituent, 2`n` closing
+        operations are performed followed by `n` erosions. This corresponds
+        to 1 opening operation of order `n` followed by a closing operator
+        of order `n`.
+
+    memory : instance of joblib.Memory or str
+        Used to cache the function call.
+
+    verbose : int, optional
+        Controls the amount of verbosity: higher numbers give
+        more messages
+
+    mask_type : {'whole-brain', 'gm', 'wm'}, optional
+        Type of mask to be computed: 'whole-brain', 'grey-matter' ('gm') or
+        'white-matter' ('wm'). Default = 'whole-brain'
+
+        .. versionadded:: 0.8.1
+
+    Returns
+    -------
+    mask : nibabel.Nifti1Image
+        The whole-brain mask (3D image)
+    """
     if verbose > 0:
-        print("Template mask computation")
+        print("Template", mask_type, "mask computation")
 
     target_img = _utils.check_niimg(target_img)
 
-    from .datasets import load_mni152_brain_mask
-    template = load_mni152_brain_mask()
-    dtype = img_data_dtype(target_img)
-    template = new_img_like(template,
-                            template.get_data().astype(dtype))
+    if mask_type == 'whole-brain':
+        template = load_mni152_template(resolution=1)
+    elif mask_type == 'gm':
+        template = load_mni152_gm_template(resolution=1)
+    elif mask_type == 'wm':
+        template = load_mni152_wm_template(resolution=1)
+    else:
+        raise ValueError(f"Unknown mask type {mask_type}. "
+                         "Only 'whole-brain', 'gm' or 'wm' are accepted.")
 
-    from .image.resampling import resample_to_img
-    resampled_template = cache(resample_to_img, memory)(template, target_img)
+    resampled_template = cache(resampling.resample_to_img, memory)(
+        template, target_img)
 
-    mask = resampled_template.get_data() >= threshold
+    mask = (get_data(resampled_template) >= threshold).astype("int8")
 
+    warning_message = (f"{mask_type} mask is empty, "
+                       "lower the threshold or check your input FOV")
     mask, affine = _post_process_mask(mask, target_img.affine, opening=opening,
                                       connected=connected,
-                                      warning_msg="Gray matter mask is empty, "
-                                                  "lower the threshold or "
-                                                  "check your input FOV")
+                                      warning_msg=warning_message)
 
     return new_img_like(target_img, mask, affine)
 
 
+@deprecated("Function 'compute_multi_gray_matter_mask' has been renamed to "
+            "'compute_multi_brain_mask' and 'compute_multi_gray_matter_mask' "
+            "will be removed in release 0.10.0")
 def compute_multi_gray_matter_mask(target_imgs, threshold=.5,
                                    connected=True, opening=2,
                                    memory=None, verbose=0, n_jobs=1, **kwargs):
@@ -601,7 +673,7 @@ def compute_multi_gray_matter_mask(target_imgs, threshold=.5,
 
     threshold: float, optional
         The value under which the MNI template is cut off.
-        Default value is 0.5
+        Default value is 0.5.
 
     connected: bool, optional
         if connected is True, only the largest connect component is kept.
@@ -638,7 +710,77 @@ def compute_multi_gray_matter_mask(target_imgs, threshold=.5,
 
     See also
     --------
-    nilearn.masking.compute_gray_matter_mask
+    nilearn.masking.compute_brain_mask
+    """
+    return compute_multi_brain_mask(target_imgs=target_imgs,
+                                    threshold=threshold, connected=connected,
+                                    opening=opening, memory=memory,
+                                    verbose=verbose, n_jobs=n_jobs,
+                                    mask_type='whole-brain', **kwargs)
+
+
+def compute_multi_brain_mask(target_imgs, threshold=.5, connected=True,
+                             opening=2, memory=None, verbose=0, n_jobs=1,
+                             mask_type='whole-brain', **kwargs):
+    """ Compute the whole-brain, grey-matter or white-matter mask for a list of
+    images. The mask is calculated through the resampling of the corresponding
+    MNI152 template mask onto the target image.
+
+    .. versionadded:: 0.8.1
+
+    Parameters
+    ----------
+    target_imgs: list of Niimg-like object
+        See http://nilearn.github.io/manipulating_images/input_output.html
+        Images used to compute the mask. 3D and 4D images are accepted.
+        The images in this list must be of same shape and affine. The mask is
+        calculated with the first element of the list for only the shape/affine
+        of the image is used for this masking strategy
+
+    threshold: float, optional
+        The value under which the MNI template is cut off.
+        Default value is 0.5.
+
+    connected: bool, optional
+        if connected is True, only the largest connect component is kept.
+        Default is True
+
+    opening: bool or int, optional
+        if opening is True, a morphological opening is performed, to keep
+        only large structures.
+        If opening is an integer `n`, it is performed via `n` erosions.
+        After estimation of the largest connected constituent, 2`n` closing
+        operations are performed followed by `n` erosions. This corresponds
+        to 1 opening operation of order `n` followed by a closing operator
+        of order `n`.
+
+    memory: instance of joblib.Memory or str
+        Used to cache the function call.
+
+    n_jobs: integer, optional
+        Argument not used but kept to fit the API
+
+    mask_type: {'whole-brain', 'gm', 'wm'}, optional
+        Type of mask to be computed: 'whole-brain', 'grey-matter' ('gm') or
+        'white-matter' ('wm'). Default = 'whole-brain'
+
+    **kwargs: optional arguments
+        arguments such as 'target_affine' are used in the call of other
+        masking strategies, which then would raise an error for this function
+        which does not need such arguments.
+
+    verbose: int, optional
+        Controls the amount of verbosity: higher numbers give
+        more messages
+
+    Returns
+    -------
+    mask: nibabel.Nifti1Image
+        The brain mask (3D image)
+
+    See also
+    --------
+    nilearn.masking.compute_brain_mask
     """
     if len(target_imgs) == 0:
         raise TypeError('An empty object - %r - was passed instead of an '
@@ -649,9 +791,10 @@ def compute_multi_gray_matter_mask(target_imgs, threshold=.5,
     for _ in imgs_generator:
         pass
 
-    mask = compute_gray_matter_mask(target_imgs[0], threshold=threshold,
-                                    connected=connected, opening=opening,
-                                    memory=memory, verbose=verbose)
+    mask = compute_brain_mask(target_imgs[0], threshold=threshold,
+                              connected=connected, opening=opening,
+                              memory=memory, verbose=verbose,
+                              mask_type=mask_type)
     return mask
 
 
@@ -696,7 +839,7 @@ def apply_mask(imgs, mask_img, dtype='f',
     Notes
     -----
     When using smoothing, ensure_finite is set to True, as non-finite
-    values would spread accross the image.
+    values would spread across the image.
     """
     mask_img = _utils.check_niimg_3d(mask_img)
     mask, mask_affine = _load_mask_img(mask_img)
@@ -717,8 +860,8 @@ def _apply_mask_fmri(imgs, mask_img, dtype='f',
 
     mask_img = _utils.check_niimg_3d(mask_img)
     mask_affine = mask_img.affine
-    mask_data = _utils.as_ndarray(mask_img.get_data(),
-                                  dtype=np.bool)
+    mask_data = _utils.as_ndarray(get_data(mask_img),
+                                  dtype=bool)
 
     if smoothing_fwhm is not None:
         ensure_finite = True
@@ -769,7 +912,7 @@ def _unmask_3d(X, mask, order="C"):
         Mask. mask.ndim must be equal to 3, and dtype *must* be bool.
     """
 
-    if mask.dtype != np.bool:
+    if mask.dtype != bool:
         raise TypeError("mask must be a boolean array")
     if X.ndim != 1:
         raise TypeError("X must be a 1-dimensional array")
@@ -802,7 +945,7 @@ def _unmask_4d(X, mask, order="C"):
         Shape: (mask.shape[0], mask.shape[1], mask.shape[2], X.shape[0])
     """
 
-    if mask.dtype != np.bool:
+    if mask.dtype != bool:
         raise TypeError("mask must be a boolean array")
     if X.ndim != 2:
         raise TypeError("X must be a 2-dimensional array")
@@ -863,3 +1006,30 @@ def unmask(X, mask_img, order="F"):
                         "got shape: %s" % str(X.shape))
 
     return new_img_like(mask_img, unmasked, affine)
+
+
+def _unmask_from_to_3d_array(w, mask):
+    """Unmask an image into whole brain, with off-mask voxels set to 0.
+    Used as a stand-alone function in low-level decoding (SpaceNet) and
+    clustering (ReNA) functions.
+
+    Parameters
+    ----------
+    w : ndarray, shape (n_features,)
+      The image to be unmasked.
+
+    mask : ndarray, shape (nx, ny, nz)
+      The mask used in the unmasking operation. It is required that
+      mask.sum() == n_features.
+
+    Returns
+    -------
+    out : 3d of same shape as `mask`.
+        The unmasked version of `w`
+    """
+
+    if mask.sum() != len(w):
+        raise ValueError("Expecting mask.sum() == len(w).")
+    out = np.zeros(mask.shape, dtype=w.dtype)
+    out[mask] = w
+    return out
