@@ -5,11 +5,12 @@ Massively Univariate Linear Model estimated with OLS and permutation test.
 #         Virgile Fritsch, <virgile.fritsch@inria.fr>, jan. 2014
 import warnings
 
+import sys
+import time
+import joblib
 import numpy as np
 from scipy import linalg
 from sklearn.utils import check_random_state
-import sklearn.externals.joblib as joblib
-
 
 def normalize_matrix_on_axis(m, axis=0):
     """ Normalize a 2D matrix on an axis.
@@ -21,6 +22,7 @@ def normalize_matrix_on_axis(m, axis=0):
 
     axis : integer in {0, 1}, optional
       A valid axis to normalize across.
+      Default=0.
 
     Returns
     -------
@@ -34,11 +36,11 @@ def normalize_matrix_on_axis(m, axis=0):
     ...     normalize_matrix_on_axis)
     >>> X = np.array([[0, 4], [1, 0]])
     >>> normalize_matrix_on_axis(X)
-    array([[ 0.,  1.],
-           [ 1.,  0.]])
+    array([[0., 1.],
+           [1., 0.]])
     >>> normalize_matrix_on_axis(X, axis=1)
-    array([[ 0.,  1.],
-           [ 1.,  0.]])
+    array([[0., 1.],
+           [1., 0.]])
 
     """
     if m.ndim > 2:
@@ -66,6 +68,9 @@ def orthonormalize_matrix(m, tol=1.e-12):
     m : numpy array,
       The matrix to orthonormalize.
 
+    tol: float, optional
+      Tolerance parameter for nullity. Default=1e-12.
+
     Returns
     -------
     ret : numpy array, shape = m.shape
@@ -111,7 +116,7 @@ def _t_score_with_covars_and_normalized_design(tested_vars, target_vars,
     target_vars : array-like, shape=(n_samples, n_target_vars)
       Targets variates. F-ordered is better for efficient computation.
 
-    covars_orthonormalized : array-like, shape=(n_samples, n_covars) or None
+    covars_orthonormalized : array-like, shape=(n_samples, n_covars) or None, optional
       Confounding variates.
 
     Returns
@@ -138,10 +143,10 @@ def _t_score_with_covars_and_normalized_design(tested_vars, target_vars,
     return beta_targetvars_testedvars * np.sqrt((dof - 1.) / rss)
 
 
-def _permuted_ols_on_chunk(scores_original_data, tested_vars, target_vars,
-                           confounding_vars=None, n_perm_chunk=10000,
+def _permuted_ols_on_chunk(scores_original_data, tested_vars, target_vars, thread_id,
+                           confounding_vars=None, n_perm=10000, n_perm_chunk=10000,
                            intercept_test=True, two_sided_test=True,
-                           random_state=None):
+                           random_state=None, verbose=0):
     """Massively univariate group analysis with permuted OLS on a data chunk.
 
     To be used in a parallel computing context.
@@ -157,26 +162,38 @@ def _permuted_ols_on_chunk(scores_original_data, tested_vars, target_vars,
     target_vars : array-like, shape=(n_samples, n_targets)
       fMRI data. F-ordered for efficient computations.
 
-    confounding_vars : array-like, shape=(n_samples, n_covars)
+    thread_id : int
+        process id, used for display.
+
+    confounding_vars : array-like, shape=(n_samples, n_covars), optional
       Clinical data (covariates).
 
-    n_perm_chunk : int,
-      Number of permutations to be performed.
+    n_perm : int, optional
+      Total number of permutations to perform, only used for
+      display in this function. Default=10000.
 
-    intercept_test : boolean,
+    n_perm_chunk : int, optional
+      Number of permutations to be performed. Default=10000.
+
+    intercept_test : boolean, optional
       Change the permutation scheme (swap signs for intercept,
-      switch labels otherwise). See [1]
+      switch labels otherwise). See [1]_.
+      Default=True.
 
-    two_sided_test : boolean,
+    two_sided_test : boolean, optional
       If True, performs an unsigned t-test. Both positive and negative
       effects are considered; the null hypothesis is that the effect is zero.
       If False, only positive effects are considered as relevant. The null
       hypothesis is that the effect is zero or negative.
+      Default=True
 
-    random_state : int or None,
+    random_state : int or None, optional
       Seed for random number generator, to have the same permutations
       in each computing units.
 
+    verbose : int, optional
+      Defines the verbosity level. Default=0.
+
     Returns
     -------
     h0_fmax_part : array-like, shape=(n_perm_chunk, )
@@ -185,7 +202,7 @@ def _permuted_ols_on_chunk(scores_original_data, tested_vars, target_vars,
 
     References
     ----------
-    [1] Fisher, R. A. (1935). The design of experiments.
+    .. [1] Fisher, R. A. (1935). The design of experiments.
 
     """
     # initialize the seed of the random generator
@@ -195,6 +212,7 @@ def _permuted_ols_on_chunk(scores_original_data, tested_vars, target_vars,
     n_descriptors = target_vars.shape[1]
 
     # run the permutations
+    t0 = time.time()
     h0_fmax_part = np.empty((n_perm_chunk, n_regressors))
     scores_as_ranks_part = np.zeros((n_regressors, n_descriptors))
     for i in range(n_perm_chunk):
@@ -220,7 +238,7 @@ def _permuted_ols_on_chunk(scores_original_data, tested_vars, target_vars,
                                                        confounding_vars))
         if two_sided_test:
             perm_scores = np.fabs(perm_scores)
-        h0_fmax_part[i] = np.amax(perm_scores, 0)
+        h0_fmax_part[i] = np.nanmax(perm_scores, axis=0)
         # find the rank of the original scores in h0_part
         # (when n_descriptors or n_perm are large, it can be quite long to
         #  find the rank of the original scores into the whole H0 distribution.
@@ -229,6 +247,23 @@ def _permuted_ols_on_chunk(scores_original_data, tested_vars, target_vars,
         scores_as_ranks_part += (h0_fmax_part[i].reshape((-1, 1))
                                  < scores_original_data.T)
 
+        if verbose > 0:
+            step = 11 - min(verbose, 10)
+            if i % step == 0:
+                # If there is only one job, progress information is fixed
+                if n_perm == n_perm_chunk:
+                    crlf = "\r"
+                else:
+                    crlf = "\n"
+                percent = float(i) / n_perm_chunk
+                percent = round(percent * 100, 2)
+                dt = time.time() - t0
+                remaining = (100. - percent) / max(0.01, percent) * dt
+                sys.stderr.write(
+                    "Job #%d, processed %d/%d permutations "
+                    "(%0.2f%%, %i seconds remaining)%s"
+                    % (thread_id, i, n_perm_chunk, percent, remaining, crlf))
+
     return scores_as_ranks_part, h0_fmax_part.T
 
 
@@ -242,13 +277,13 @@ def permuted_ols(tested_vars, target_vars, confounding_vars=None,
     Ordinary Least Squares criterion.
     Confounding variates may be included in the model.
     Permutation testing is used to assess the significance of the relationship
-    between the tested variates and the target variates [1, 2]. A max-type
-    procedure is used to obtain family-wise corrected p-values.
+    between the tested variates and the target variates [1]_, [2]_.
+    A max-type procedure is used to obtain family-wise corrected p-values.
 
     The specific permutation scheme implemented here is the one of
-    Freedman & Lane [3]. Its has been demonstrated in [1] that this scheme
-    conveys more sensitivity than alternative schemes. This holds for
-    neuroimaging applications, as discussed in details in [2].
+    [3]_. Its has been demonstrated in [1]_ that this
+    scheme conveys more sensitivity than alternative schemes. This holds
+    for neuroimaging applications, as discussed in details in [2]_.
 
     Permutations are performed on parallel computing units. Each of them
     performs a fraction of permutations on the whole dataset. Thus, the max
@@ -267,39 +302,40 @@ def permuted_ols(tested_vars, target_vars, confounding_vars=None,
       fMRI data, trying to be explained by explanatory and confounding
       variates.
 
-    confounding_vars : array-like, shape=(n_samples, n_covars)
+    confounding_vars : array-like, shape=(n_samples, n_covars), optional
       Confounding variates (covariates), fitted but not tested.
       If None, no confounding variate is added to the model
       (except maybe a constant column according to the value of
       `model_intercept`)
 
-    model_intercept : bool,
+    model_intercept : bool, optional
       If True, a constant column is added to the confounding variates
       unless the tested variate is already the intercept.
+      Default=True
 
-    n_perm : int,
+    n_perm : int, optional
       Number of permutations to perform.
       Permutations are costly but the more are performed, the more precision
-      one gets in the p-values estimation.
+      one gets in the p-values estimation. Default=10000.
 
-    two_sided_test : boolean,
+    two_sided_test : boolean, optional
       If True, performs an unsigned t-test. Both positive and negative
       effects are considered; the null hypothesis is that the effect is zero.
       If False, only positive effects are considered as relevant. The null
-      hypothesis is that the effect is zero or negative.
+      hypothesis is that the effect is zero or negative. Default=True.
 
-    random_state : int or None,
+    random_state : int or None, optional
       Seed for random number generator, to have the same permutations
       in each computing units.
 
-    n_jobs : int,
+    n_jobs : int, optional
       Number of parallel workers.
       If 0 is provided, all CPUs are used.
       A negative number indicates that all the CPUs except (abs(n_jobs) - 1)
-      ones will be used.
+      ones will be used. Default=1.
 
-    verbose: int, optional
-        verbosity level (0 means no message).
+    verbose : int, optional
+        verbosity level (0 means no message). Default=0.
 
     Returns
     -------
@@ -320,15 +356,14 @@ def permuted_ols(tested_vars, target_vars, confounding_vars=None,
 
     References
     ----------
-    [1] Anderson, M. J. & Robinson, J. (2001).
-        Permutation tests for linear models.
-        Australian & New Zealand Journal of Statistics, 43(1), 75-88.
-    [2] Winkler, A. M. et al. (2014).
-        Permutation inference for the general linear model.
-        Neuroimage.
-    [3] Freedman, D. & Lane, D. (1983).
-        A nonstochastic interpretation of reported significance levels.
-        J. Bus. Econ. Stats., 1(4), 292-298
+    .. [1] Anderson, M. J. & Robinson, J. (2001). Permutation tests for
+       linear models. Australian & New Zealand Journal of Statistics, 43(1), 75-88.
+
+    .. [2] Winkler, A. M. et al. (2014). Permutation inference for the general
+       linear model. Neuroimage.
+
+    .. [3] Freedman, D. & Lane, D. (1983). A nonstochastic interpretation of reported
+       significance levels. J. Bus. Econ. Stats., 1(4), 292-298
 
     """
     # initialize the seed of the random generator
@@ -352,6 +387,10 @@ def permuted_ols(tested_vars, target_vars, confounding_vars=None,
                             "s" if target_vars.ndim > 1 else ""))
     target_vars = np.asfortranarray(target_vars)  # efficient for chunking
     n_descriptors = target_vars.shape[1]
+    if np.any(np.all(target_vars == 0, axis=0)):
+        warnings.warn("Some descriptors in 'target_vars' have zeros across all "
+                      "samples. These descriptors will be ignored during null "
+                      "distribution generation.")
 
     # check explanatory variates dimensions
     if tested_vars.ndim == 1:
@@ -440,17 +479,19 @@ def permuted_ols(tested_vars, target_vars, confounding_vars=None,
         if two_sided_test:
             scores_original_data = (scores_original_data
                                     * sign_scores_original_data)
-        return np.asarray([]), scores_original_data,  np.asarray([])
+        return np.asarray([]), scores_original_data.T, np.asarray([])
+
     # actual permutations, seeded from a random integer between 0 and maximum
     # value represented by np.int32 (to have a large entropy).
     ret = joblib.Parallel(n_jobs=n_jobs, verbose=verbose)(
         joblib.delayed(_permuted_ols_on_chunk)(
             scores_original_data, testedvars_resid_covars,
-            targetvars_resid_covars.T, covars_orthonormalized,
-            n_perm_chunk=n_perm_chunk, intercept_test=intercept_test,
-            two_sided_test=two_sided_test,
-            random_state=rng.random_integers(np.iinfo(np.int32).max - 1))
-        for n_perm_chunk in n_perm_chunks)
+            targetvars_resid_covars.T, thread_id + 1, covars_orthonormalized,
+            n_perm=n_perm, n_perm_chunk=n_perm_chunk,
+            intercept_test=intercept_test, two_sided_test=two_sided_test,
+            random_state=rng.randint(1, np.iinfo(np.int32).max - 1),
+            verbose=verbose)
+        for thread_id, n_perm_chunk in enumerate(n_perm_chunks))
     # reduce results
     scores_as_ranks_parts, h0_fmax_parts = zip(*ret)
     h0_fmax = np.hstack((h0_fmax_parts))
