@@ -7,6 +7,7 @@ import numpy as _np
 
 from matplotlib import cm as _cm
 from matplotlib import colors as _colors
+from matplotlib import rcParams as _rcParams
 
 ################################################################################
 # Custom colormaps for two-tailed symmetric statistics
@@ -38,16 +39,16 @@ def _pigtailed_cmap(cmap, swap_order=('green', 'red', 'blue')):
     orig_cdict = cmap._segmentdata.copy()
 
     cdict = dict()
-    cdict['green'] = [(0.5*(1-p), c1, c2)
-                        for (p, c1, c2) in reversed(orig_cdict[swap_order[0]])]
-    cdict['blue'] = [(0.5*(1-p), c1, c2)
-                        for (p, c1, c2) in reversed(orig_cdict[swap_order[1]])]
-    cdict['red'] = [(0.5*(1-p), c1, c2)
-                        for (p, c1, c2) in reversed(orig_cdict[swap_order[2]])]
+    cdict['green'] = [(0.5 * (1 - p), c1, c2)
+                      for (p, c1, c2) in reversed(orig_cdict[swap_order[0]])]
+    cdict['blue'] = [(0.5 * (1 - p), c1, c2)
+                     for (p, c1, c2) in reversed(orig_cdict[swap_order[1]])]
+    cdict['red'] = [(0.5 * (1 - p), c1, c2)
+                    for (p, c1, c2) in reversed(orig_cdict[swap_order[2]])]
 
     for color in ('red', 'green', 'blue'):
-        cdict[color].extend([(0.5*(1+p), c1, c2) 
-                                    for (p, c1, c2) in orig_cdict[color]])
+        cdict[color].extend([(0.5 * (1 + p), c1, c2)
+                             for (p, c1, c2) in orig_cdict[color]])
 
     return cdict
 
@@ -88,13 +89,23 @@ def _concat_cmap(cmap1, cmap2):
 
 def alpha_cmap(color, name='', alpha_min=0.5, alpha_max=1.):
     """ Return a colormap with the given color, and alpha going from
-        zero to 1.
+    zero to 1.
+
+    Parameters
+    ----------
+    color : (r, g, b), or a string
+        A triplet of floats ranging from 0 to 1, or a matplotlib
+        color string.
+
+    name : string, optional
+        Name of the colormap. Default=''.
+
+    alpha_min : Float, optional
+        Minimum value for alpha. Default=0.5.
+
+    alpha_max : Float, optional
+        Maximum value for alpha. Default=1.0.
 
-        Parameters
-        ----------
-        color: (r, g, b), or a string
-            A triplet of floats ranging from 0 to 1, or a matplotlib
-            color string
     """
     red, green, blue = _colors.colorConverter.to_rgb(color)
     if name == '' and hasattr(color, 'startswith'):
@@ -103,7 +114,7 @@ def alpha_cmap(color, name='', alpha_min=0.5, alpha_max=1.):
                 (red, green, blue, 1.),
                ]
     cmap = _colors.LinearSegmentedColormap.from_list(
-                                '%s_transparent' % name, cmapspec, _cm.LUTSIZE)
+        '%s_transparent' % name, cmapspec, _rcParams['image.lut'])
     cmap._init()
     cmap._lut[:, -1] = _np.linspace(alpha_min, alpha_max, cmap._lut.shape[0])
     cmap._lut[-1, -1] = 0
@@ -149,19 +160,28 @@ _bwr_data = ((0.0, 0.0, 1.0), (1.0, 1.0, 1.0), (1.0, 0.0, 0.0))
 _cmaps_data['bwr'] = _colors.LinearSegmentedColormap.from_list(
     'bwr', _bwr_data)._segmentdata.copy()
 
+
 ################################################################################
 # Build colormaps and their reverse.
+
+# backported and adapted from matplotlib since it's deprecated in 3.2
+def _revcmap(data):
+    data_r = {}
+    for key, val in data.items():
+        data_r[key] = [(1.0 - x, y1, y0) for x, y0, y1 in reversed(val)]
+    return data_r
+
+
 _cmap_d = dict()
 
 for _cmapname in list(_cmaps_data.keys()):  # needed as dict changes within loop
     _cmapname_r = _cmapname + '_r'
     _cmapspec = _cmaps_data[_cmapname]
-    _cmaps_data[_cmapname_r] = _cm.revcmap(_cmapspec)
+    _cmaps_data[_cmapname_r] = _revcmap(_cmapspec)
     _cmap_d[_cmapname] = _colors.LinearSegmentedColormap(
-                            _cmapname, _cmapspec, _cm.LUTSIZE)
+        _cmapname, _cmapspec, _rcParams['image.lut'])
     _cmap_d[_cmapname_r] = _colors.LinearSegmentedColormap(
-                            _cmapname_r, _cmaps_data[_cmapname_r],
-                            _cm.LUTSIZE)
+        _cmapname_r, _cmaps_data[_cmapname_r], _rcParams['image.lut'])
 
 ################################################################################
 # A few transparent colormaps
@@ -176,7 +196,7 @@ for color, name in (((1, 0, 0), 'red'),
 
 ###############################################################################
 # HCP Connectome Workbench colormaps
-# As seen in  https://github.com/Washington-University/workbench src/Pallete
+# As seen in  https://github.com/Washington-University/workbench src/Palette
 roy_big_bl = _np.array([(255, 255, 0), (255, 200, 0),
                         (255, 120, 0), (255, 0, 0),
                         (200, 0, 0), (150, 0, 0),
@@ -199,10 +219,13 @@ _cmap_d['videen_style'] = _colors.LinearSegmentedColormap.from_list(
     'videen_style', videen_style)
 
 # Save colormaps in the scope of the module
-locals().update(_cmap_d)
+globals().update(_cmap_d)
 # Register cmaps in matplotlib too
 for k, v in _cmap_d.items():
-    _cm.register_cmap(name=k, cmap=v)
+    try:  # "bwr" is in latest matplotlib
+        _cm.register_cmap(name=k, cmap=v)
+    except ValueError:
+        pass
 
 
 ################################################################################
@@ -212,13 +235,15 @@ for k, v in _cmap_d.items():
 def dim_cmap(cmap, factor=.3, to_white=True):
     """ Dim a colormap to white, or to black.
     """
-    assert factor >= 0 and factor <=1, ValueError(
-            'Dimming factor must be larger than 0 and smaller than 1, %s was passed.' 
-                                                        % factor)
+    assert 0 <= factor <= 1, ValueError(
+        'Dimming factor must be larger than 0 and smaller than 1, %s was '
+        'passed.' % factor)
     if to_white:
-        dimmer = lambda c: 1 - factor*(1-c)
+        def dimmer(c):
+            return 1 - factor * (1 - c)
     else:
-        dimmer = lambda c: factor*c
+        def dimmer(c):
+            return factor * c
     cdict = cmap._segmentdata.copy()
     for c_index, color in enumerate(('red', 'green', 'blue')):
         color_lst = list()
@@ -227,19 +252,17 @@ def dim_cmap(cmap, factor=.3, to_white=True):
         cdict[color] = color_lst
 
     return _colors.LinearSegmentedColormap(
-                                '%s_dimmed' % cmap.name,
-                                cdict,
-                                _cm.LUTSIZE)
+        '%s_dimmed' % cmap.name, cdict, _rcParams['image.lut'])
 
 
 def replace_inside(outer_cmap, inner_cmap, vmin, vmax):
     """ Replace a colormap by another inside a pair of values.
     """
     assert vmin < vmax, ValueError('vmin must be smaller than vmax')
-    assert vmin >= 0,    ValueError('vmin must be larger than 0, %s was passed.' 
-                                        % vmin)
-    assert vmax <= 1,    ValueError('vmax must be smaller than 1, %s was passed.' 
-                                        % vmax)
+    assert vmin >= 0, ValueError('vmin must be larger than 0, %s was passed.'
+                                 % vmin)
+    assert vmax <= 1, ValueError('vmax must be smaller than 1, %s was passed.'
+                                 % vmax)
     outer_cdict = outer_cmap._segmentdata.copy()
     inner_cdict = inner_cmap._segmentdata.copy()
 
@@ -257,7 +280,6 @@ def replace_inside(outer_cmap, inner_cmap, vmin, vmax):
                 this_cdict['green'].append((p, g, g))
                 this_cdict['blue'].append((p, b, b))
 
-
     for c_index, color in enumerate(('red', 'green', 'blue')):
         color_lst = list()
 
@@ -266,8 +288,9 @@ def replace_inside(outer_cmap, inner_cmap, vmin, vmax):
                 break
             color_lst.append((value, c1, c2))
 
-        color_lst.append((vmin, outer_cmap(vmin)[c_index], 
-                                inner_cmap(vmin)[c_index]))
+        color_lst.append((vmin,
+                          outer_cmap(vmin)[c_index],
+                          inner_cmap(vmin)[c_index]))
 
         for value, c1, c2 in inner_cdict[color]:
             if value <= vmin:
@@ -276,8 +299,9 @@ def replace_inside(outer_cmap, inner_cmap, vmin, vmax):
                 break
             color_lst.append((value, c1, c2))
 
-        color_lst.append((vmax, inner_cmap(vmax)[c_index],
-                                outer_cmap(vmax)[c_index]))
+        color_lst.append((vmax,
+                          inner_cmap(vmax)[c_index],
+                          outer_cmap(vmax)[c_index]))
 
         for value, c1, c2 in outer_cdict[color]:
             if value <= vmax:
@@ -287,8 +311,5 @@ def replace_inside(outer_cmap, inner_cmap, vmin, vmax):
         cdict[color] = color_lst
 
     return _colors.LinearSegmentedColormap(
-                                '%s_inside_%s' % (inner_cmap.name, outer_cmap.name),
-                                cdict,
-                                _cm.LUTSIZE)
-
-
+        '%s_inside_%s' % (inner_cmap.name, outer_cmap.name),
+        cdict, _rcParams['image.lut'])
