@@ -1,47 +1,68 @@
 """Parcellation tools such as KMeans or Ward for fMRI images
 """
 
+import warnings
 import numpy as np
 
 from sklearn.base import clone
 from sklearn.feature_extraction import image
-from sklearn.externals.joblib import Memory, delayed, Parallel
+from joblib import Memory, delayed, Parallel
 
+from .rena_clustering import ReNA
 from ..decomposition.multi_pca import MultiPCA
 from ..input_data import NiftiLabelsMasker
-from .._utils.compat import _basestring
 from .._utils.niimg import _safe_get_data
 from .._utils.niimg_conversions import _iter_check_niimg
+from .._utils import fill_doc
 
 
-def _estimator_fit(data, estimator):
-    """ Estimator to fit on the data matrix
+def _estimator_fit(data, estimator, method=None):
+    """Estimator to fit on the data matrix
 
     Parameters
     ----------
     data : numpy array
-        Data matrix
+        Data matrix.
 
     estimator : instance of estimator from sklearn
-        MiniBatchKMeans or AgglomerativeClustering
+        MiniBatchKMeans or AgglomerativeClustering.
+
+    method : str, {'kmeans', 'ward', 'complete', 'average', 'rena'}, optional
+        A method to choose between for brain parcellations.
 
     Returns
     -------
     labels_ : numpy.ndarray
-        labels_ estimated from estimator
-    """
-    estimator = clone(estimator)
-    estimator.fit(data.T)
+        labels_ estimated from estimator.
 
-    return estimator.labels_
+    """
+    if method == 'rena':
+        rena = ReNA(mask_img=estimator.mask_img,
+                    n_clusters=estimator.n_clusters,
+                    scaling=estimator.scaling,
+                    n_iter=estimator.n_iter,
+                    threshold=estimator.threshold,
+                    memory=estimator.memory,
+                    memory_level=estimator.memory_level,
+                    verbose=estimator.verbose)
+        rena.fit(data)
+        labels_ = rena.labels_
+
+    else:
+        estimator = clone(estimator)
+        estimator.fit(data.T)
+        labels_ = estimator.labels_
+
+    return labels_
 
 
 def _check_parameters_transform(imgs, confounds):
     """A helper function to check the parameters and prepare for processing
     as a list.
+
     """
     if not isinstance(imgs, (list, tuple)) or \
-            isinstance(imgs, _basestring):
+            isinstance(imgs, str):
         imgs = [imgs, ]
         single_subject = True
     elif isinstance(imgs, (list, tuple)) and len(imgs) == 1:
@@ -54,7 +75,7 @@ def _check_parameters_transform(imgs, confounds):
 
     if confounds is not None:
         if not isinstance(confounds, (list, tuple)) or \
-                isinstance(confounds, _basestring):
+                isinstance(confounds, str):
             confounds = [confounds, ]
 
     if len(confounds) != len(imgs):
@@ -64,7 +85,7 @@ def _check_parameters_transform(imgs, confounds):
 
 
 def _labels_masker_extraction(img, masker, confound):
-    """ Helper function for parallelizing NiftiLabelsMasker extractor
+    """Helper function for parallelizing NiftiLabelsMasker extractor
     on list of Nifti images.
 
     Parameters
@@ -73,44 +94,53 @@ def _labels_masker_extraction(img, masker, confound):
         Image to process.
 
     masker : instance of NiftiLabelsMasker
-        Used for extracting signals with fit_transform
+        Used for extracting signals with fit_transform.
 
-    confound : csv file or numpy array
+    confound : csv file, numpy ndarray or pandas DataFrame
         Confound used for signal cleaning while extraction.
-        Passed to signal.clean
+        Passed to signal.clean.
 
     Returns
     -------
     signals : numpy array
-        Signals extracted on given img
+        Signals extracted on given img.
+
     """
     masker = clone(masker)
     signals = masker.fit_transform(img, confounds=confound)
     return signals
 
 
+@fill_doc
 class Parcellations(MultiPCA):
-    """Learn parcellations on fMRI images.
+    """Learn :term:`parcellations<parcellation>`
+    on :term:`fMRI` images.
 
-    Four different types of clustering methods can be used such as kmeans,
-    ward, complete, average. Kmeans will call MiniBatchKMeans whereas
-    ward, complete, average are used within in Agglomerative Clustering.
-    All methods are leveraged from scikit-learn.
+    Five different types of clustering methods can be used:
+    kmeans, ward, complete, average and rena.
+    kmeans will call MiniBatchKMeans whereas
+    ward, complete, average are used within in Agglomerative Clustering and
+    rena will call ReNA.
+    kmeans, ward, complete, average are leveraged from scikit-learn.
+    rena is built into nilearn.
 
     .. versionadded:: 0.4.1
 
     Parameters
     ----------
-    method : str, {'kmeans', 'ward', 'complete', 'average'}
+    method : :obj:`str`, {'kmeans', 'ward', 'complete', 'average', 'rena'}
         A method to choose between for brain parcellations.
+        For a small number of parcels, kmeans is usually advisable.
+        For a large number of parcellations (several hundreds, or thousands),
+        ward and rena are the best options. Ward will give higher quality
+        parcels, but with increased computation time. ReNA is most useful as a
+        fast data-reduction step, typically dividing the signal size by ten.
+    %(n_parcels)s
+    %(random_state)s
+        Default=0.
 
-    n_parcels : int, default=50
-        Number of parcellations to divide the brain data into.
-
-    random_state : int or RandomState
-        Pseudo number generator state used for random sampling.
-
-    mask : Niimg-like object or NiftiMasker, MultiNiftiMasker instance
+    mask : Niimg-like object or :class:`nilearn.input_data.NiftiMasker`,\
+ :class:`nilearn.input_data.MultiNiftiMasker`, optional
         Mask/Masker used for masking the data.
         If mask image if provided, it will be used in the MultiNiftiMasker.
         If an instance of MultiNiftiMasker is provided, then this instance
@@ -118,83 +148,107 @@ class Parcellations(MultiPCA):
         masker parameters.
         If None, mask will be automatically computed by a MultiNiftiMasker
         with default parameters.
+    %(smoothing_fwhm)s
+        Default=4.0.
+    %(standardize_false)s
+    %(detrend)s
 
-    smoothing_fwhm : float, optional default=4.
-        If smoothing_fwhm is not None, it gives the full-width half maximum in
-        millimeters of the spatial smoothing to apply to the signal.
+        .. note::
+            This parameter is passed to :func:`nilearn.signal.clean`.
+            Please see the related documentation for details.
 
-    standardize : boolean, optional
-        If standardize is True, the time-series are centered and normed:
-        their mean is put to 0 and their variance to 1 in the time dimension.
+        Default=False.
+    %(low_pass)s
 
-    detrend : boolean, optional
-        Whether to detrend signals or not.
-        This parameter is passed to signal.clean. Please see the related
-        documentation for details
+        .. note::
+            This parameter is passed to :func:`nilearn.signal.clean`.
+            Please see the related documentation for details.
 
-    low_pass: None or float, optional
-        This parameter is passed to signal.clean. Please see the related
-        documentation for details
+    %(high_pass)s
 
-    high_pass: None or float, optional
-        This parameter is passed to signal.clean. Please see the related
-        documentation for details
+        .. note::
+            This parameter is passed to :func:`nilearn.signal.clean`.
+            Please see the related documentation for details.
 
-    t_r : float, optional
-        This parameter is passed to signal.clean. Please see the related
-        documentation for details
+    %(t_r)s
 
-    target_affine : 3x3 or 4x4 matrix, optional
-        This parameter is passed to image.resample_img. Please see the
-        related documentation for details. The given affine will be
-        considered as same for all given list of images.
+        .. note::
+            This parameter is passed to :func:`nilearn.signal.clean`.
+            Please see the related documentation for details.
 
-    target_shape : 3-tuple of integers, optional
-        This parameter is passed to image.resample_img. Please see the
-        related documentation for details.
+    %(target_affine)s
 
-    memory : instance of joblib.Memory or str
-        Used to cache the masking process.
-        By default, no caching is done. If a string is given, it is the
-        path to the caching directory.
+        .. note::
+            This parameter is passed to :func:`nilearn.image.resample_img`.
+            Please see the related documentation for details.
 
-    memory_level : integer, optional
-        Rough estimator of the amount of memory used by caching. Higher value
-        means more memory for caching.
+        .. note::
+            The given affine will be considered as same for all
+            given list of images.
 
-    n_jobs : integer, optional
-        The number of CPUs to use to do the computation. -1 means
-        'all CPUs', -2 'all CPUs but one', and so on.
+    %(target_shape)s
 
-    verbose : integer, optional
-        Indicate the level of verbosity. By default, nothing is printed.
+        .. note::
+            This parameter is passed to :func:`nilearn.image.resample_img`.
+            Please see the related documentation for details.
 
-    Returns
-    -------
-    labels_img_ : Nifti1Image
+    %(mask_strategy)s
+
+        .. note::
+             Depending on this value, the mask will be computed from
+             :func:`nilearn.masking.compute_background_mask`,
+             :func:`nilearn.masking.compute_epi_mask`, or
+             :func:`nilearn.masking.compute_brain_mask`.
+
+        Default='epi'.
+
+    mask_args : :obj:`dict`, optional
+        If mask is None, these are additional parameters passed to
+        masking.compute_background_mask or masking.compute_epi_mask
+        to fine-tune mask computation. Please see the related documentation
+        for details.
+
+    scaling : :obj:`bool`, optional
+        Used only when the method selected is 'rena'. If scaling is True, each
+        cluster is scaled by the square root of its size, preserving the
+        l2-norm of the image. Default=False.
+
+    n_iter : :obj:`int`, optional
+        Used only when the method selected is 'rena'. Number of iterations of
+        the recursive neighbor agglomeration. Default=10.
+    %(memory)s
+    %(memory_level)s
+    %(n_jobs)s
+    %(verbose0)s
+
+    Attributes
+    ----------
+    `labels_img_` : :class:`nibabel.nifti1.Nifti1Image`
         Labels image to each parcellation learned on fmri images.
 
-    masker_ : instance of NiftiMasker or MultiNiftiMasker
-        The masker used to mask the data
+    `masker_` : :class:`nilearn.input_data.NiftiMasker` or\
+ :class:`nilearn.input_data.MultiNiftiMasker`
+        The masker used to mask the data.
 
-    connectivity_ : numpy.ndarray
-        voxel-to-voxel connectivity matrix computed from a mask.
+    `connectivity_` : :class:`numpy.ndarray`
+        Voxel-to-voxel connectivity matrix computed from a mask.
         Note that this attribute is only seen if selected methods are
         Agglomerative Clustering type, 'ward', 'complete', 'average'.
 
     Notes
     -----
-        * Transforming list of Nifti images to data matrix takes few steps.
-          Reducing the data dimensionality using randomized SVD, build brain
-          parcellations using KMeans or various Agglomerative methods.
+    * Transforming list of Nifti images to data matrix takes few steps.
+      Reducing the data dimensionality using randomized SVD, build brain
+      parcellations using KMeans or various Agglomerative methods.
 
-        * This object uses spatially-constrained AgglomerativeClustering for
-          method='ward' or 'complete' or 'average'. Spatial connectivity matrix
-          (voxel-to-voxel) is built-in object which means no need of explicitly
-          giving the matrix.
+    * This object uses spatially-constrained AgglomerativeClustering for
+      method='ward' or 'complete' or 'average' and spatially-constrained
+      ReNA clustering for method='rena'. Spatial connectivity matrix
+      (voxel-to-voxel) is built-in object which means no need of explicitly
+      giving the matrix.
 
     """
-    VALID_METHODS = ['kmeans', 'ward', 'complete', 'average']
+    VALID_METHODS = ['kmeans', 'ward', 'complete', 'average', 'rena']
 
     def __init__(self, method, n_parcels=50,
                  random_state=0, mask=None, smoothing_fwhm=4.,
@@ -202,10 +256,14 @@ class Parcellations(MultiPCA):
                  low_pass=None, high_pass=None, t_r=None,
                  target_affine=None, target_shape=None,
                  mask_strategy='epi', mask_args=None,
-                 memory=Memory(cachedir=None),
+                 scaling=False, n_iter=10,
+                 memory=Memory(location=None),
                  memory_level=0, n_jobs=1, verbose=1):
+
         self.method = method
         self.n_parcels = n_parcels
+        self.scaling = scaling
+        self.n_iter = n_iter
 
         MultiPCA.__init__(self, n_components=200,
                           random_state=random_state,
@@ -222,7 +280,7 @@ class Parcellations(MultiPCA):
                           verbose=verbose)
 
     def _raw_fit(self, data):
-        """ Fits the parcellation method on this reduced data.
+        """Fits the parcellation method on this reduced data.
 
         Data are coming from a base decomposition estimator which computes
         the mask and reduces the dimensionality of images using
@@ -230,18 +288,19 @@ class Parcellations(MultiPCA):
 
         Parameters
         ----------
-        data : ndarray
+        data : :class:`numpy.ndarray`
             Shape (n_samples, n_features)
 
         Returns
         -------
-        labels_ : numpy.ndarray
+        labels : :class:`numpy.ndarray`
             Labels to each cluster in the brain.
 
-        connectivity_ : numpy.ndarray
-            voxel-to-voxel connectivity matrix computed from a mask.
+        connectivity : :class:`numpy.ndarray`
+            Voxel-to-voxel connectivity matrix computed from a mask.
             Note that, this attribute is returned only for selected methods
             such as 'ward', 'complete', 'average'.
+
         """
         valid_methods = self.VALID_METHODS
         if self.method is None:
@@ -262,7 +321,7 @@ class Parcellations(MultiPCA):
         # happening in Travis
         try:
             from nilearn import plotting
-        except:
+        except Exception:
             pass
 
         components = MultiPCA._raw_fit(self, data)
@@ -280,8 +339,19 @@ class Parcellations(MultiPCA):
                                      verbose=max(0, self.verbose - 1))
             labels = self._cache(_estimator_fit,
                                  func_memory_level=1)(components.T, kmeans)
+
+        elif self.method == 'rena':
+            rena = ReNA(mask_img_, n_clusters=self.n_parcels,
+                        scaling=self.scaling, n_iter=self.n_iter,
+                        memory=self.memory, memory_level=self.memory_level,
+                        verbose=max(0, self.verbose - 1))
+            method = 'rena'
+            labels = \
+                self._cache(_estimator_fit, func_memory_level=1)(components.T,
+                                                                 rena, method)
+
         else:
-            mask_ = _safe_get_data(mask_img_).astype(np.bool)
+            mask_ = _safe_get_data(mask_img_).astype(bool)
             shape = mask_.shape
             connectivity = image.grid_to_graph(n_x=shape[0], n_y=shape[1],
                                                n_z=shape[2], mask=mask_)
@@ -299,6 +369,14 @@ class Parcellations(MultiPCA):
             self.connectivity_ = connectivity
         # Avoid 0 label
         labels = labels + 1
+        unique_labels = np.unique(labels)
+
+        # Check that appropriate number of labels were created
+        if len(unique_labels) != self.n_parcels:
+            n_parcels_warning = ('The number of generated labels does not '
+                                 'match the requested number of parcels.')
+            warnings.warn(message=n_parcels_warning, category=UserWarning,
+                          stacklevel=3)
         self.labels_img_ = self.masker_.inverse_transform(labels)
 
         return self
@@ -310,27 +388,33 @@ class Parcellations(MultiPCA):
             raise ValueError("Object has no labels_img_ attribute. "
                              "Ensure that fit() is called before transform.")
 
+    @fill_doc
     def transform(self, imgs, confounds=None):
-        """Extract signals from parcellations learned on fmri images.
+        """Extract signals from :term:`parcellations<parcellation>` learned
+        on :term:`fMRI` images.
 
         Parameters
         ----------
-        imgs : List of Nifti-like images
-            See http://nilearn.github.io/manipulating_images/input_output.html.
+        %(imgs)s
             Images to process.
 
-        confounds: List of CSV files or arrays-like, optional
+        confounds : :obj:`list` of CSV files, arrays-like,\
+ or :class:`pandas.DataFrame`, optional
             Each file or numpy array in a list should have shape
             (number of scans, number of confounds)
-            This parameter is passed to signal.clean. Please see the related
-            documentation for details. Must be of same length of imgs.
+            Must be of same length as imgs.
+
+            .. note::
+                This parameter is passed to :func:`nilearn.signal.clean`.
+                Please see the related documentation for details.
 
         Returns
         -------
-        region_signals: List of or 2D numpy.ndarray
+        region_signals : :obj:`list` of or 2D :class:`numpy.ndarray`
             Signals extracted for each label for each image.
             Example, for single image shape will be
             (number of scans, number of labels)
+
         """
         self._check_fitted()
         imgs, confounds, single_subject = _check_parameters_transform(
@@ -362,49 +446,58 @@ class Parcellations(MultiPCA):
         else:
             return region_signals
 
+    @fill_doc
     def fit_transform(self, imgs, confounds=None):
-        """Fit the images to parcellations and then transform them.
+        """Fit the images to :term:`parcellations<parcellation>` and
+        then transform them.
 
         Parameters
         ----------
-        imgs : List of Nifti-like images
-            See http://nilearn.github.io/manipulating_images/input_output.html.
+        %(imgs)s
             Images for process for fit as well for transform to signals.
 
-        confounds : List of CSV files or arrays-like, optional
+        confounds : :obj:`list` of CSV files, arrays-like or\
+ :class:`pandas.DataFrame`, optional
             Each file or numpy array in a list should have shape
             (number of scans, number of confounds).
-            This parameter is passed to signal.clean. Given confounds
-            should have same length as images if given as a list.
+            Given confounds should have same length as images if
+            given as a list.
 
-            Note: same confounds will used for cleaning signals before
-            learning parcellations.
+            .. note::
+                This parameter is passed to :func:`nilearn.signal.clean`.
+                Please see the related documentation for details.
+
+            .. note::
+                Confounds will be used for cleaning signals before
+                learning parcellations.
 
         Returns
         -------
-        region_signals: List of or 2D numpy.ndarray
+        region_signals : :obj:`list` of or 2D :class:`numpy.ndarray`
             Signals extracted for each label for each image.
             Example, for single image shape will be
             (number of scans, number of labels)
-        """
-        return self.fit(imgs, confounds=confounds).transform(imgs,
-                                                             confounds=confounds)
 
+        """
+        return self.fit(imgs, confounds=confounds).transform(imgs, confounds)
+
+    @fill_doc
     def inverse_transform(self, signals):
-        """Transform signals extracted from parcellations back to brain
-        images.
+        """Transform signals extracted from :term:`parcellations<parcellation>`
+        back to brain images.
 
         Uses `labels_img_` (parcellations) built at fit() level.
 
         Parameters
         ----------
-        signals : List of 2D numpy.ndarray
-            Each 2D array with shape (number of scans, number of regions)
+        signals : :obj:`list` of 2D :class:`numpy.ndarray`
+            Each 2D array with shape (number of scans, number of regions).
 
         Returns
         -------
-        imgs : List of or Nifti-like image
-            Brain image(s)
+        %(imgs)s
+            Brain image(s).
+
         """
         from .signal_extraction import signals_to_img_labels
 
