@@ -3,28 +3,28 @@ Better brain parcellations for Region of Interest analysis
 """
 
 import numbers
-import collections
+import collections.abc
 import numpy as np
 
 from scipy import ndimage
 from scipy.stats import scoreatpercentile
 
-from sklearn.externals.joblib import Memory
+from joblib import Memory
 
 from .. import masking
 from ..input_data import NiftiMapsMasker
-from .._utils import check_niimg, check_niimg_3d, check_niimg_4d
+from .._utils import (check_niimg, check_niimg_3d,
+                      check_niimg_4d, fill_doc)
 from ..image import new_img_like, resample_img
 from ..image.image import _smooth_array, threshold_img
 from .._utils.niimg_conversions import concat_niimgs, _check_same_fov
 from .._utils.niimg import _safe_get_data
-from .._utils.compat import _basestring
 from .._utils.ndimage import _peak_local_max
 from .._utils.segmentation import _random_walker
 
 
 def _threshold_maps_ratio(maps_img, threshold):
-    """ Automatic thresholding of atlas maps image.
+    """Automatic thresholding of atlas maps image.
 
     Considers the given threshold as a ratio to the total number of voxels
     in the brain volume. This gives a certain number within the data
@@ -33,17 +33,19 @@ def _threshold_maps_ratio(maps_img, threshold):
 
     Parameters
     ----------
-    maps_img: Niimg-like object
-        an image of brain atlas maps.
-    threshold: float
+    maps_img : Niimg-like object
+        An image of brain atlas maps.
+
+    threshold : float
         If float, value is used as a ratio to n_voxels to get a certain threshold
         size in number to threshold the image. The value should be positive and
         within the range of number of maps (i.e. n_maps in 4th dimension).
 
     Returns
     -------
-    threshold_maps_img: Nifti1Image
-        gives us thresholded image.
+    threshold_maps_img : Nifti1Image
+        Gives us thresholded image.
+
     """
     maps = check_niimg(maps_img)
     n_maps = maps.shape[-1]
@@ -55,7 +57,8 @@ def _threshold_maps_ratio(maps_img, threshold):
     else:
         ratio = threshold
 
-    maps_data = _safe_get_data(maps, ensure_finite=True).copy()
+    # Get a copy of the data
+    maps_data = _safe_get_data(maps, ensure_finite=True, copy_data=True)
 
     abs_maps = np.abs(maps_data)
     # thresholding
@@ -78,12 +81,12 @@ def _remove_small_regions(input_data, index, affine, min_size):
     input_data : numpy.ndarray
         Values inside the regions defined by labels contained in input_data
         are summed together to get the size and compare with given min_size.
-        For example, see scipy.ndimage.label
+        For example, see scipy.ndimage.label.
 
     index : numpy.ndarray
         A sequence of label numbers of the regions to be measured corresponding
         to input_data. For example, sequence can be generated using
-        np.arange(n_labels + 1)
+        np.arange(n_labels + 1).
 
     affine : numpy.ndarray
         Affine of input_data is used to convert size in voxels to size in
@@ -99,6 +102,7 @@ def _remove_small_regions(input_data, index, affine, min_size):
         Data returned will have regions removed specified by min_size
         Otherwise, if criterion is not met then same input data will be
         returned.
+
     """
     # with return_counts argument is introduced from numpy 1.9.0.
     # _, region_sizes = np.unique(input_data, return_counts=True)
@@ -123,52 +127,47 @@ def _remove_small_regions(input_data, index, affine, min_size):
     return input_data
 
 
+@fill_doc
 def connected_regions(maps_img, min_region_size=1350,
                       extract_type='local_regions', smoothing_fwhm=6,
                       mask_img=None):
-    """ Extraction of brain connected regions into separate regions.
+    """Extraction of brain connected regions into separate regions.
 
-    Note: the region size should be defined in mm^3. See the documentation for
-    more details.
+    .. note::
+        The region size should be defined in mm^3.
+        See the documentation for more details.
 
     .. versionadded:: 0.2
 
     Parameters
     ----------
-    maps_img: Niimg-like object
-        an image of brain activation or atlas maps to be extracted into set of
+    maps_img : Niimg-like object
+        An image of brain activation or atlas maps to be extracted into set of
         separate brain regions.
 
-    min_region_size: int, default 1350 mm^3, optional
+    min_region_size : :obj:`float`, optional
         Minimum volume in mm3 for a region to be kept. For example, if the voxel
-        size is 3x3x3 mm then the volume of the voxel is 27mm^3. By default, it
-        is 1350mm^3 which means we take minimum size of 1350 / 27 = 50 voxels.
+        size is 3x3x3 mm then the volume of the voxel is 27mm^3.
+        Default=1350mm^3, which means we take minimum size of 1350 / 27 = 50 voxels.
 
-    extract_type: str {'connected_components', 'local_regions'} \
-        default local_regions, optional
-        If 'connected_components', each component/region in the image is extracted
-        automatically by labelling each region based upon the presence of unique
-        features in their respective regions.
-        If 'local_regions', each component/region is extracted based on their
-        maximum peak value to define a seed marker and then using random walker
-        segementation algorithm on these markers for region separation.
-
-    smoothing_fwhm: scalar, default 6mm, optional
+    %(extract_type)s
+    smoothing_fwhm : :obj:`float`, optional
         To smooth an image to extract most sparser regions. This parameter
         is passed `_smooth_array` and exists only for extract_type 'local_regions'.
+        Default=6.
 
-    mask_img: Niimg-like object, default None
+    mask_img : Niimg-like object, optional
         If given, mask image is applied to input data.
         If None, no masking is applied.
 
     Returns
     -------
-    regions_extracted_img: Nifti1Image
-        gives the image in 4D of extracted brain regions. Each 3D image consists
+    regions_extracted_img : :class:`nibabel.nifti1.Nifti1Image`
+        Gives the image in 4D of extracted brain regions. Each 3D image consists
         of only one separated region.
 
-    index_of_each_map: numpy array
-        an array of list of indices where each index denotes the identity
+    index_of_each_map : :class:`numpy.ndarray`
+        An array of list of indices where each index denotes the identity
         of each extracted region to their family of brain maps.
 
     See Also
@@ -179,11 +178,12 @@ def connected_regions(maps_img, min_region_size=1350,
     nilearn.regions.RegionExtractor : A class can be used for both
         region extraction on continuous type atlas images and
         also time series signals extraction from regions extracted.
+
     """
     all_regions_imgs = []
     index_of_each_map = []
     maps_img = check_niimg(maps_img, atleast_4d=True)
-    maps = _safe_get_data(maps_img).copy()
+    maps = _safe_get_data(maps_img, copy_data=True)
     affine = maps_img.affine
     min_region_size = min_region_size / np.abs(np.linalg.det(affine[:3, :3]))
 
@@ -239,6 +239,7 @@ def connected_regions(maps_img, min_region_size=1350,
     return regions_extracted_img, index_of_each_map
 
 
+@fill_doc
 class RegionExtractor(NiftiMapsMasker):
     """Class for brain region extraction.
 
@@ -248,29 +249,32 @@ class RegionExtractor(NiftiMapsMasker):
     Particularly, to show that each decomposed brain maps can be
     used to focus on a target specific Regions of Interest analysis.
 
+    See [1]_.
+
     .. versionadded:: 0.2
 
     Parameters
     ----------
-    maps_img: 4D Niimg-like object
+    maps_img : 4D Niimg-like object
         Image containing a set of whole brain atlas maps or statistically
         decomposed brain maps.
 
-    mask_img: Niimg-like object or None,  default None, optional
+    mask_img : Niimg-like object or None, optional
         Mask to be applied to input data, passed to NiftiMapsMasker.
         If None, no masking is applied.
 
-    min_region_size: float, default 1350 mm^3, optional
+    min_region_size : :obj:`float`, optional
         Minimum volume in mm3 for a region to be kept. For example, if
-        the voxel size is 3x3x3 mm then the volume of the voxel is
-        27mm^3. By default, it is 1350mm^3 which means we take minimum
-        size of 1350 / 27 = 50 voxels.
+        the voxel size is 3x3x3 mm then the volume of the voxel is 27mm^3.
+        Default=1350mm^3, which means we take minimum size of 1350 / 27 = 50 voxels.
 
-    threshold: number, default 1., optional
+    threshold : number, optional
         A value used either in ratio_n_voxels or img_value or percentile
         `thresholding_strategy` based upon the choice of selection.
+        Default=1.0.
 
-    thresholding_strategy: str {'ratio_n_voxels', 'img_value', 'percentile'}, optional
+    thresholding_strategy : :obj:`str` {'ratio_n_voxels', 'img_value',\
+ 'percentile'}, optional
         If default 'ratio_n_voxels', we apply thresholding that will keep
         the more intense nonzero brain voxels (denoted as n_voxels)
         across all maps (n_voxels being the number of voxels in the brain
@@ -284,77 +288,62 @@ class RegionExtractor(NiftiMapsMasker):
         intensities across all maps. A value given in `threshold`
         parameter indicates that we keep only those voxels which have
         intensities more than this value.
+        Default='ratio_n_voxels'.
 
-    extractor: str {'connected_components', 'local_regions'} default 'local_regions', optional
-        If 'connected_components', each component/region in the image is
-        extracted automatically by labelling each region based upon the
-        presence of unique features in their respective regions. If
-        'local_regions', each component/region is extracted based on
-        their maximum peak value to define a seed marker and then using
-        random walker segementation algorithm on these markers for region
-        separation.
-
-    smoothing_fwhm: scalar, default 6mm, optional
+    %(extractor)s
+    smoothing_fwhm : :obj:`float`, optional
         To smooth an image to extract most sparser regions. This parameter
         is passed to `connected_regions` and exists only for extractor
         'local_regions'. Please set this parameter according to maps
         resolution, otherwise extraction will fail.
+        Default=6mm.
+    %(standardize_false)s
 
-    standardize: bool, True or False, default False, optional
-        If True, the time series signals are centered and normalized by
-        putting their mean to 0 and variance to 1. Recommended to
-        set as True if signals are not already standardized.
-        passed to class NiftiMapsMasker.
+        .. note::
+            Recommended to set to True if signals are not already standardized.
+            Passed to :class:`nilearn.input_data.NiftiMapsMasker`.
 
-    detrend: bool, True or False, default False, optional
-        This parameter is passed to nilearn.signal.clean basically
-        indicates whether to detrend timeseries signals or not.
-        passed to class NiftiMapsMasker.
+    %(detrend)s
 
-    low_pass: float, default None, optional
-        This value will be applied on the signals by passing to signal.clean
-        Please see the related documentation signal.clean for more details.
-        passed to class NiftiMapsMasker.
+        .. note::
+            Passed to :func:`nilearn.signal.clean`.
 
-    high_pass: float, default None, optional
-        This value will be applied on the signals by passing to signal.clean
-        Please see the related documentation signal.clean for more details.
-        passed to NiftiMapsMasker.
+        Default=False.
 
-    t_r: float, default None, optional
-        Repetition time in sec. This value is given to signal.clean
-        Please see the related documentation for details.
-        passed to NiftiMapsMasker.
+    %(low_pass)s
 
-    memory: instance of joblib.Memory, string, default None, optional
-        Used to cache the masking process. If a string is given, the path
-        is set with this string as a folder name in the directory.
-        passed to NiftiMapsMasker.
+        .. note::
+            Passed to :func:`nilearn.signal.clean`.
 
-    memory_level: int, default 0, optional
-        Aggressiveness of memory catching. The higher the number, the higher
-        the number of functions that will be cached. Zero mean no caching.
-        passed to NiftiMapsMasker.
+    %(high_pass)s
 
-    verbose: int, default 0, optional
-        Indicates the level of verbosity by printing the message. Zero
-        indicates nothing is printed.
+        .. note::
+            Passed to :func:`nilearn.signal.clean`.
+
+    %(t_r)s
+
+        .. note::
+            Passed to :func:`nilearn.signal.clean`.
+
+    %(memory)s
+    %(memory_level)s
+    %(verbose0)s
 
     Attributes
     ----------
-    `index_` : numpy array
-        array of list of indices where each index value is assigned to
+    `index_` : :class:`numpy.ndarray`
+        Array of list of indices where each index value is assigned to
         each separate region of its corresponding family of brain maps.
 
-    `regions_img_` : Nifti1Image
+    `regions_img_` : :class:`nibabel.nifti1.Nifti1Image`
         List of separated regions with each region lying on an
         original volume concatenated into a 4D image.
 
     References
     ----------
-    * Abraham et al. "Region segmentation for sparse decompositions:
-      better brain parcellations from rest fMRI", Sparsity Techniques in
-      Medical Imaging, Sep 2014, Boston, United States. pp.8
+    .. [1] Abraham et al. "Region segmentation for sparse decompositions:
+       better brain parcellations from rest fMRI", Sparsity Techniques in
+       Medical Imaging, Sep 2014, Boston, United States. pp.8
 
     See Also
     --------
@@ -367,7 +356,7 @@ class RegionExtractor(NiftiMapsMasker):
                  extractor='local_regions', smoothing_fwhm=6,
                  standardize=False, detrend=False,
                  low_pass=None, high_pass=None, t_r=None,
-                 memory=Memory(cachedir=None), memory_level=0, verbose=0):
+                 memory=Memory(location=None), memory_level=0, verbose=0):
         super(RegionExtractor, self).__init__(
             maps_img=maps_img, mask_img=mask_img,
             smoothing_fwhm=smoothing_fwhm,
@@ -392,7 +381,7 @@ class RegionExtractor(NiftiMapsMasker):
                        "either of these {0}").format(list_of_strategies)
             raise ValueError(message)
 
-        if self.threshold is None or isinstance(self.threshold, _basestring):
+        if self.threshold is None or isinstance(self.threshold, str):
             raise ValueError("The given input to threshold is not valid. "
                              "Please submit a valid number specific to either of "
                              "the strategy in {0}".format(list_of_strategies))
@@ -403,7 +392,7 @@ class RegionExtractor(NiftiMapsMasker):
             else:
                 if self.thresholding_strategy == 'percentile':
                     self.threshold = "{0}%".format(self.threshold)
-                threshold_maps = threshold_img(maps_img, mask_img=self.mask_img,
+                threshold_maps = threshold_img(maps_img, mask_img=self.mask_img, copy=True,
                                                threshold=self.threshold)
 
         # connected component extraction
@@ -421,46 +410,47 @@ class RegionExtractor(NiftiMapsMasker):
 
 def connected_label_regions(labels_img, min_size=None, connect_diag=True,
                             labels=None):
-    """ Extract connected regions from a brain atlas image defined by labels
+    """Extract connected regions from a brain atlas image defined by labels
     (integers).
 
-    For each label in an parcellations, separates out connected
+    For each label in a :term:`parcellation`, separates out connected
     components and assigns to each separated region a unique label.
 
     Parameters
     ----------
-
     labels_img : Nifti-like image
         A 3D image which contains regions denoted as labels. Each region
         is assigned with integers.
 
-    min_size : float, in mm^3 optional (default None)
-        Minimum region size in volume required to keep after extraction.
+    min_size : :obj:`float`, optional
+        Minimum region size (in mm^3) in volume required to keep after extraction.
         Removes small or spurious regions.
 
-    connect_diag : bool (default True)
+    connect_diag : :obj:`bool`, optional
         If 'connect_diag' is True, two voxels are considered in the same region
         if they are connected along the diagonal (26-connectivity). If it is
         False, two voxels are considered connected only if they are within the
-        same x, y, or z direction.
+        same x, y, or z direction. Default=True.
 
-    labels : 1D numpy array or list of str, (default None), optional
+    labels : 1D :class:`numpy.ndarray` or :obj:`list` of :obj:`str`, optional
         Each string in a list or array denote the name of the brain atlas
         regions given in labels_img input. If provided, same names will be
         re-assigned corresponding to each connected component based extraction
         of regions relabelling. The total number of names should match with the
         number of labels assigned in the image.
 
-        NOTE: The order of the names given in labels should be appropriately
-        matched with the unique labels (integers) assigned to each region
-        given in labels_img (also excluding 'Background' label).
+    Notes
+    -----
+    The order of the names given in labels should be appropriately matched with
+    the unique labels (integers) assigned to each region given in labels_img
+    (also excluding 'Background' label).
 
     Returns
     -------
-    new_labels_img : Nifti-like image
+    new_labels_img : :class:`nibabel.nifti1.Nifti1Image`
         A new image comprising of regions extracted on an input labels_img.
 
-    new_labels : list, optional
+    new_labels : :obj:`list`, optional
         If labels are provided, new labels assigned to region extracted will
         be returned. Otherwise, only new labels image will be returned.
 
@@ -501,8 +491,8 @@ def connected_label_regions(labels_img, min_size=None, connect_diag=True,
         unique_labels.remove(0)
 
     if labels is not None:
-        if (not isinstance(labels, collections.Iterable) or
-                isinstance(labels, _basestring)):
+        if (not isinstance(labels, collections.abc.Iterable) or
+                isinstance(labels, str)):
             labels = [labels, ]
         if len(unique_labels) != len(labels):
             raise ValueError("The number of labels: {0} provided as input "
